/* MORGAN: Copyright 1991-2005: Univ. of Washington. All rights reserved. */
/* You may copy, use, modify, but NOT redistribute, modified or otherwise.*/
/* See: http://www.stat.washington.edu/thompson/Genepi/license.shtml      */
#include<stdio.h>
#include<stdlib.h>
#include<search.h>
int   compareval (const void *a, const void *b);

double normcdf (double value)           /*Calculate the p-value of an observation using normal distribution */
{
   int   i;
   int   size = 2000;
   double *cdf_array = NULL;
   double rank, pvalue = 0;

   cdf_array = (double *) calloc (size, sizeof (double));
   alloc_chk (WHERE, cdf_array);
   for (i = 0; i < size; i++)
   {
      cdf_array[i] = normal (0, 1);
   }

   qsort ((void *) cdf_array, size, sizeof (double), compareval);
   /*for(i=0;i < 1000; i++)
      printf("array[%d] = %lf\n",i,cdf_array[i]);
    */
   rank = 1.0;
   for (i = 0; i < size; i++)
   {

      if (cdf_array[i] <= value)
      {
         rank = rank + 1.0;
      }
      else
      {
         break;
      }
   }
   free (cdf_array);
   rank = (double) (size + 1) - rank;
   pvalue = (double) (rank / (size + 1));
/* printf("pvalue = %lf\n",pvalue);*/
   return pvalue;
}

int compareval (const void *a, const void *b)
{
   double elem1 = *(double *) a;
   double elem2 = *(double *) b;
   if (elem1 > elem2)
   {
      return 1;
   }
   else if (elem1 == elem2)
   {
      return 0;
   }
   else
   {
      return -1;
   }
}

double cdfLR (double Value, double p0, double p1) /*Calculate the p-value of likelihood-ratio statistic under (lambda,p) model */
{
   int   i;
   int   size = 2000;
   double *cdf_array = NULL;
   double rank, pvalue = 0;

   cdf_array = (double *) calloc (size, sizeof (double));
   alloc_chk (WHERE, cdf_array);
   for (i = 0; i < size; i++)
   {
      double ran_num = rans ();
      if (ran_num <= p0)
      {
         cdf_array[i] = 1;
      }
      else if ((ran_num > p0) && (ran_num <= (p0 + p1)))
      {
         double ran_num1 = rans ();
         cdf_array[i] = -2.0 * log (1 - ran_num1);
      }
      else
      {
         double ran_num2 = normal (0, 1);
         cdf_array[i] = (double) (ran_num2 * ran_num2);
      }
   }

   qsort ((void *) cdf_array, size, sizeof (double), compareval);
   /*for(i=0;i < 1000; i++)
      printf("array[%d] = %lf\n",i,cdf_array[i]);
    */
   rank = 1.0;
   for (i = 0; i < size; i++)
   {

      if (cdf_array[i] <= Value)
      {
         rank = rank + 1.0;
      }
      else
      {
         break;
      }
   }
   free (cdf_array);
   rank = (double) (size + 1) - rank;
   pvalue = (double) (rank / (size + 1));
/* printf("pvalue = %lf\n",pvalue);*/
   return pvalue;
}

double cdfdelta (double Value)          /*calculate p-value of the likelihood-ratio statistic under (delta) model */
{

   int   i;
   int   size = 2000;
   double *cdf_array = NULL;
   double rank, pvalue = 0;

   cdf_array = (double *) calloc (size, sizeof (double));
   alloc_chk (WHERE, cdf_array);
   for (i = 0; i < size; i++)
   {
      double ran_num2 = normal (0, 1);
      cdf_array[i] = (double) (ran_num2 * ran_num2);
   }
   qsort ((void *) cdf_array, size, sizeof (double), compareval);
   /*for(i=0;i < 1000; i++)
      printf("array[%d] = %lf\n",i,cdf_array[i]);
    */
   rank = 1.0;
   for (i = 0; i < size; i++)
   {

      if (cdf_array[i] <= Value)
      {
         rank = rank + 1.0;
      }
      else
      {
         break;
      }
   }
   free (cdf_array);
   rank = (double) (size + 1) - rank;
   pvalue = (double) (rank / (size + 1));
/* printf("pvalue = %lf\n",pvalue);*/
   return pvalue;
}
