/* MORGAN: Copyright 1991-2005: Univ. of Washington. All rights reserved. */
/* You may copy, use, modify, but NOT redistribute, modified or otherwise.*/
/* See: http://www.stat.washington.edu/thompson/Genepi/license.shtml      */
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "readconfigfile.h"

/********************************************************
| define Global Parameters name                        **
********************************************************/

#define NUMBER_OF_LOCUS             "NUMBER_OF_LOCUS"
#define NUMBER_OF_ITERATION         "NUMBER_OF_PERMUTATION"
#define NUMBER_OF_GROUP             "NUMBER_OF_GROUP"
#define STATISTICS_INFO             "STATISTICS_INFO"
#define NUMBER_OF_AFFECTED          "NUMBER_OF_AFFECTED"
#define TOTAL_SIZE                  "TOTAL_SIZE"
#define NUMBER_OF_SEGL              "NUMBER_OF_SEGL"
#define NUMBER_OF_SEGP              "NUMBER_OF_SEGP"
#define AUTO_CHKPNTS                "AUTO_CHECK_POINTS"
#define INTERVAL                    "INTERVAL"
#define OUTPUT_FILE                 "OUTPUT_FILE"
#define TEMP_FILE_DIR               "TEMP_FILE_DIR"
#define PERM_FILE_DIR               "PERM_FILE_DIR"
#define STAT_SPAIR                  "spairs"
#define STAT_LIKELIHOOD             "likelihood"
#define STAT_ROBDOM                 "Srobdom"
#define STAT_NEWSTAT                "saffunaff"
#define STAT_LAMBDA                 "slambda"
#define STAT_DELTA                  "delta"
#define PERMUTATION_TEST            "Permutation"
#define SIMULATION_TEST             "Simulation"
#define NORMAL_TEST                 "Normal"
#define CHI_SQUARE_TEST             "chi-Square"
#define STAT_METHODS                "STAT_TESTS"
#define STAT_TYPE                   "STAT_TYPE"

#define STAT_ANDERSON_STR           "anderson"
#define STAT_MEAN_STR               "mean"
#define STAT_QUANTILE_STR           "quantile"

/*********************************************
| define constants                          **
**********************************************/

#define NAME_SEPERATOR '='
#define COMMENT         '#'
#define MAX_LINE_LENGTH 256
#define STAT_SEPERATOR   ','

/********************************************************
| define Global variable here                          **
********************************************************/
/*
   int NO_LOCUS ;
   int NO_ITER ;
   int NO_GROUP ;
   int NO_STAT ;
   int NO_AFF ;
   int TOT_SIZE ;
   int NO_SEGL ;
   int NO_SEGP ;
   int AUTO_CKPNTS ;
   int BREAK ;
 */

/*
   int nNumOfLocus ;
   int nNumOfIter ;
   int nNumOfGroup ;
   int nNumOfStat ;
   int nNumOfAff ;
   int nTotalSize ;
   int nNumOfSegl ;
   int nNumOfSegp ;
   int nAutoChkPts ;
   int nInterval ;
   int bPermutationEnable = 0;
   int bResimuEnable = 0;
   int bNormalEnable = 0;
   int nChiSquareEnable = 0 ;
 */
/********************************************
* Process ths Tests Information ********
*********************************************/
int ProcessTestInfo (char *testInfoPtr)
{
   char *tempPtr = NULL;
   int   i;
   int   pos = 0;
   int   nParamFound = 0;
   int   nNumOfTest = 0;
   if (testInfoPtr == NULL)
   {
      return 0;
   }
   tempPtr = strchr (testInfoPtr, STAT_SEPERATOR);
   if (tempPtr == NULL)
   {
      printf ("ERROR: No Tests Specified in config file\n");
      return 0;
   }
   nNumOfTest = atoi (testInfoPtr);
   pos = tempPtr - testInfoPtr;
   if ((nNumOfTest < 1) || (nNumOfTest > 4))
   {
      printf ("ERROR: Invalid number of Tests specified in config file\n");
      return 0;
   }
   for (i = 0; i < nNumOfTest; i++)
   {
      char *tPtr = NULL;
      tPtr = strchr (testInfoPtr + pos + 1, STAT_SEPERATOR);
      if (tPtr != NULL)
      {
         char  buffer[50] = { 0 };
         int   statLen = 0;
         statLen = tPtr - (testInfoPtr + pos + 1);
         strncpy (buffer, testInfoPtr + pos + 1, statLen);

         if (strncasecmp (buffer, PERMUTATION_TEST, statLen) == 0)
         {
            bPermutationEnable = 1;
            nParamFound++;
         }
         else if (strncasecmp (buffer, SIMULATION_TEST, statLen) == 0)
         {
            bResimuEnable = 1;
            nParamFound++;
         }
         else if (strncasecmp (buffer, NORMAL_TEST, statLen) == 0)
         {
            bNormalEnable = 1;
            nParamFound++;
         }
         else if (strncasecmp (buffer, CHI_SQUARE_TEST, statLen) == 0)
         {
            nChiSquareEnable = 1;
            nParamFound++;
         }
         pos = pos + statLen + 1;
      }
   }
   if (nParamFound != nNumOfTest)
   {
      printf
         ("ERROR: 'STAT_TESTS' parameter information is incorrect , Mentioned(%d), Given(%d)\n",
          nNumOfTest, nParamFound);
      return 0;
   }
   return 1;
}

/********************************************
* Process ths statistics Information ********
*********************************************/
int ProcessStatisticsInfo (char *statInfoPtr)
{
   char *tempPtr = NULL;
   int   i;
   int   pos = 0;
   int   nParamFound = 0;
   if (statInfoPtr == NULL)
   {
      return 0;
   }
   tempPtr = strchr (statInfoPtr, STAT_SEPERATOR);
   if (tempPtr == NULL)
   {
      printf ("ERROR: No STATISTICS_INFO parameter Specified in config file\n");
      return 0;
   }
   nNumOfStat = atoi (statInfoPtr);
   pos = tempPtr - statInfoPtr;
   if ((nNumOfStat < 1) || (nNumOfStat > 5))
   {
      printf
         ("ERROR: Invalid number of statistic info specified in config file\n");
      return 0;
   }
   for (i = 0; i < nNumOfStat; i++)
   {
      char *tPtr = NULL;
      tPtr = strchr (statInfoPtr + pos + 1, STAT_SEPERATOR);
      if (tPtr != NULL)
      {
         char  buffer[50] = { 0 };
         int   statLen = 0;
         statLen = tPtr - (statInfoPtr + pos + 1);
         strncpy (buffer, statInfoPtr + pos + 1, statLen);
         if (strncasecmp (buffer, STAT_SPAIR, statLen) == 0)
         {
            bSpairsOn = 1;
            nParamFound++;
         }
         else if (strncasecmp (buffer, STAT_LIKELIHOOD, statLen) == 0)
         {
            bLikelihoodOn = 1;
            nParamFound++;
         }
         else if (strncasecmp (buffer, STAT_ROBDOM, statLen) == 0)
         {
            bRobdomOn = 1;
            nParamFound++;
         }
         else if (strncasecmp (buffer, STAT_NEWSTAT, statLen) == 0)
         {
            bNewStatOn = 1;
            nParamFound++;
         }
         else if (strncasecmp (buffer, STAT_DELTA, statLen) == 0)
         {
            bDeltaOn = 1;
            nParamFound++;
         }
         else if (strncasecmp (buffer, STAT_LAMBDA, statLen) == 0)
         {
            blambdaOn = 1;
            nParamFound++;
         }
         pos = pos + statLen + 1;
      }
   }
   if (nParamFound != nNumOfStat)
   {
      printf
         ("ERROR: 'STATISTICS_INFO' parameter information is incorrect , Mentioned(%d), Given(%d)\n",
          nNumOfStat, nParamFound);
      return 0;
   }
   return 1;
}

int ProcessStatType (char *statInfoPtr)
{
   char *tempPtr = NULL;
   int   length = 0;
   if (statInfoPtr == NULL)
   {
      printf ("ERROR: No STAT_TYPE parameter Specified in config file\n");
      return 0;
   }
   length = strlen (statInfoPtr);
   if (length == 0)
   {
      printf ("ERROR: 'STAT_TYPE' parameter is invalid  in config file\n");
      return 0;
   }
   tempPtr = strchr (statInfoPtr, STAT_SEPERATOR);
   if (tempPtr == NULL)
   {
      char  buffer[100] = { 0 };
      strncpy (buffer, statInfoPtr, length - 1);
      /* This is not a quantile */
      if (strcasecmp (buffer, STAT_MEAN_STR) == 0)
      {
         nStatType = STAT_MEAN;
      }
      /* else if(strcasecmp(buffer,STAT_ANDERSON_STR)==0)
         {
         nStatType = STAT_ANDERSON ;
         } */
      else
      {
         printf
            ("ERROR: 'STAT_TYPE' Invalid statistics type specified in config file(%s)\n",
             buffer);
         return 0;
      }
   }
   else
   {
      char  buffer[30] = { 0 };
      int   statLen = tempPtr - statInfoPtr;
      strncpy (buffer, statInfoPtr, statLen);
      if (strncasecmp (statInfoPtr, STAT_QUANTILE_STR, statLen) == 0)
      {
         int   nNoOfQuant = 0;
         int   i, pos;

         nStatType = STAT_QUANTILE;
         nNoOfQuant = atoi (tempPtr + 1);
         if (nNoOfQuant > 0)
         {
            pQuantileList = (double *) calloc (nNoOfQuant + 1, sizeof (double));
            alloc_chk (WHERE, pQuantileList);

         }
         pos = 1;
         for (i = 0; i < (nNoOfQuant + 1); i++)
         {
            char *tmpPtr = NULL;
            tmpPtr = strchr (tempPtr + pos, STAT_SEPERATOR);
            if (tmpPtr != NULL)
            {
               pQuantileList[i] = atof (tempPtr + pos);
               pos = tmpPtr - tempPtr + 1;
            }
            if (pQuantileList[i] <= 0)
            {
               printf
                  ("ERROR: Something wrong in the quantile info in the config file\n");
               free (pQuantileList);
               pQuantileList = NULL;
               return 0;
            }
         }
      }
      else
      {
         printf
            ("ERROR: 'STAT_TYPE' Invalid statistics type specified in config file(%s)\n",
             statInfoPtr);
         return 0;
      }
   }
   return 1;
}

int ValidateMethods (int app_type)
{
   if (bPermutationEnable * bResimuEnable)
   {
      printf
         ("ERROR: Could not be able to process Simulation Test and Permutation Test together\n");
      return 0;
   }
   if (app_type == 0)
   {
      if (nChiSquareEnable)
      {
         printf
            ("ERROR: Invalid stat test combination specified in config file\n");
         return 0;
      }
   }
   else
   {
      if (bNormalEnable)
      {
         printf
            ("ERROR: Invalid stat test combination specified in config file\n");
         return 0;
      }
   }
   return 1;
}

/************************************************************
| readConfigFile() : read the config parameters from file  **
************************************************************/

int readConfigFile (char *fileName)
{
   FILE *fp;
   int   bTestsMentioned = 0;
   int   bStatsMentioned = 0;
   char  lineBuffer[MAX_LINE_LENGTH];
   if (fileName == NULL)
   {
      printf ("ERROR: Invalid config file(NULL)\n");
      return 0;
   }

   fp = open_chkq (WHERE, fileName, "r");

   /* read line by line */
   while (fgets (lineBuffer, MAX_LINE_LENGTH, fp) != NULL)
   {
      int   i = 0;
      int   k = 0;
      int   nParamLength = 0;
      char *pdest = NULL;
      while (lineBuffer[i] == ' ')
      {
         i++;
         continue;
      }
      if (lineBuffer[i] == COMMENT)     /* this is a comment so proceed further */
      {
         continue;
      }
      /* search for the parameter name */
      pdest = strchr (lineBuffer + i, NAME_SEPERATOR);
      if (pdest == NULL)
      {
         continue;
      }

      /* trim the spaces if any */
      nParamLength = pdest - (lineBuffer + i);
      if (nParamLength == 0)
         continue;
      while (lineBuffer[i + nParamLength - k - 1] == ' ')
      {
         k++;
         continue;
      }
      nParamLength = nParamLength - k;

      /* compare and check the parameter name */

      if (strncasecmp (lineBuffer + i, NUMBER_OF_LOCUS, nParamLength) == 0)
      {
         nNumOfLocus = atoi (pdest + 1);
      }
      else if (strncasecmp (lineBuffer + i, NUMBER_OF_ITERATION, nParamLength)
               == 0)
      {
         nNumOfIter = atoi (pdest + 1);
      }
      else if (strncasecmp (lineBuffer + i, NUMBER_OF_GROUP, nParamLength) == 0)
      {
         nNumOfGroup = atoi (pdest + 1);
      }
      else if (strncasecmp (lineBuffer + i, STATISTICS_INFO, nParamLength) == 0)
      {
         int   result = 1;
         result = ProcessStatisticsInfo (pdest + 1);
         if (result == 0)
         {
            printf ("ERROR: Invalid statistics info parameter specified\n");
            fclose (fp);
            return 0;
         }
         bStatsMentioned = 1;
      }
      else if (strncasecmp (lineBuffer + i, STAT_METHODS, nParamLength) == 0)
      {

         int   result = 1;
         result = ProcessTestInfo (pdest + 1);
         if (result == 0)
         {
            printf ("ERROR: Invalid Tests specified in config file\n");
            fclose (fp);
            return 0;
         }
         bTestsMentioned = 1;

      }
      else if (strncasecmp (lineBuffer + i, STAT_TYPE, nParamLength) == 0)
      {
         int   result = 0;
         result = ProcessStatType (pdest + 1);
         if (result == 0)
         {
            printf ("ERROR: Invalid Stat Type specified in config file\n");
            fclose (fp);
            return 0;
         }

      }
      else if (strncasecmp (lineBuffer + i, NUMBER_OF_AFFECTED, nParamLength) ==
               0)
      {
         nNumOfAff = atoi (pdest + 1);
      }
      else if (strncasecmp (lineBuffer + i, TOTAL_SIZE, nParamLength) == 0)
      {
         nTotalSize = atoi (pdest + 1);
      }
      else if (strncasecmp (lineBuffer + i, NUMBER_OF_SEGL, nParamLength) == 0)
      {
         nNumOfSegl = atoi (pdest + 1);
      }
      else if (strncasecmp (lineBuffer + i, NUMBER_OF_SEGP, nParamLength) == 0)
      {
         nNumOfSegp = atoi (pdest + 1);
      }
      else if (strncasecmp (lineBuffer + i, AUTO_CHKPNTS, nParamLength) == 0)
      {
         nAutoChkPts = atoi (pdest + 1);
      }
      else if (strncasecmp (lineBuffer + i, INTERVAL, nParamLength) == 0)
      {
         nInterval = atoi (pdest + 1);
      }
      else if (strncasecmp (lineBuffer + i, OUTPUT_FILE, nParamLength) == 0)
      {
         /* printf ("Line buffer (%s)\n", pdest + 1); */
         strncpy (pStrOutputFile, pdest + 1, strlen (pdest + 1) - 1);
      }
      else if (strncasecmp (lineBuffer + i, TEMP_FILE_DIR, nParamLength) == 0)
      {
         /* printf ("Line buffer (%s)\n", pdest + 1); */
         strncpy (pStrTempDir, pdest + 1, strlen (pdest + 1) - 1);
      }
      else if (strncasecmp (lineBuffer + i, PERM_FILE_DIR, nParamLength) == 0)
      {
         /* printf ("Line buffer (%s)\n", pdest + 1); */
         strncpy (pStrPermDir, pdest + 1, strlen (pdest + 1) - 1);
      }
      else
      {
         continue;
      }
   }
   fclose (fp);
   if ((bTestsMentioned * bStatsMentioned) == 0)
   {
      printf
         ("ERROR: Either No tests or Stats Mentioned in the parameter file\n");
      return 0;
   }
   return 1;
   /* return ValidateMethods (app_type); */

}

void printConfigParams ()
{
   printf
      ("L(%d), I(%d), G(%d), S(%d), A(%d), T(%d), SG(%d), SP(%d), AC(%d), IN(%d) Out(%s) dir(%s) perm(%s)\n",
       nNumOfLocus, nNumOfIter, nNumOfGroup, nNumOfStat, nNumOfAff, nTotalSize,
       nNumOfSegl, nNumOfSegp, nAutoChkPts, nInterval, pStrOutputFile,
       pStrTempDir, pStrPermDir);
   printf ("statcount(%d),SP(%d)Lkh(%d)RD(%d)NS(%d)DL(%d)LAM(%d)\n", nNumOfStat,
           bSpairsOn, bLikelihoodOn, bRobdomOn, bNewStatOn, bDeltaOn,
           blambdaOn);
   printf ("stat_type=(%d)\n", nStatType);
}
