/* MORGAN: Copyright 1991-2005: Univ. of Washington. All rights reserved. */
/* You may copy, use, modify, but NOT redistribute, modified or otherwise.*/
/* See: http://www.stat.washington.edu/thompson/Genepi/license.shtml      */
/*::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::*/
/*         errspars.h:  error messages produced by Pars/ functions            */
/*::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::*/

#ifndef ERRSPARS_H
#define ERRSPARS_H

/**
 * @addtogroup parse
 * @{
 */

#define E 'E'
#define W 'W'
#define F 'F'
#define Q 'Q'

static int      n_warn = 0;
static int      n_err = 0;

/**
 * Parse error codes. error codes which result in the error messages and error
 * operations defined in proc_errors below.
 */
enum parsEnum {
      ERR_NUL,   /**< Null Error. */
      ERR_FNU,   /**< File type from command line not recognized. */
      ERR_PCL,   /**< File type from command line not used by this program. */
      ERR_FCL,   /**< Incorrect specification of files in command line. */
      ERF_EPF,
      ERR_EIF,
      ERR_OPF,
      ERR_CLF,
      ERR_SIF,
      ERR_LTL,
      ERR_KEY,
      ERR_DAT,
      ERR_HEX,
      ERR_STR,
      ERR_NML,
      ERR_BGM,
      ERR_PGP,
      ERR_BNN,
      ERR_AIL,
      ERR_XTR,
      ERR_NOT,
      ERR_RED,
      ERR_PED,
      ERR_PEX,
      ERR_ALU,
      ERR_ALP,
      ERR_ALC,
      ERR_ALO,
      ERR_PPI,
      ERR_PPR,
      ERR_PF1,
      ERR_TRI,
      ERR_TRK,
      ERR_WTI,
      ERR_WT2,
      ERR_PF2,
      ERR_PN3,
      ERR_GND,
      ERR_OPA,
      ERR_ASG,
      ERR_IGG,
      ERR_ORD,
      ERR_STT,
      ERR_SCN,
      ERR_MNW,
      ERR_MXW,
      ERR_SSU,
      ERR_NSI,
      ERR_SIP,
      ERR_SCI,
      ERR_SBI,
      ERR_SPI,
      ERR_SNI,
      ERR_TRO,
      ERR_TRR,
      ERR_FFO,
      ERR_FFD,
      ERR_MNR,
      ERR_FFR,
      ERR_LVM,
      ERR_VAM,
      ERR_COM,
      ERR_CFM,
      ERR_CFO,
      ERD_CHO,
      ERD_MK0,
      ERD_MKM,
      ERD_TR0,
      ERD_TRM,
      ERD_RSM,
      ERD_RMP,
      ERA_RLW,
      ERK_KNN,
      ERK_RCP,
      ERK_DNM,
      ERK_NRF,
      ERK_XRF,
      ERA_PPW,
      ERA_PPN,
      ERD_RCS,
      ERD_UTM,
      ERD_MDM,
      ERD_MRM,
      ERD_MPM,
      ERD_O1M, 
      ERD_TMM,
      ERD_MDO,
      ERD_MFO,
      ERD_MPO,
      ERD_MPD,
      ERD_TDO,
      ERD_TFO,
      ERA_TIO,
      ERA_TFO,
      ERA_TDO,
      ERA_TID,
      ERA_TFD,
      ERA_TDD,
      ERA_TIN,
      ERA_OXP,
      ERA_OXC,
      ERA_WDX,
      ERA_OAP,
      ERA_OAC,
      ERA_OIP,
      ERA_OIC,
      ERA_TPW,
      ERD_MKO,
      ERD_TRO,
      ERD_LCO,
      ERR_TRP,
      ERD_MLO,
      ERD_TLO,
      ERD_AFO,
      ERD_SMA,
      ERD_MAF,
      ERD_TAF,
      ERD_TGM,
      ERD_TGR,
      ERD_AVR,
      ERD_RVR,
      ERD_VMN,
      ERA_BLS,
      ERA_MDL,
      ERA_BMD,
      ERA_RMD,
      ERA_RMN,
      ERA_MNO,
      ERA_PAL,
      ERA_UAL,
      ERA_DIN,
      ERA_DMN,
      ERA_CMP,
      ERA_SCS,
      ERA_WPO,
      ERA_OSC,
      ERA_OAI,
      ERA_OSP,
      ERA_PGL,
      ERA_PGX,
      ERA_BGM,
      ERA_DPG,
      ERA_DAI,
      ERR_EIM,
      ERF_PE0,
      ERF_PE1,
      ERF_PE2,
      ERF_IPR,
      ERF_NIN,
      ERF_NRL,
      ERR_STN,
      ERR_SFN,
      ERR_TRF,
      ERR_FFF,
      ERR_SEL,
      ERR_STV,
      ERR_VA0,
      ERR_VR0,
      ERR_VSA,
      ERR_VSR,
      ERR_VAN,
      ERR_VRN,
      ERR_CSA,
      ERR_CSR,
      ERR_CFA,
      ERR_CFR,
      ERR_CAF,
      ERR_CRF,
      ERR_CAZ,
      ERR_CRZ,
      ERR_PDA,
      ERR_PDR,
      ERR_ET1,
      ERR_ET2,
      ERR_PT1,
      ERK_KIN,
      ERK_DRF,
      ERD_SIM,
      ERD_CHR,
      ERD_NFM,
      ERD_NMA,
      ERD_NFA,
      ERD_MMM,
      ERA_1MA,
      ERA_MN1,
      ERD_NME,
      ERD_NAM,
      ERA_MNL,
      ERD_MTM,
      ERD_TFM,
      ERD_TMA,
      ERD_TFA,
      ERA_MBM,
      ERA_MXM,
      ERA_DAP,
      ERA_DBE,
      ERA_IMI,
      ERA_XEI,
      ERA_MIO,
      ERA_WIP,
      ERA_MIP,
      ERD_NAT,
      ERD_NMT,
      ERD_NFT,
      ERA_NAT,
      ERD_MMX,
      ERD_TID,
      ERD_MID,
      ERD_LLL,
      ERD_LUL,
      ERD_NML,
      ERD_MAN,
      ERD_TAV,
      ERD_TRV,
      ERD_NAR,
      ERD_SVR,
      ERD_NTL,
      ERD_TAN,
      ERD_NGM,
      ERD_MTD,
      ERD_SEE,
      ERD_OTS,
      ERD_OMS,
      ERA_OSS, 
      ERA_OPS, 
      ERD_SSD,
      ERD_STD,
      ERA_SSP,
      ERA_SSS,
      ERF_OPF,
      ERD_OST,
      ERD_OSM,
      ERA_USS,
      ERA_USP,
      ERA_SL1,
      ERA_SLU,
      ERA_SEL,
      ERA_OOS,
      ERA_OOT,
      ERA_OIT,
      ERA_MOT,
      ERA_MUL,
      ERA_MIT,
      ERA_NMD,
      ERA_BND,
      ERA_BNM,
      ERA_BMA,
      ERA_WMA,
      ERA_MPG,
      ERA_PIG,
      ERA_MLW,
      ERA_MSS,
      ERA_BPC,
      ERA_NAF,
      ERA_BAF,
      ERA_NPP,
      ERA_ZUT,
      ERA_LWO,
      ERA_NPV,
      ERA_RPV,
      ERA_OPV,
      ERA_MPV,
      ERA_PNI,
      ERA_NIT,
      ERF_OSF,
      ERA_IPT,
      ERA_BIR,
      ERA_BI2,
      ERA_BRR,
      ERA_NEI,
      ERA_NER,
      ERA_NEJ,
      ERA_NEK,
      ERA_DRP, 
      ERA_TEI,
      ERA_TE2,
      ERA_TER,
      ERG_NDN,
      ERG_TMC,
      ERG_TMI,
      ERR_FFL,
      ERR_FFN,
      ERR_CHG,
      ERR_EIP,
      ERR_OEF,
      ERR_EIE,
      ERR_OBS,
      ERR_MAX,
      ERR_FAT,
      ERR_QIT };

static int err_typ = 0;

static char errnumb[80];
static char * err_txt;             /*ptr to text for txtflg below       */

/* error message table                                                  */

/* error level is   'F' causes immediate termination of program         */
/*                  'E' causes termination of program after all input   */
/*                      statements are processed                        */
/*                  'W' warning statement only                          */

struct _errors {
    char * msg;                         /*text for error                */
    char   level;                       /*error level                   */
    int    tokflg;                      /*whether tokens are to be prntd*/
    int    txtflg;                      /*whether an arg is to be prntd */
    int    statflg;                     /*whether correct form printed  */
               } *perr;

static struct _errors proc_errors[] = {
{"***"                                                          ,F,N,N,N}, /*ERR_NUL*/
{" File type \"%s\" from command line not recognized"           ,F,N,Y,N}, /*ERR_FNU*/
{" File type \"%s\" from command line not used by this program" ,W,N,Y,N}, /*ERR_PCL*/
{" Incorrect specification of files in command line.  Correct form is:\n\n"
 "   %s"                                                        ,F,N,Y,N}, /*ERR_FCL*/
{" No pedigree file specified in command line or \"input pedigree file\" statement"
                                                                ,E,N,N,N}, /*ERF_EPF*/
{" No parameter file specified in command line"                 ,F,N,N,N}, /*ERR_EIF*/
{" Can't open %s"                                               ,E,N,Y,N}, /*ERR_OPF*/
{" Error when closing %s"                                       ,W,N,Y,N}, /*ERR_CLF*/
{" No parameter statements in file \"%s\""                      ,W,N,Y,N}, /*ERR_SIF*/
{" %s\n\n"
 "    Line is too long;  the maximum number of characters is " PMAXLINLEN
                                                                ,F,N,Y,N}, /*ERR_LTL*/
{" Unrecognizable word or word order"                           ,E,Y,N,N}, /*ERR_KEY*/
{" Missing data in parameter statement"                         ,E,Y,N,Y}, /*ERR_DAT*/
{" Format error in hexadecimal data value"                      ,E,Y,N,Y}, /*ERR_HEX*/
{" Expected string in statement"                                ,E,Y,N,Y}, /*ERR_STR*/
{" Names of individuals may have a maximum of %s characters:\n"
 "         name(s) truncated"                                   ,W,Y,Y,N}, /*ERR_NML*/
{" \"%s\" not an integer;  expected pairs of names and integers"
                                                                ,E,Y,Y,Y}, /*ERR_BGM*/
{" Expected pairs of names and integers"                        ,E,Y,N,Y}, /*ERR_PGP*/
{" Expected a positive integer, not \"%s\""                     ,E,Y,Y,Y}, /*ERR_BNN*/
{" Wrong number of data;  expected sets of names and 2 x %s marker(s)"
                                                                ,E,Y,Y,Y}, /*ERR_AIL*/
{" Too many parameters in input statement"                      ,E,Y,N,Y}, /*ERR_XTR*/
{" This statement is not implemented for program \"%s\""        ,W,Y,Y,N}, /*ERR_NOT*/
{" Overrides previous statement of same type"                   ,W,Y,N,N}, /*ERR_RED*/
{" Pedigree size must be greater than 2"                        ,E,Y,N,N}, /*ERR_PED*/
{" Pedigree size is greater than default maximum size of %s"    ,W,Y,Y,N}, /*ERR_PEX*/
{" Overrides default maximum pedigree size of %s\n"
 "    Program \"unibig\" is recommended instead for large pedigrees"
                                                                ,W,Y,Y,N}, /*ERR_ALU*/
{" Overrides default maximum pedigree size of %s"               ,W,Y,Y,N}, /*ERR_ALP*/
{" Overrides default maximum component size of %s"              ,W,Y,Y,N}, /*ERR_ALC*/
{" Overrides default maximum number of observed individuals of %s"
                                                                ,W,Y,Y,N}, /*ERR_ALO*/
{" Preferred form of this statement is:\n"
 "        \"input pedigree record names  3  integers  I\""      ,W,Y,N,N}, /*ERR_PPI*/
{" Preferred form of this statement is:\n"
 "        \"input pedigree record names  3  integers  I  reals  J\""
                                                                ,W,Y,N,N}, /*ERR_PPR*/
{" Number of integers out of range"                             ,E,Y,N,N}, /*ERR_PF1*/
{" The position in the record for the integer trait data is out of range"
                                                                ,E,Y,N,N}, /*ERR_TRI*/
{" The position in the record for real (quantitative) trait data\n"
 "         is out of range"                                     ,E,Y,N,N}, /*ERR_TRK*/
{" There must be the same number of traits and corresponding record positions"
                                                                ,E,Y,N,N}, /*ERR_WTI*/
{" There must be the same number of traits and corresponding pairs of record positions"
                                                                ,E,Y,N,N}, /*ERR_WT2*/
{" Number of reals is out of range"                             ,E,Y,N,N}, /*ERR_PF2*/
{" Number of names must be 3"                                   ,E,Y,N,N}, /*ERR_PN3*/
{" Overrides previous gender present/absent statement"          ,W,Y,N,N}, /*ERR_GND*/
{" Overrides previous observed present/absent statement"        ,W,Y,N,N}, /*ERR_OPA*/
{" Overrides previous ignore gender statement"                  ,W,Y,N,N}, /*ERR_ASG*/
{" Overrides previous assign gender statement"                  ,W,Y,N,N}, /*ERR_IGG*/
{" Overrides previous record order statement"                   ,W,Y,N,N}, /*ERR_ORD*/
{" Overrides previous set trait data statement"                 ,W,Y,N,N}, /*ERR_STT*/
{" Overrides previous sample by scan/step statement"            ,W,Y,N,N}, /*ERR_SCN*/
{" Locus window size out of range;  must be > 1"                ,E,Y,N,N}, /*ERR_MNW*/
{" Locus window size out of range;  must be <= number of loci"  ,E,Y,N,N}, /*ERR_MXW*/
{" Overrides previous sampler setup statement"                  ,W,Y,N,N}, /*ERR_SSU*/
{" Number of sequential imputation samples for setup is out of range"
                                                                ,E,Y,N,N}, /*ERR_NSI*/
{" Frequency of sequential proposals is out of range"           ,E,Y,N,N}, /*ERR_SIP*/
{" Frequency of scores computation is out of range"             ,E,Y,N,N}, /*ERR_SCI*/
{" Number of burn-in iterations is out of range"                ,E,Y,N,N}, /*ERR_SBI*/
{" Number of preliminary iterations is out of range"            ,E,Y,N,N}, /*ERR_SPI*/
{" Number of MCMC iterations is out of range"                   ,E,Y,N,N}, /*ERR_SNI*/
{" Trait number is out of range"                                ,E,Y,N,N}, /*ERR_TRO*/
{" Overrides previous statement for the trait"                  ,W,Y,N,N}, /*ERR_TRR*/
{" Fixed effect number is out of range"                         ,E,Y,N,N}, /*ERR_FFO*/
{" Repeated fixed effect number in statement"                   ,W,Y,N,N}, /*ERR_FFD*/
{" Overrides previous mean for the trait"                       ,W,Y,N,N}, /*ERR_MNR*/
{" Overrides previous fixed effect levels for the trait"        ,W,Y,N,N}, /*ERR_FFR*/
{" Too many starting values for fixed effect levels"            ,E,Y,N,N}, /*ERR_LVM*/
{" Too many starting values for variances"                      ,E,Y,N,N}, /*ERR_VAM*/
{" Too many starting values for covariances or correlations"    ,E,Y,N,N}, /*ERR_COM*/
{" Too many fit flags for covariances or correlations"          ,E,Y,N,N}, /*ERR_CFM*/
{" Statement ignored.  %s specified in command line"            ,W,Y,Y,N}, /*ERD_CFO*/
{" Chromosome number is out of range"                           ,E,Y,N,N}, /*ERD_CHO*/
{" At least one marker expected"                                ,E,Y,N,N}, /*ERD_MK0*/
{" Too many markers"                                            ,E,Y,N,N}, /*ERD_MKM*/
{" Bad number of traits"                                        ,E,Y,N,N}, /*ERD_TR0*/
{" Too many traits"                                             ,E,Y,N,N}, /*ERD_TRM*/
{" Overrides previous selection/simulation statement"           ,W,Y,N,N}, /*ERD_RSM*/
{" Overrides previous map statement"                            ,W,Y,N,N}, /*ERD_RMP*/
{" Overrides previous locus window statement"                   ,W,Y,N,N}, /*ERA_RLW*/
{" Wrong number of names:  %s of probands required"             ,E,Y,Y,N}, /*ERK_KNN*/
{" Overrides previous statement for this component"             ,W,Y,N,N}, /*ERK_RCP*/
{" Duplicate name \"%s\" for computation of kinship coefficient",E,Y,Y,N}, /*ERK_DNM*/
{" Recombination frequencies cannot be less than 0"             ,E,Y,N,N}, /*ERK_NRF*/
{" Recombination frequencies must be <= .5"                     ,E,Y,N,N}, /*ERK_XRF*/
{" A small positive value was substituted for pseudo-priors of 0.0"
                                                                ,W,Y,N,N}, /*ERA_PPW*/
{" Pseudo-priors may not be negative"                           ,E,Y,N,N}, /*ERA_PPN*/
{" Overrides previous statement for this chromosome"            ,W,Y,N,N}, /*ERD_RCS*/
{" Too many unlinked traits"                                    ,E,Y,N,N}, /*ERD_UTM*/
{" Too many marker distances"                                   ,E,Y,N,N}, /*ERD_MDM*/
{" Too many marker recombination fractions"                     ,E,Y,N,N}, /*ERD_MRM*/
{" Too many marker positions"                                   ,E,Y,N,N}, /*ERD_MPM*/
{" Since a single marker position is specified, there is no marker map"
                                                                ,W,Y,N,N}, /*ERD_O1M*/
{" There must be the same number of traits, markers\n"
 "        and distances, positions or recombination fractions"  ,E,Y,N,N}, /*ERD_TMM*/
{" Marker distances must be >= 0. and <= 500. cM"               ,E,Y,N,N}, /*ERD_MDO*/
{" Marker recombination fractions must be >= 0. and < .5"       ,E,Y,N,N}, /*ERD_MFO*/
{" Marker positions must be > -500. and <= 500. cM"             ,E,Y,N,N}, /*ERD_MPO*/
{" Marker positions must be in increasing order"                ,E,Y,N,N}, /*ERD_MPD*/
{" Trait distances must be >= 0. and <= 500. cM"                ,E,Y,N,N}, /*ERD_TDO*/
{" Trait recombination fractions must be >= 0. and < .5"        ,E,Y,N,N}, /*ERD_TFO*/
{" Trait interval proportions must be > 0. and < 1."            ,E,Y,N,N}, /*ERA_TIO*/
{" Trait recombination fractions must be > 0. and < .5"         ,E,Y,N,N}, /*ERA_TFO*/
{" Trait distances must be > 0. and <= 500."                    ,E,Y,N,N}, /*ERA_TDO*/
{" Trait interval proportions must be strictly increasing"      ,E,Y,N,N}, /*ERA_TID*/
{" Trait beginning, ending or external recombination fractions\n"
 "        must be strictly increasing"                          ,E,Y,N,N}, /*ERA_TFD*/
{" Trait beginning, ending or external distances must be strictly increasing"
                                                                ,E,Y,N,N}, /*ERA_TDD*/
{" Interval number must be > 0 and < no. markers"               ,E,Y,N,N}, /*ERA_TIN*/
{" Overrides previous statement for beginning and/or ending map positions"
                                                                ,W,Y,N,N}, /*ERA_OXP*/
{" Overrides previous statement for this chromosome for beginning\n"
 "        and/or ending map positions"                          ,W,Y,N,N}, /*ERA_OXC*/
{" The number of beginning or ending map positions for the trait\n"
 "        is not the same for males and females"                ,E,N,N,N}, /*ERA_WDX*/
{" Overrides interval proportions previously specified for all intervals"
                                                                ,W,Y,N,N}, /*ERA_OAP*/
{" Overrides proportions previously specified for this chromosome\n"
 "        for all intervals"                                    ,W,Y,N,N}, /*ERA_OAC*/
{" Overrides proportions previously specified for one or more intervals"
                                                                ,W,Y,N,N}, /*ERA_OIP*/
{" Overrides proportions previously specified for this chromosome\n"
 "        for one or more intervals"                            ,W,Y,N,N}, /*ERA_OIC*/
{" Window width, the number of hypothesized trait positions on either side\n"
 "        of current position, is out of range"                 ,E,Y,N,N}, /*ERA_TPW*/
{" Marker number out of range"                                  ,E,Y,N,N}, /*ERD_MKO*/
{" Trait number out of range"                                   ,E,Y,N,N}, /*ERD_TRO*/
{" Trait locus number out of range"                             ,E,Y,N,N}, /*ERD_LCO*/
{" Trait data type redefined for one or more traits"            ,W,Y,N,N}, /*ERR_TRP*/
{" Too few or too many marker alleles"                          ,E,Y,N,N}, /*ERD_MLO*/
{" Expected two alleles for trait locus"                        ,E,Y,N,N}, /*ERD_TLO*/
{" Allele frequencies must be between 0. and 1."                ,E,Y,N,N}, /*ERD_AFO*/
{" Sum of allele frequencies is not in range " Q_LIMITS         ,W,Y,N,N}, /*ERD_SMA*/
{" Overrides previous allele frequencies for marker(s)"         ,W,Y,N,N}, /*ERD_MAF*/
{" Overrides previous allele frequencies for trait(s)"          ,W,Y,N,N}, /*ERD_TAF*/
{" There must be 3 mean values, one each for the (1 1), the (1 2) or (2 1),\n"
 "           and the (2 2) genotypes"                           ,E,Y,N,N}, /*ERD_TGM*/
{" Overrides previous genotype means for trait(s)"              ,W,Y,N,N}, /*ERD_TGR*/
{" Overrides previous additive variance(s) for trait(s)"        ,W,Y,N,N}, /*ERD_AVR*/
{" Overrides previous residual variance(s) for trait(s)"        ,W,Y,N,N}, /*ERD_RVR*/
{" Negative variance"                                           ,E,Y,N,N}, /*ERD_VMN*/
{" L sampler probability must be >= 0 and <= 1."                ,E,Y,N,N}, /*ERA_BLS*/
{" Marker data error;  provide for each observed individual:\n"
 "     name and a pair of alleles for each marker"              ,E,Y,N,Y}, /*ERA_MDL*/
{" Negative name or allele \"%s\" in marker data"               ,E,Y,Y,Y}, /*ERA_BMD*/
{" Overrides previous marker data statement"                    ,W,Y,N,N}, /*ERA_RMD*/
{" Overrides previous marker names statement"                   ,W,Y,N,N}, /*ERA_RMN*/
{" No. of markers \"%d\" out of range"                          ,E,Y,Y,Y}, /*ERA_MNO*/
{" Marker data error for individual %s:  negative allele"       ,E,Y,Y,N}, /*ERA_PAL*/
{" Marker data error for individual %s:  both alleles for a marker \n"
 "     must be observed (non-zero) or unobserved (zero)"        ,E,Y,Y,N}, /*ERA_UAL*/
{" Duplicate name \"%s\" in marker data"                        ,E,N,Y,N}, /*ERA_DIN*/
{" Duplicate marker name \"%s\""                                ,E,N,Y,N}, /*ERA_DMN*/
{" Bad component number \"%s\""                                 ,E,Y,Y,N}, /*ERA_CMP*/
{" Bad scoreset number \"%s\""                                  ,E,Y,Y,N}, /*ERA_SCS*/
{" One or more IBD pattern labels are out of range, including \"%s\""
                                                                ,E,Y,Y,N}, /*ERA_WPO*/
{" Overrides previous proband gametes for this component and scoreset"
                                                                ,W,Y,N,N}, /*ERA_OSC*/
{" Overrides previous list of affected individuals for this component"
                                                                ,W,Y,N,N}, /*ERA_OAI*/
{" Overrides previous IBD pattern labels for this component and scoreset"
                                                                ,W,Y,N,N}, /*ERA_OSP*/
{" There must be more than one proband gamete"                  ,E,Y,N,N}, /*ERA_PGL*/
{" Too many proband gametes;  %s permitted"                     ,E,Y,Y,N}, /*ERA_PGX*/
{" \"%s\" not a haplotype indicator;\n"
 "     must be \"0\" (maternal) or \"1\" (paternal)"            ,E,Y,Y,N}, /*ERA_BGM*/
{" Duplicate name and haplotype indicator:  %s"                 ,E,Y,Y,N}, /*ERA_DPG*/
{" Duplicate name \"%s\""                                       ,E,Y,Y,N}, /*ERA_DAI*/
{" Number of eigenvalues greater than pedigree size"            ,E,N,N,N}, /*ERR_EIM*/
{" \"input pedigree size\" statement missing"                   ,E,N,N,N}, /*ERF_PE0*/
{" Pedigree size is too large.  Use \"unibig\" instead for big pedigrees"
                                                                ,E,N,N,N}, /*ERF_PE1*/
{" Pedigree size too large.  Use \"allow pedigree size\" statement"
   "to override default"                                        ,E,N,N,N}, /*ERF_PE2*/
{" \"input pedigree record names ...\" statement missing"
                                                                ,E,N,N,N}, /*ERF_IPR*/
{" Expected \"input pedigree record names ...\" to specify at least %s integer(s)"
                                                                ,E,N,Y,N}, /*ERF_NIN*/
{" Expected \"input pedigree record names ...\" to specify at least %s real(s)"
                                                                ,E,N,Y,N}, /*ERF_NRL*/
{" Incorrect number of traits selected for program \"%s\""      ,E,N,Y,N}, /*ERR_STN*/
{" Incorrect number of fixed effects selected for program \"%s\""
                                                                ,E,N,Y,N}, /*ERR_SFN*/
{" A trait number is > the number of real numbers in the pedigree file"
                                                                ,E,N,N,N}, /*ERR_TRF*/
{" A fixed effect number is > the number of integers - 1 (for gender) in the pedigree file"
                                                                ,E,N,N,N}, /*ERR_FFF*/
{" Starting values given for non-selected traits or non-selected fixed effects"
                                                                ,W,N,N,N}, /*ERR_SEL*/
{" Starting value(s) given for trait mean or fixed effect levels\n"
 "     (will not be computed)"                                  ,W,N,N,N}, /*ERR_STV*/
{" Starting values must be given for the additive variances"    ,E,N,N,N}, /*ERR_VA0*/
{" Starting values must be given for the residual variances"    ,E,N,N,N}, /*ERR_VR0*/
{" Wrong number of starting values for additive variances"      ,E,N,N,N}, /*ERR_VSA*/
{" Wrong number of starting values for residual variances"      ,E,N,N,N}, /*ERR_VSR*/
{" Starting values for additive variances must be non-zero"     ,E,N,N,N}, /*ERR_VAN*/
{" Starting values for residual variances must be non-zero"     ,E,N,N,N}, /*ERR_VRN*/
{" Wrong no. of starting values for additive genetic covariances or correlations"
                                                                ,E,N,N,N}, /*ERR_CSA*/
{" Wrong no. of starting values for residual covariances or correlations"
                                                                ,E,N,N,N}, /*ERR_CSR*/
{" Wrong no. of fit/no-fit indicators for additive genetic covariances or correlations"
                                                                ,E,N,N,N}, /*ERR_CFA*/
{" Wrong no. of fit/no-fit indicators for residual covariances or correlations"
                                                                ,E,N,N,N}, /*ERR_CFR*/
{" Inconsistent independent groups in \"fit additive covariances/correlations\" statement"
                                                                ,E,N,N,N}, /*ERR_CAF*/
{" Inconsistent independent groups in \"fit residual covariances/correlations\" statement"
                                                                ,E,N,N,N}, /*ERR_CRF*/
{" Non-zero starting values for independent additive covariances ignored"
                                                                ,W,N,N,N}, /*ERR_CAZ*/
{" Non-zero starting values for independent residual covariances ignored"
                                                                ,W,N,N,N}, /*ERR_CRZ*/
{" Additive genetic covariance matrix is not positive definite" ,E,N,N,N}, /*ERR_PDA*/
{" Residual covariance matrix is not positive definite"         ,E,N,N,N}, /*ERR_PD1*/
{" \"compute eigenvalues\" overrides \"set eigenvalues\" statement"
                                                                ,W,N,N,N}, /*ERR_ET1*/
{" Eigenvalues will be computed"                                ,W,N,N,N}, /*ERR_ET2*/
{" Command line specification of peeling sequence file \n"
 "       overrides \"input peeling sequence file\""             ,W,N,N,N}, /*ERR_PT1*/
{" Expected at least one \"compute\" statement"                 ,E,N,N,N}, /*ERK_KIN*/
{" Duplicate recombination frequency in list"                   ,W,N,N,N}, /*ERK_DRF*/
{" Expected at least one \"simulate\" statement"                ,E,N,N,N}, /*ERD_SIM*/
{" Cannot specify selection/simulation of both unnamed and named chromosomes"
                                                                ,E,N,N,N}, /*ERD_CHR*/
{" No. of markers mapped differs for females and males %s"      ,E,N,Y,N}, /*ERD_NFM*/
{" No. of markers differs in the male and sex-averaged maps %s" ,E,N,Y,N}, /*ERD_NMA*/
{" No. of markers differs in the female and sex-averaged maps %s"
                                                                ,E,N,Y,N}, /*ERD_NFA*/
{" Marker map is missing %s"                                    ,E,N,Y,N}, /*ERD_MMM*/
{" One marker only is assumed, as there is no marker map %s"    ,W,N,Y,N}, /*ERA_1MA*/
{" Marker map expected, as selected marker's no. is > 1 %s"
                                                                ,E,N,Y,N}, /*ERA_MN1*/
{" Since 1 marker only is simulated, there should be no marker map %s"
                                                                ,E,N,Y,N}, /*ERD_NME*/
{" No. of markers mapped %s is not the same as no. simulated"
                                                                ,E,N,Y,N}, /*ERD_NAM*/
{" Marker no. in select statement is greater than the number of markers mapped %s"
                                                                ,E,N,Y,N}, /*ERA_MNL*/
{" Trait map is missing %s"                                     ,E,N,Y,N}, /*ERD_MTM*/
{" No. of traits mapped differs for females and males %s"       ,E,N,Y,N}, /*ERD_TFM*/
{" No. of traits differs in the male and sex-averaged maps %s"  ,E,N,Y,N}, /*ERD_TMA*/
{" No. of traits differs in the female and sex-averaged maps %s",E,N,Y,N}, /*ERD_TFA*/
{" Map statements are missing for the trait's starting and ending positions %s"
                                                                ,E,N,Y,N}, /*ERA_MBM*/
{" Map statements are missing for the trial trait positions %s" ,E,N,Y,N}, /*ERA_MXM*/
{" No trial trait positions were provided;  default positions will be used"
                                                                ,W,N,Y,N}, /*ERA_DAP*/
{" Default positions will be used for the trait at the beginning and/or end\n"
  "    of the chromosome and/or between markers %s"             ,W,N,Y,N}, /*ERA_DBE*/
{" Default interval proportions will be used for the trial trait positions\n"
 "      between markers %s"                                     ,W,N,Y,N}, /*ERA_IMI*/
{" Interval proportions for the trait will be ignored %s\n"
 "     (there is only one marker selected)"                     ,W,N,Y,N}, /*ERA_XEI*/
{" Marker interval number out of range %s;  it may not be greater than one less\n"
 "     than the number of markers mapped"                       ,E,N,Y,N}, /*ERA_MIO*/
{" The number of interval proportions for mapping the trait\n"
 "     is not the same for males and females %s"                ,E,N,Y,N}, /*ERA_WIP*/
{" Specification of trait interval proportions is missing %s"   ,E,N,Y,N}, /*ERA_MIP*/
{" No. of traits mapped %s is not the same as no. selected/simulated"
                                                                ,E,N,Y,N}, /*ERD_NAT*/
{" No. of traits mapped %s for males is not the same as no. selected/simulated"
                                                                ,E,N,Y,N}, /*ERD_NMT*/
{" No. of traits mapped %s for females is not the same as no. selected/simulated"
                                                                ,E,N,Y,N}, /*ERD_NFT*/
{" One trait must be specified in the \"select markers ...\" statement"
                                                                ,E,N,Y,N}, /*ERA_NAT*/
{" Marker no. in trait map %s > no. of markers mapped"
                                                                ,E,N,Y,N}, /*ERD_MMX*/
{" Trait id's differ for gender-specific or sex-averaged maps %s"
                                                                ,E,N,Y,N}, /*ERD_TID*/
{" Order of marker and trait loci differ for gender-specific or sex-averaged maps %s"
                                                                ,E,N,Y,N}, /*ERD_MID*/
{" Trait %s has more than 1 locus"                              ,E,N,Y,N}, /*ERD_LLL*/
{" Trait %s is specified as both mapped and unlinked"           ,E,N,Y,N}, /*ERD_LUL*/
{" Allele frequencies missing for %s:  defaults are .5 .5"      ,W,N,Y,N}, /*ERD_NML*/
{" Allele frequencies normalized for %s"                        ,W,N,Y,N}, /*ERD_MAN*/
{" Additive variance not given for trait %s:  default is 0.0"   ,W,N,Y,N}, /*ERD_TAV*/
{" Residual variance not given for trait %s:  default is 0.0"   ,W,N,Y,N}, /*ERD_TRV*/
{" Neither the additive nor residual variance is given for trait %s"
                                                                ,E,N,Y,N}, /*ERD_NAR*/
{" Sum of additive and residual variance for trait %s is 0.0"   ,E,N,Y,N}, /*ERD_SVR*/
{" Allele frequencies missing for trait %s:  defaults are .5 .5",W,N,Y,N}, /*ERD_NTL*/
{" Allele frequencies normalized for trait %s"                  ,W,N,Y,N}, /*ERD_TAN*/
{" Genotype means not given for trait %s:  defaults are 0. 0. 0."
                                                                ,W,N,Y,N}, /*ERD_NGM*/
{" Marker to trait distance is > marker to marker distance for trait %s"
                                                                ,E,N,Y,N}, /*ERD_MTD*/
{" Congruential seed (1st seed) must be odd,\n"
 " Tausworthe seed (2nd seed) must be non-zero"                 ,W,Y,N,N}, /*ERD_SEE*/
{" Overrides \"output trait seeds only\" statement"             ,W,Y,N,N}, /*ERD_OTS*/
{" Overrides \"output marker seeds only\" statement"            ,W,Y,N,N}, /*ERD_OMS*/
{" Overrides \"output sampler seeds only\" statement"           ,W,Y,N,N}, /*ERA_OSS*/
{" Overrides \"output permutation seeds only\" statement"       ,W,Y,N,N}, /*ERA_OPS*/

{" Default marker seeds are used"                               ,W,N,N,N}, /*ERD_SSD*/
{" Default trait seeds are used"                                ,W,N,N,N}, /*ERD_STD*/
{" Default permutation seeds are used"                          ,W,N,N,N}, /*ERA_SSP*/
{" Default sampler seeds are used"                              ,W,N,N,N}, /*ERA_SSS*/
{" Output pedigree file is not named in parameter statement or command line"
                                                                ,E,N,N,N}, /*ERF_OPF*/
{" No output seed file named.  \"output trait seeds only\" ignored"
                                                                ,W,N,N,N}, /*ERD_OST*/
{" No output seed file named.  \"Output marker seeds only\" ignored"
                                                                ,W,N,N,N}, /*ERD_OSM*/
{" No output seed file named.  \"output sampler seeds only\" ignored"
                                                                ,W,N,N,N}, /*ERA_USS*/
{" No output seed file named.  \"output permutation seeds only\" ignored"
                                                                ,W,N,N,N}, /*ERA_USP*/
{" The \"simulate ... markers ...\" statement is missing"       ,E,N,N,N}, /*ERA_SL1*/
{" The \"simulate ... markers ... using ...\" statement is missing"
                                                                ,E,N,N,N}, /*ERA_SLU*/
{" The \"select ... markers ...\" statement is missing"         ,E,N,N,N}, /*ERA_SEL*/
{" Only one chromosome may be selected"                         ,E,N,N,N}, /*ERA_OOS*/
{" One mapped or unlinked trait must be specified in \"select\" statement"
                                                                ,E,N,N,N}, /*ERA_OOT*/
{" One mapped or unlinked trait must be specified in \"simulate\" statement"
                                                                ,E,N,N,N}, /*ERA_OIT*/
{" %s traits selected;  only one may be specified in \"select\" statements"
                                                                ,E,N,Y,N}, /*ERA_MOT*/
{" A unlinked null locus must be selected"                      ,E,N,N,N}, /*ERA_MUL*/

{" %s traits selected;  only one may be specified in \"simulate\" statements"
                                                                ,E,N,Y,N}, /*ERA_MIT*/
{" No marker data given %s"                                     ,W,N,Y,N}, /*ERA_NMD*/
{" Wrong number of marker data:  provide name of each observed individual\n"
 "     together with a pair of alleles for each marker"         ,E,N,N,N}, /*ERA_BND*/
{" The number of marker names is not the same as the number of markers\n"
 "     mapped on the chromosome"                                ,E,N,N,N}, /*ERA_BNM*/
{" Marker data error for individual %s:  alleles must be integers"
                                                                ,E,N,Y,N}, /*ERA_BMA*/
{" Marker data error for individual %s:\n"
 "     specified allele > number of allele frequencies for marker"
                                                                ,E,N,Y,N}, /*ERA_WMA*/
{" No proband gametes specified"                                ,E,N,N,N}, /*ERA_MPG*/
{" Too many proband gametes:  with locus window size, use pairs of proband gametes"
                                                                ,E,N,N,N}, /*ERA_PIG*/
{" Window IBD pattern labels for scoring are ignored,\n"
 "     as no locus window size is specified"                    ,W,N,N,N}, /*ERA_MLW*/
{" IBD pattern labels, but not proband gametes, are given for\n"
 "     one or scoresets, including %s"                          ,W,N,Y,N}, /*ERA_MSS*/
{" IBD pattern label \"%s\" is out of range"                    ,E,N,Y,N}, /*ERA_BPC*/
{" No affected individuals;  provide names or trait no. for affecteds"
                                                                ,E,N,N,N}, /*ERA_NAF*/
{" Affected individuals are named in the parameter statements;\n"
 "     trait genotype codes in the pedigree file will be ignored"
                                                                ,W,N,N,N}, /*ERA_BAF*/
{" Wrong number of pseudo-priors;  expected %s, one greater than the number of"
 "     test positions"                                          ,E,N,Y,N}, /*ERA_NPP*/
{" Pseudo-prior for the unlinked position (0) must be non-zero" ,E,N,N,N}, /*ERA_ZUT*/
{" Locus window size out of range:  must be > 1 and <= %s, the number of loci"
                                                                ,E,N,Y,N}, /*ERA_LWO*/
{" Wrong number of incomplete penetrances.\n"
 "     Provide 1 each for the (1 1) (1 2) and (2 2) genotypes"  ,E,Y,N,Y}, /*ERA_NPV*/
{" Incomplete penetrances out of range;  values must be between 0. and 1."
                                                                ,E,Y,N,N}, /*ERA_RPV*/
{" Check incomplete penetrances:\n" 
 "     expected genotype (1 1) <= genotype (1 2) <= genotype (2 2)"
                                                                ,W,Y,N,N}, /*ERA_OPV*/
{" Incomplete penetrances are missing.\n"
 "    Use \"set incomplete penetrances ...\" statement"         ,E,N,N,N}, /*ERA_MPV*/
{" Incomplete penetrances are ignored as there is no trait data",W,N,N,N}, /*ERA_PNI*/
{" Number of MC iterations must be specified"                   ,E,N,N,N}, /*ERA_NIT*/
{" Interim scores will not be saved:\n"
 "     output scores file not named in parameter statement or command line"
                                                                ,W,N,N,N}, /*ERF_OSF*/
{" Trait type specification ignored:  there is no trait data,\n"
 "      only inheritance indicators at trait locus"             ,W,N,N,N}, /*ERA_IPT*/
{" Incompatible pedigree data type for trait %s,\n"
 "     (see \"input pedigree record traits ... integers ...\" statement)"
                                                                ,E,N,Y,N}, /*ERA_BIR*/
{" Incompatible pedigree data type for trait %s,\n" 
 "     (see \"input pedigree record traits ... integer pairs ...\" statement)"
                                                                ,E,N,Y,N}, /*ERA_BI2*/
{" Incompatible pedigree data type for quantitative trait %s,\n" 
 "     (see \"input pedigree record traits ...  reals ...\" statement)"
                                                                ,E,N,Y,N}, /*ERA_BRR*/
{" Not enough integers in pedigree record to include trait number %s"
                                                                ,E,N,Y,N}, /*ERA_NEI*/
{" Not enough reals in pedigree record to include trait number %s"
                                                                ,E,N,Y,N}, /*ERA_NER*/
{" Trait data was specified at position %s in the pedigree record\n"
 "     this is fewer than the number of integers in the record" ,E,N,Y,N}, /*ERA_NEJ*/
{" A quantitative trait was specified at position %s in the pedigree record\n"
 "     this is fewer than the number of reals in the record"    ,E,N,Y,N}, /*ERA_NEK*/
{" Duplicate record position %s for trait\n" 
 "     (see \"input pedigree record traits ...  integer pairs ...\" statement)"
                                                                ,E,N,Y,N}, /*ERA_DRP*/
{" Record position for trait %s assumed, as not specified in\n" 
 "     (\"input pedigree record traits ...  integers ...\" statement)"
                                                                ,W,N,Y,N}, /*ERA_TEI*/
{" Record position for trait %s assumed, as not specified in\n" 
 "     (\"input pedigree record traits ...  integer pairs ...\" statement)"
                                                                ,W,N,Y,N}, /*ERA_TE2*/
{" Record position for trait %s assumed, as not specified in\n" 
 "     (\"input pedigree record traits ...  reals ...\" statement)"
                                                                ,W,N,Y,N}, /*ERA_TER*/
{" Display of segregations and/or founder labels requested;\n"
 "     please specify names of individuals to be displayed"     ,E,N,N,N}, /*ERG_NDN*/ 
{" Graphical display requested for more than one pedigree component\n"
 "     The component %d only will be displayed"                 ,E,N,Y,N}, /*ERG_TMC*/ 
{" The display frequency is greater than the total number of MCMC iterations"
                                                                ,E,N,N,N}, /*ERG_TMI*/ 
{" The number of fixed effect levels in the pedigree file \n"
 "     is > the maximum no. defined in limdefs.h"               ,F,N,N,N}, /*ERR_FFL*/
{" The number of starting values given for fixed effect levels\n"
 "     is not the same as the number of levels found in the pedigree file"
                                                                ,W,N,N,N}, /*ERR_FFN*/
{" The largest component exceeds the maximum size permitted\n"
 "     Rerun with the \"allow component size\" statement"       ,F,N,N,N}, /*ERR_CHG*/
{" The number of eigenvalues in the \"set eigenvalues\" statement\n"
 "    is not the same as the number of observed individuals"    ,F,N,N,N}, /*ERR_EIP*/
{" Can't open eigenvalues file \"%s\""                          ,F,N,Y,N}, /*ERR_OEF*/
{" The number of eigenvalues in the eigenvalues file\n"
 "    is not the same as the number of observed individuals"    ,F,N,N,N}, /*ERR_EIE*/
{" The number of observed individuals exceeds the maximum number permitted\n"
 " Rerun with the \"allow observed individuals\" statement"     ,F,N,N,N}, /*ERR_OBS*/
{" Program quitting due to too many input errors"               ,Q,N,N,N}, /*ERR_MAX*/
{" Program quitting due to fatal error"                         ,Q,N,N,N}, /*ERR_FAT*/
{" Program quitting due to errors in parameter statements"      ,Q,N,N,N}};/*ERR_QIT*/

#undef E
#undef W
#undef F
#undef Q

#endif  /*ERRSPARS_H*/
