/* MORGAN: Copyright 1991-2005: Univ. of Washington. All rights reserved. */
/* You may copy, use, modify, but NOT redistribute, modified or otherwise.*/
/* See: http://www.stat.washington.edu/thompson/Genepi/license.shtml      */
/**
 * @file   globs.h
 *
 * @brief header file defining global variables.
 *
 *   -  global in scope
 *   -  included by main programs and function libraries
 *   -  contains modelling or simulation parameters
 *      derived from the input statements
 *   -  values are stored by Pars/ functions
 *
 * $Id: globs.h,v 1.71 2005/08/14 00:19:36 thompson Exp $
 **/

#ifndef  GLOBS_H
#define  GLOBS_H

/**
 * @addtogroup Libraries
 * @{
 */

#ifdef  DEFINE_DATA
GLOBAL char * ProgNam = PNAM;           /* name of Morgan program */
#else
GLOBAL char * ProgNam;
#endif /*DEFINE_DATA*/

/***  scalar variables, etc, derived from the parameter statements  ****/

GLOBAL int    PedSize;
GLOBAL int    NNams,        NInts,        NReals,       NFix,         NNInts;
GLOBAL int    NStdIn,       StdIn[5],     NStdOut,      StdOut[5];
GLOBAL int    NLoci,        NMarks,       NTraits,
              NLnkd,        NUnlnkd,      NVary,        UnNullLoc;

#ifdef  DEFINE_DATA                                  /* default values */
GLOBAL int SexIn  = UNDEF;
GLOBAL int SexOut = UNDEF;
GLOBAL int SexOp  = UNDEF;
GLOBAL int ObsIn  = UNDEF;
GLOBAL int AffTrIn  = 0;
GLOBAL int ObsOp  = UNDEF;
GLOBAL int ObsOut = UNDEF;
GLOBAL int LabelsOut = UNDEF;
GLOBAL int LabelOrd = UNDEF;
GLOBAL int LatentOut = UNDEF;
GLOBAL int DiscardReals = UNDEF;
GLOBAL int MarkSeedSet = N;
GLOBAL int TraitSeedSet = N;
GLOBAL int SamplSeedSet = N;
GLOBAL int PermSeedSet = N;
GLOBAL int MarkSeedsOut = N;
GLOBAL int TraitSeedsOut = N;
GLOBAL int SamplSeedsOut = N;
GLOBAL int PermSeedsOut = N;
GLOBAL int ChronOut = N;
GLOBAL int AllVarOut = N;
#else
GLOBAL int SexIn;
GLOBAL int SexOut;
GLOBAL int SexOp;
GLOBAL int ObsIn;
GLOBAL int AffTrIn;
GLOBAL int ObsOp;
GLOBAL int ObsOut;
GLOBAL int LabelsOut;
GLOBAL int LabelOrd;
GLOBAL int LatentOut;
GLOBAL int DiscardReals;
GLOBAL int MarkSeedSet;
GLOBAL int TraitSeedSet;
GLOBAL int SamplSeedSet;
GLOBAL int PermSeedSet;
GLOBAL int MarkSeedsOut;
GLOBAL int TraitSeedsOut;
GLOBAL int SamplSeedsOut;
GLOBAL int PermSeedsOut;
GLOBAL int ChronOut;
GLOBAL int AllVarOut;
#endif  /*DEFINE_DATA*/

GLOBAL uint32 MarkSeedC,    MarkSeedT,    TraitSeedC,   TraitSeedT;
GLOBAL uint32 SamplSeedC,   SamplSeedT,   PermSeedC,    PermSeedT;

       enum sEnums {NAM, PA, MA, SEX, OBS};

/*****     the following variables are used by Nghds/ functions     *****/

GLOBAL int NColData;                    /*No. of integers + 1 (including*/
                                        /*names, to selectively read    */
                                        /*from ped file                 */
                                        /*Is 1 greater than max SelInt[]*/
                                        /*which doesn't make sense??    */

GLOBAL int MaxNCmps;                    /*No. of components allocated   */

GLOBAL int * SelInt;                    /*Selection/nonselection of     */
                                        /*integers in the  ped file     */
                                        /*The subscript corresponds to  */ 
                                        /*the integer in the ped record */
                                        /*(starting from 0),            */
                                        /*the value to the column in    */
                                        /*data[][] (starting from 1)    */

GLOBAL int * SelReal;                   /*Selection/nonselection of     */
                                        /*reals (traits) from ped file  */

/**** the following variables and tables are used by Quant functions ****/

GLOBAL int FENo[FIXEFF];                /*For ea fixed effect selected  */
                                        /*the fixed effect no.          */

GLOBAL int TrNo[MAX_TR];                /*For traits selected, the      */
                                        /*trait no.'s                   */

GLOBAL int FESex;                       /*Gender a fixed effect?        */

GLOBAL int FfDex[MAX_TR][FIXEFF];       /*For ea trait selected, the    */
                                        /*presence/absence of ea fixed  */
                                        /*effects selected              */

GLOBAL int IFix[FIXEFF];                /*Number of levels (categories) */
                                        /*for ea fixed effect selected  */

GLOBAL int IOCor;                       /*flag for printing correlations*/
                                        /*(vs. covariances)             */

GLOBAL int ChkEigC;                     /*flag for printing progress in */
                                        /*computing eigenvalues         */


/*** the following variables are used by Quant/ and PolyEM functions ****/

GLOBAL int FinPheno;                    /*flag for printing final       */
                                        /*adjusted phenotypes           */

GLOBAL double Mufti[MAX_FEP][FIXEFF][LEVELS+1];  /*Fixed effect params; */
                                        /*Up to MAX_FEP combinations    */

GLOBAL double SigAdd, SigErr;           /* variance parameters          */
                                        /* (univariate)                 */

GLOBAL double TauSq[MAX_TR*(MAX_TR+1)/2],           /* covariances      */
              SigSq[MAX_TR*(MAX_TR+1)/2];

GLOBAL int MuftiStart[MAX_FEP][FIXEFF]; /* flags for starting values    */
                                        /* given for Mufti[][]          */

GLOBAL double * peigen;


/**  Limits for each program                                            */
struct _prog  {
  char *pnam;                        /**< program name                  */
  char pcod;                         /**< one char program code         */
  int  min_trait;                    /**< minimum no. of traits         */
  int  max_trait;                    /**< maximum no. of traits         */
  int  max_fef;                      /**< maximum no. of fixed effects  */
  int  max_chrom;                    /**< maximum chromosome no.        */
  int  max_mark;                     /**< maximum no. of markers        */
  int  max_allele;                   /**< maximum no. of alleles        */
  int  max_ped;                      /**< maximum pedigree size         */
  int  max_obs;                      /**< maximum no. observed          */
  int  max_cmp;                      /**< maximum component size        */
  int  max_n_cmp;                    /**< maximum number of components  */
               } *pmx;

#ifdef DEFINE_DATA
struct _prog Prog[] = {
 {"univar"         , 'u',  1,          1,   FIXEFF,         0,         0,
                              0, MAX_U_PED, MAX_U_OBS,         0,   MAX_NCMP },
 {"unibig"         , 'g',  1,          1,   FIXEFF,         0,         0,
                              0,   MAX_PED,   MAX_OBS, MAX_CPSIZ,   MAX_NCMP },
 {"bivar"          , 'b',  2,          2,   FIXEFF,         0,         0,
                              0,   MAX_PED,   MAX_OBS, MAX_CPSIZ,   MAX_NCMP },
 {"multivar"       , 'm',  1,  MAX_TRAIT,   FIXEFF,         0,         0,
                              0,   MAX_PED,   MAX_OBS, MAX_CPSIZ,   MAX_NCMP },
 {"pedcheck"       , 'p',  0,          0,        0,         0,         0,
                              0,   MAX_PED,         0,         0,          0 },
 {"kin"            , 'k',  0,          0,        0,         0,         0,
                              0,   MAX_PED,   MAX_OBS, MAX_CPSIZ,   MAX_NCMP },
 {"genedrop"       , 'd',  0, MAX_SIM_TR,        0, MAX_CHROM,  MAX_MARK,
                      MAX_ALLEL,   MAX_PED,         0,         0,          0 },
 {"ibddrop"        , 'i',  1, MAX_SIM_TR,        0, MAX_CHROM,   MAX_LOC,
                      MAX_ALLEL,   MAX_PED,      NOBS, MAX_CPSIZ, MAX_A_NCMP },
 {"markerdrop"     , 'r',  1, MAX_SIM_TR,        0, MAX_CHROM,   MAX_LOC,
                      MAX_ALLEL,   MAX_PED,      NOBS, MAX_CPSIZ, MAX_A_NCMP },
 {"lm_map"         , 'q',  1,          1,        0, MAX_CHROM,   MAX_LOC,
                      MAX_ALLEL,   MAX_PED,      NOBS, MAX_CPSIZ, MAX_A_NCMP },
 {"lm_pval"        , 'v',  1,          0,        0, MAX_CHROM,   MAX_LOC,
                      MAX_ALLEL,   MAX_PED,      NOBS, MAX_CPSIZ, MAX_A_NCMP },
 {"lm_ibdtests"    , 't',  1,          0,        0, MAX_CHROM,   MAX_LOC,
                      MAX_ALLEL,   MAX_PED,      NOBS, MAX_CPSIZ, MAX_A_NCMP },
 {"lm_auto"        , 'a',  1, MAX_SIM_TR,        0, MAX_CHROM,   MAX_LOC,
                      MAX_ALLEL,   MAX_PED,      NOBS, MAX_CPSIZ, MAX_A_NCMP },
 {"lm_lods"        , 'l',  1, MAX_SIM_TR,        0, MAX_CHROM,   MAX_LOC,
                      MAX_ALLEL,   MAX_PED,      NOBS, MAX_CPSIZ, MAX_A_NCMP },
 {"lm_bayes"       , 'y',  1, MAX_SIM_TR,        0, MAX_CHROM,   MAX_LOC,
                      MAX_ALLEL,   MAX_PED,      NOBS, MAX_CPSIZ, MAX_A_NCMP },
 {"lm_markers"     , 'e',  1, MAX_SIM_TR,        0, MAX_CHROM,   MAX_LOC,
                      MAX_ALLEL,   MAX_PED,      NOBS, MAX_CPSIZ, MAX_A_NCMP },
 {"lm_schnell"     , 's',  1, MAX_SIM_TR,   FIXEFF, MAX_CHROM,   MAX_LOC,
                      MAX_ALLEL,   MAX_PED,      NOBS, MAX_CPSIZ, MAX_A_NCMP },
 {"lm_twoqtl"      , 'w',  1, MAX_SIM_TR,        0, MAX_CHROM,   MAX_LOC,
                      MAX_ALLEL,   MAX_PED,      NOBS, MAX_CPSIZ, MAX_A_NCMP },
 {""               ,'\0',  0,           0,       0,         0,         0,
                              0,         0,         0,         0,          0 }};
#else
extern struct _prog Prog[];
#endif /*DEFINE_DATA*/

#define UNIVAR      (pmx->pcod == 'u')
#define UNIBIG      (pmx->pcod == 'g')
#define BIVAR       (pmx->pcod == 'b')
#define MULTIVAR    (pmx->pcod == 'm')
#define PEDCHECK    (pmx->pcod == 'p')
#define KIN         (pmx->pcod == 'k')
#define GENEDROP    (pmx->pcod == 'd')
#define IBDDROP     (pmx->pcod == 'i')
#define MARKERDROP  (pmx->pcod == 'r')
#define LM_MAP      (pmx->pcod == 'q')
#define LM_PVAL     (pmx->pcod == 'v')
#define LM_IBDTESTS (pmx->pcod == 't')
#define LM_AUTO     (pmx->pcod == 'a')
#define LM_LODS     (pmx->pcod == 'l')
#define LM_BAYES    (pmx->pcod == 'y')
#define LM_MARKERS  (pmx->pcod == 'e')
#define LM_SCHNELL  (pmx->pcod == 's')
#define LM_TWOQTL   (pmx->pcod == 'w')

#define IS_UNIVAR      (!strcmp (ProgNam, "univar"))
#define IS_UNIBIG      (!strcmp (ProgNam, "unibig"))
#define IS_BIVAR       (!strcmp (ProgNam, "bivar"))
#define IS_MULTIVAR    (!strcmp (ProgNam, "multivar"))
#define IS_PEDCHECK    (!strcmp (ProgNam, "pedcheck"))
#define IS_KIN         (!strcmp (ProgNam, "kin"))
#define IS_GENEDROP    (!strcmp (ProgNam, "genedrop"))
#define IS_IBDDROP     (!strcmp (ProgNam, "ibddrop"))
#define IS_MARKERDROP  (!strcmp (ProgNam, "markerdrop"))
#define IS_LM_MAP      (!strcmp (ProgNam, "lm_map"))
#define IS_LM_PVAL     (!strcmp (ProgNam, "lm_pval"))
#define IS_LM_IBDTESTS (!strcmp (ProgNam, "lm_ibdtests"))
#define IS_LM_AUTO     (!strcmp (ProgNam, "lm_auto"))
#define IS_LM_LODS     (!strcmp (ProgNam, "lm_lods"))
#define IS_LM_BAYES    (!strcmp (ProgNam, "lm_bayes"))
#define IS_LM_MARKERS  (!strcmp (ProgNam, "lm_markers"))
#define IS_LM_SCHNELL  (!strcmp (ProgNam, "lm_schnell"))
#define IS_LM_TWOQTL   (!strcmp (ProgNam, "lm_twoqtl"))

#define PAR      "par"
#define PED      "ped"
#define MDAT     "mark"
#define IGEN     "eig"
#define PEEL     "peel"
#define SEED     "seed"
#define OPED     "oped"
#define RPED     "rped"
#define OSCORE   "oscor"
#define ORBEST   "orbest"
#define OSEED    "oseed"
#define RSEED    "rseed"
#define IRESC    "iresc"
#define ORESC    "oresc"


/*** the following structure, _filtab, is used by Stuff/ functions      */
/*   and by other MORGAN functions                                      */

/**
 * Command Line or Parameter Statement Specified File Type Structore
 */

/*** the following structure, _filtab, is used by Stuff/ functions      */
/*   and by other MORGAN functions                                      */

/**
 * Command Line or Parameter Statement Specified File Type Structore
 */
struct _filtab
{
   char *type;                      /**< file type (com. line keyword)      */
   int command;                     /**< flag for named in command line     */
   int param;                       /**< flag for named in par statemnt     */
   int done;                        /**< flag for file has been read        */
   int parstat;                     /**< flag for contains param stmnts     */
   int reclo;                       /**< flag for rewrite command line type */ 
   int rewrit;                      /**< flag for rewrite option set        */
   char *pcods;                     /**< progs which use this file type     */
   char *fop;                       /**< fopen mode "r", "w" or "a"         */
   char *fname;                     /**< file name                          */
   char *fdesc;                     /**< description of file                */
};
typedef struct _filtab * FILTAB_PTR;

#ifdef  DEFINE_DATA
 struct _filtab Filtab[] = {
 {PAR   ,N,N,N,Y,N,N,"ugbmpkdirqvtalyes","r",NULL,"parameter file"      },
 {PED   ,N,N,N,Y,N,N,"ugbmpkdirqvtalyes","r",NULL,"input pedigree file" },
 {MDAT  ,N,N,N,Y,N,N,"         qvtalyes","r",NULL,"marker data file"    },
 {IGEN  ,N,N,N,N,N,N,"  bgm            ","r",NULL,"eigenvalue file"     },
 {PEEL  ,N,N,N,N,N,N," u               ","r",NULL,"peeling file"        },
 {SEED  ,N,N,N,Y,N,N,"      dirqvtalyes","r",NULL,"seed file"           },
 {OPED  ,N,N,N,N,N,N,"ugbmpkdirqvtalyes","w",NULL,"output pedigree file"},
 {RPED  ,N,N,N,N,Y,N,"ugbmpkdirqvtalyes","w",NULL,"output (overwrite) pedigree file"},
 {OSCORE,N,N,N,N,N,N,"            a  e ","w",NULL,"output score file"   },
 {ORBEST,N,N,N,N,N,N,"              y  ","w",NULL,"output rao-black estimates file"},
 {OSEED ,N,N,N,N,N,N,"      dirqvtalyes","a",NULL,"output seed file"    },
 {RSEED ,N,N,N,N,Y,N,"      dirqvtalyes","w",NULL,"output (overwrite) seed file"},
 {IRESC ,N,N,N,N,N,N,"            al   ","r",NULL,"input rescue file"   },
 {ORESC ,N,N,N,N,N,N,"            al   ","w",NULL,"output rescue file"  },
 {""    ,N,N,N,N,N,N,""                 ,"" ,NULL,""                    }};
#else
  extern struct _filtab Filtab[];
#endif  /*DEFINE_DATA*/

/** @} */
  
/**** the following variables and arrays are used by PedComp programs ***/
/**
 * @addtogroup PedComp
 * @{
 */
/** compute kinship                                                     */
#define KINSHP  0
/** compute inbreeding                                                  */
#define INBRD  1
/** compute 2-locus inbreeding                                          */
#define INBRD2  2

/** number of components for which kinship computations are requested   */
GLOBAL int NKinCmp;
/**
 * the no. of names of probands for each type of computation and for each
 * component
 */
GLOBAL int ** NKinNams;
/**
 * the names for each computation, for each component and for each proband.
 */
GLOBAL char **** KinNam;
/**
 * the index for each computation, for each component and for each proband.
 */
GLOBAL int *** KinIx;
/**
 * number of recomb fracts.
 */
GLOBAL int NInbreedRF;
/**
 * recomb fracts for computing 2-locus inbreeding coeffs.
 */
GLOBAL double * InbreedRF;
/** @} */
/**** the following variables and tables are used by PolyEM programs ****/
/**
 * @addtogroup polyem
 * @{
 */
                                        /*See limits.h for default      */
                                        /*values                        */
#ifdef  DEFINE_DATA
GLOBAL int ItMo = ITMO;                 /*Breeding-value iterations     */
GLOBAL int ItEM = ITEM;                 /*Number of EM iterations       */
GLOBAL int ItWt = ITWT;                 /*How often to write out estmts */
GLOBAL int IPtEM = 0;                   /*iem =1 partitioned EM         */
                                        /*iem =0 full EM                */
GLOBAL int IEn = 0;                     /*ien =1 fit environmental model*/
GLOBAL double Diff = DIFF;              /*Convergence criterion for     */
                                        /*breeding values iteration     */
GLOBAL int ICh = 0;                     /*ich =1 -- call bigcheig       */
                                        /*to compute eigenvalues        */
GLOBAL int ChkEig = N;                  /*flag for printing eigenvals   */
GLOBAL int ChkGnv = N;                  /*flag for printing G-iverse    */
GLOBAL int ChkGmt = N;                  /*flag for printing G-matrix    */
GLOBAL int ChkTrc = N;                  /*flag for printing trace       */

#else
GLOBAL int ItMo;
GLOBAL int ItEM;
GLOBAL int ItWt;
GLOBAL int IPtEM;
GLOBAL int IEn;
GLOBAL double Diff;
GLOBAL int ICh;
GLOBAL int ChkEig;
GLOBAL int ChkGnv;
GLOBAL int ChkGmt;
GLOBAL int ChkTrc;

#endif /*DEFINE_DATA*/


GLOBAL int    TauModel[MAX_TR*(MAX_TR+1)/2],            /* fit/no fit   */
              SigModel[MAX_TR*(MAX_TR+1)/2];            /* covariances  */
                                                        /* (multivar)   */
/** @} */

/******     the following structures are used by Genedrop          ******/
/**
 * @addtogroup GDrop
 * @{
 */

#define FM 1                    /**< first marker (on chromosome)       */
#define OM 2                    /**< other than first marker            */
#define FT 3                    /**< trait is first locus on chromosome */
#define IT 4                    /**< intermediate trait - has preceding
                                  marker/trait locus & following marker */
#define ET 5                    /**< trait with no following markers    */
#define UT 6                    /**< unlinked trait                     */

                        /*recombMap:  identifies neighbors and          */
                        /*            gender-specific probabilities     */
                        /*            used in recombination algorithms  */

struct recombMap {
    int no;                             /* markers & traits are in order*/
    int recombTyp;                      /* of chrom, locus;  unlinked   */
    int iA;                             /* traits are last              */
    int iB;
    double probA[2];
    double probB[2];
                 };
typedef struct recombMap RMAP;
typedef RMAP * RMAP_PTR;
GLOBAL RMAP_PTR PRecombMap;

                        /*traitPars:  parameters used for computing     */
                        /*            the additive and residual         */
                        /*            of the phenotype                  */
struct traitPars {
    int no;                             /* ids are in recombMap order   */
    double means[3];
    double addi_var;
    double resi_var;
    double addi_term_found;
    double addi_term;
    double resi_term;
                 };
typedef struct traitPars TPARS;
typedef TPARS * TPARS_PTR;
GLOBAL TPARS_PTR PTraitPars;

                        /*traitOut:  traits in numerical order          */
struct traitOut {
    int no;
    int wid;
    int dec;
    int ixG;
    int ixPh;
                };
typedef struct traitOut TOUT;
typedef TOUT * TOUT_PTR;
GLOBAL TOUT_PTR PTraitOut;

                        /*alleleFreq:  allele frequencies for each locus*/
struct alleleFreq{
    int nAlleles;                       /*no. of alleles at this locus  */
    double * pCumFreq;                  /*ptr to list of cumulative frqs*/
                 };
typedef struct alleleFreq ALLELE;
typedef ALLELE * ALLELE_PTR;
GLOBAL ALLELE_PTR PAlleleFreq;

GLOBAL int WidAlle;                     /*no. characters for printing   */
                                        /*alleles                       */

/** the following variables are used by Genedrop and Autozyg programs  **/

GLOBAL int NLocWndw;            /* window size (no. loci)               */
                                /* for ibddrop and lm_auto              */
GLOBAL int NWndw;               /* no. of windows for the chromosome    */


/** @} */

/**** the following variables and tables are used by Autozyg programs ***/

/**
 * @addtogroup autozyg
 * @{
 */

#ifdef  DEFINE_DATA

GLOBAL int PedDatTyp = UNDEF;           /* INHERITANCE for pair of      */
                                        /*     inheritance indicators   */
                                        /* GENOTYPIC for genotype code  */
                                        /* DISCRETE for phenotype       */
                                        /* DISCR_LIAB for discrete plus */
                                        /*     liability class          */ 
                                        /* QUANTITATIVE for continuous  */

GLOBAL int TraitTyp = UNDEF;
                                        /* GENOTYPIC for genotype code  */
                                        /* DISCRETE for phenotypic      */
                                        /* QUANTITATIVE for continuous  */
                                        /* NULLLOC for no trait locus   */
                                        /* UNLNKDNULL, no trait locus   */

GLOBAL int SmplSetup = UNDEF;           /* which sample setup to use    */
GLOBAL int NSeqImp = UNDEF;             /* number of samples for        */
                                        /* sequential imputation        */
GLOBAL int Scan = UNDEF;                /* sample by scan (T) or single */
                                        /* step(F)                      */
GLOBAL double LSamplProb = 0.2;         /* locus sampler probability    */
GLOBAL int WindWdth = 6;                /* width of test position wndow */
GLOBAL int SeqImpIt = UNDEF;            /* how often to propose seq imp */
GLOBAL int ScorIt = 1;                  /* score ECLLD and recomb counts*/
                                        /* every ScorIt steps or scans  */
GLOBAL int NBrnIt = UNDEF;              /* number of burnin iterations  */
GLOBAL int NPreIt = UNDEF;              /* number of preliminary iter   */
GLOBAL int NIt  = UNDEF;                /* number of step or scan       */
                                        /* iterations (main)            */
GLOBAL int ChkIt  = UNDEF;              /* how often to report iter no. */
GLOBAL int FilIt = UNDEF;               /* how often to write outpt file*/
GLOBAL int OutRB = UNDEF;               /* flag for writting rao-black  */
                                        /* estimates to a file          */
GLOBAL int InPseudo = N;                /* flag for pseudo-priors input */
#else
GLOBAL int UnNullLoc;
GLOBAL int TraitTyp;
GLOBAL int PedDatTyp;
GLOBAL int SmplSetup;
GLOBAL int NSeqImp;
GLOBAL int Scan;
GLOBAL double LSamplProb;
GLOBAL int WindWdth;
GLOBAL int SeqImpIt;
GLOBAL int ScorIt;
GLOBAL int NBrnIt;
GLOBAL int NPreIt;
GLOBAL int NIt;
GLOBAL int ChkIt;
GLOBAL int FilIt;
GLOBAL int OutRB;
GLOBAL int InPseudo;
#endif /*DEFINE_DATA*/

GLOBAL int NPenVals;          /* number of penetrance values                  */
GLOBAL double PenVal[3];      /* incomplete penetrances for                   */
                              /* (1,1) (1,2) and (2,2) genotypes              */

                              /* no. of marker plus test trait positions      */
GLOBAL int NLocPos;

                              /* LocPos:  description of each marker          */
                              /*            and trial trait position          */
struct locus_pos {
    int locus_typ;            /* is this a marker or trail trait position?    */
    int intv_no;              /* interval no., 0 ... n                        */
                              /*     0 : before marker 1,                     */
                              /*     i : marker i,                            */
                              /*         between marker i, i+1, if trait pos  */
                              /*     n : marker n,                            */
                              /*         or following marker n, if trait pos  */
    int no;                   /* marker or trait no. (no. from maps)          */
    double rf[2];             /* sex-specific recomb fract with preced marker */
                              /* or marker 1, for interval "0"                */
    double pos[2];            /* sex-specific distance to marker 1, in cM     */
                     };
typedef struct locus_pos LOCPOS;
typedef LOCPOS * LOCPOS_PTR;
GLOBAL LOCPOS_PTR LocPos;

GLOBAL char ** MarkNam;       /* Name of each marker                          */

GLOBAL char ** TraitNam;      /* Name of each trait                           */

GLOBAL int LocNamLen;         /* No. chars in longest marker or trait name    */

                              /* trial trait positions:                       */
                              /* total no. of positions on chrom for          */
GLOBAL int NTPos;             /* lod scoring.  (This is the no. derived from  */
                              /* the parameter statements)                    */
GLOBAL int NTPos1;            /* this is the old definition of NTPos          */

                              /* no. of test positions for trait for          */
GLOBAL int * NTPosInt;        /* each interval                                */

                              /* alleles:                                     */
                              /* no. of alleles at each locus                 */
                              /* NAll[0]...NAll[Nloci-1]                      */
GLOBAL int * NAll;            /* ordered by name (chromo[][])                 */

                              /* allele frequencies at each locus             */
                              /* Qq[0][1]....Qq[0][NAll[0]]                   */
GLOBAL double **Qq;           /*          ...Qq[NLoci-1][NAll[NLoci-1]]       */

                              /* marker data:                                 */
GLOBAL int NObs;              /* no. individuals observed                     */
                              /* (autozyg, for marker data,                   */
                              /* (polyEM, for selected traits)                */
GLOBAL char ** ObsNam;        /* names of observed individuals                */
GLOBAL int  *  ObsIx;         /* indices of observed individuals              */
GLOBAL int *** ObsMark;       /* for ea observed individual, for ea           */
                              /* marker, a pair of alleles                    */

                              /* proband gametes and pattern labels:    */
GLOBAL int MxGam;             /* the largest number of gametes in a set */

                              /* proband gametes:                       */
GLOBAL int NScCmp;            /* greatest component no. with proband    */
                              /* gamete sets;  may be reset to no.      */
                              /* components read if smaller             */ 
GLOBAL int NScCmpAlloc;       /* greatest comp. no. with proband gamete */
                              /* sets (for freeing)                     */
GLOBAL int * NScSet;          /* for each component, the number of sets */
                              /* of proband gametes                     */
GLOBAL int ** ScSetNo;        /* for each component for each set,       */
                              /* its set no. (need not be contiguous)   */
GLOBAL int ** NPrGam;         /* for each component for each set,       */
                              /* number of probands                     */
GLOBAL char **** ProbNam;     /* for each component for each set,       */
                              /* the list of names of individuals       */
                              /* corresponding to the proband gametes   */
                              /* to be scored                           */
GLOBAL int *** ProbIx;        /* the list of indices to the individuals */
GLOBAL int *** ProbGam;       /* the list of gametes corresponding to   */
                              /* the list of individuals                */

                              /* IBD pattern labels:                    */
GLOBAL int ** NPtLab;         /* for each component and each set,       */
                              /* number of IBD pattern labels           */
GLOBAL int *** PatLab;        /* for each component and each set,       */
                              /* the list of IBD pattern labels         */

                              /* affected individuals:                  */
GLOBAL int NAffCmp;           /* number of components with affected     */
                              /* individuals                            */
GLOBAL int * NAffInd;         /* number of affected individs in comp    */
GLOBAL char *** AffNam;       /* for the component, list of names of    */
                              /* affected individuals                   */

                              /* lm_bayes preliminary step:             */
GLOBAL double * PseudoPrior;  /* the pseudo-priors (input or calculated)*/
                              /* for each position                      */

/**
 * Marker recombination fractions, dimension 2 by @ref NMarks + 1.
 * The first subscript is for sex, 0 = female; 1 = male.  The second subscript
 * is marker: Marek[][0] = .5 (not used); Marek[][1] = recomb between marker
 * "1" and marker "2"; Marek[][n] -- recomb between marker "n" and marker
 * "n+1";  Marek[][NMarks] -- .5.
 */
GLOBAL double ** Marek;

/**
 * Trial positions for linked trait.
 */
GLOBAL int Loc0;
GLOBAL double Rec0[2];        /* and gender-specific recomb fractions   */

GLOBAL int * IntVal;          /* [NTPos+2] positions for trait          */
GLOBAL double ** RecVal;      /* [2][NTPos+2] sex-specifc recomb fracts */
                              /* (extra position on ea end with recomb  */
                              /* fraction of .5                         */

/**** the following variables are used for graphics displays        *****/
/*
 * Autozyg/lm_auto graphics uses OpenGl and freeglut
 * The GLDisp/ library contains graphics functions to call these
 */

GLOBAL int GDispOpt;           /* DEFLT_DISP, SEGR_DISP, LABL_DISP      */
GLOBAL int GDispIt;            /* how often to display                  */
GLOBAL int GDispNIt;           /* current iteration                     */

GLOBAL int GDispNamLen;        /* how many chars are needed for name    */
GLOBAL int NGDispCmp;          /* no. of ped components to display      */
GLOBAL int * NGDispNams;       /* no. individuals to display for cmpnt  */
GLOBAL char *** GDispNam;      /* names of individusal                  */
GLOBAL int ** GDispIx;         /* indices to individuals                */ 

GLOBAL int NGDispLoc;          /* no. of loci to display                */
GLOBAL int * GDispLoc;         /* loci to display                       */ 


/***** the following variables are for Morgan programmers               */

GLOBAL int DebugMain;
GLOBAL int DebugCMF;
GLOBAL int DebugGLDisp;
GLOBAL int DebugMarkers;
GLOBAL int DebugNghds;
GLOBAL int DebugPars;
GLOBAL int DebugPedchk;
GLOBAL int DebugPeel;
GLOBAL int DebugQuant;
GLOBAL int DebugRans;
GLOBAL int DebugSample;
GLOBAL int DebugStuff;

/** @} */

#endif  /*GLOBS_H*/
