/* MORGAN: Copyright 1991-2005: Univ. of Washington. All rights reserved. */
/* You may copy, use, modify, but NOT redistribute, modified or otherwise.*/
/* See: http://www.stat.washington.edu/thompson/Genepi/license.shtml      */
/*:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::*/
/*       parsedefs.h:  header file for the Pars/ library               */
/*                     -  of file scope                                */
/*                     -  included by Pars/ functions                  */
/*                     -  contains definitions of parameter statements */
/*                        and command line paramters for main programs */
/*                     -  definitions of statements are included from  */
/*                        parsedefs_d.h                                */
/*                     -  texts of statments are included from         */
/*                        parsedefs_f.h                                */
/*:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::*/

#ifndef  PARSEDEFS_H
#define  PARSEDEFS_H

/**
 * @addtogroup parse
 * @{
 */

#define KEYWD 0                         /*Token is a keyword            */
#define PDAT 1                          /*Token is data                 */
#define MAXNKEY  8                      /*The max # of keywords in a    */
                                        /*statement                     */
#define MAXNTOKEN 200                   /*The max # of tokens in a      */
                                        /*statement (init. # only)      */
#define NOMTOKLEN  8                    /*Nominal # of characters per   */
                                        /*token in an input statement   */

#define COM_LIN_FORM  "<prognam> <parfile>"

/*  line_in[]   - the next line of input data                           */
/*  token[][]   - the tokens found in the input statemnt                */
/*  token[]     - a list of pointers to the start of each token         */
/*  tok_typ[]   - corresponding token type                              */
/*                (alphabetic, numeric or string)                       */
/*  aggre_key[] - the aggregate key                                     */
/*                (concatenated first 4 characters of each keyword)     */
/*  key_ix      - the statement type, as determined by the aggregate key*/

static char     line_in[MAXLINLEN+2];
static char  ** token;
static int    * tok_typ;
static char     aggre_key[MAXNKEY*4+1];
static int      key_ix;

/*  function prototypes for functions which correspond to the various   */
/*  input statement types                                               */

/*  for later, note that a typedef can be used;  
 *  try typedef static int PARSTAT_FUNC (void)
 *  PARSTAT_FUNC allow_cmpsiz, allow_obsiz, ...
 */

static int
    not_impl       (void),
    allow_cmpsiz   (void),
    allow_obsiz    (void),
    allow_pedsiz   (void),
    assign_gend    (void),
    assum_all_obs  (void),
    check_eigen    (void),
    check_eig_cmp  (void),
    check_ginv     (void),
    check_gmat     (void),
    check_mc_iter  (void),
    check_trace    (void),
    cmpu_cinbreed  (void),
    cmpu_ckinshp   (void),
    cmpu_scor_iter (void),
    cmpu_c2_loc    (void),
    comp_eigen     (void),
    cmpu_inbreed   (void),
    cmpu_kinshp    (void),
    cmpu_2_loc     (void),
    disp_deflt     (void),
    disp_iter      (void),
    disp_indvs     (void),
    disp_cp_indvs  (void),
    disp_labls     (void),
    disp_segrg     (void),
    fit_addi       (void),
    fit_no_gen     (void),
    fit_resi       (void),
    ignor_gend     (void),
    in_eig_fil     (void),
    in_mrkdat_fil  (void),
    in_gend_abs    (void),
    in_gend_pres   (void),
    in_ord_fath    (void),
    in_ord_moth    (void),
    in_obs_abs     (void),
    in_obs_pres    (void),
    in_ped_fil     (void),
    in_ped_int     (void),
    in_ped_int_rl  (void),
    in_ped_n       (void),
    in_ped_n_i     (void),
    in_ped_n_i_r   (void),
    in_ped_n_r     (void),
    in_ped_tr_int  (void),
    in_ped_tr_int2 (void),
    in_ped_tr_real (void),
    in_pedsiz      (void),
    in_peel_fil    (void),
    in_resc_fil    (void),
    in_seed_fil    (void),
    lim_breed      (void),
    lim_em_iter    (void),
    map_cmarkd     (void),
    map_cmarkd_f   (void),
    map_cmarkd_m   (void),
    map_cmarkp     (void),
    map_cmarkp_f   (void),
    map_cmarkp_m   (void),
    map_cmarkr     (void),
    map_cmarkr_f   (void),
    map_cmarkr_m   (void),
    map_ctraitd    (void),
    map_ctraitd_f  (void),
    map_ctraitd_m  (void),
    map_ctraitr    (void),
    map_ctraitr_f  (void),
    map_ctraitr_m  (void),
    map_ctr_begd_f (void),
    map_ctr_begd_m (void),
    map_ctr_begd   (void),
    map_ctr_begr_f (void),
    map_ctr_begr_m (void),
    map_ctr_begr   (void),
    map_ctr_endd_f (void),
    map_ctr_endd_m (void),
    map_ctr_endd   (void),
    map_ctr_endr_f (void),
    map_ctr_endr_m (void),
    map_ctr_endr   (void),
    map_ctr_extd_f (void),
    map_ctr_extd_m (void),
    map_ctr_extd   (void),
    map_ctr_extr_f (void),
    map_ctr_extr_m (void),
    map_ctr_extr   (void),
    map_ctr_intv_f (void),
    map_ctr_antv_f (void),
    map_ctr_intv_m (void),
    map_ctr_antv_m (void),
    map_ctr_intv   (void),
    map_ctr_antv   (void),
    map_markd      (void),
    map_markd_f    (void),
    map_markd_m    (void),
    map_markp      (void),
    map_markp_f    (void),
    map_markp_m    (void),
    map_markr      (void),
    map_markr_f    (void),
    map_markr_m    (void),
    map_traitd     (void),
    map_traitd_f   (void),
    map_traitd_m   (void),
    map_traitr     (void),
    map_traitr_f   (void),
    map_traitr_m   (void),
    map_tr_begd_f  (void),
    map_tr_begd_m  (void),
    map_tr_begd    (void),
    map_tr_begr_f  (void),
    map_tr_begr_m  (void),
    map_tr_begr    (void),
    map_tr_endd_f  (void),
    map_tr_endd_m  (void),
    map_tr_endd    (void),
    map_tr_endr_f  (void),
    map_tr_endr_m  (void),
    map_tr_endr    (void),
    map_tr_extd_f  (void),
    map_tr_extd_m  (void),
    map_tr_extd    (void),
    map_tr_extr_f  (void),
    map_tr_extr_m  (void),
    map_tr_extr    (void),
    map_tr_intv_f  (void),
    map_tr_antv_f  (void),
    map_tr_intv_m  (void),
    map_tr_antv_m  (void),
    map_tr_intv    (void),
    map_tr_antv    (void),
    map_cmarkKd    (void),
    map_cmarkKd_f  (void),
    map_cmarkKd_m  (void),
    map_cmarkKp    (void),
    map_cmarkKp_f  (void),
    map_cmarkKp_m  (void),
    map_ctraitKd   (void),
    map_ctraitKd_f (void),
    map_ctraitKd_m (void),
    map_ctr_begKd_f(void),
    map_ctr_begKd_m(void),
    map_ctr_begKd  (void),
    map_ctr_endKd_f(void),
    map_ctr_endKd_m(void),
    map_ctr_endKd  (void),
    map_ctr_extKd_f(void),
    map_ctr_extKd_m(void),
    map_ctr_extKd  (void),
    map_markKd     (void),
    map_markKd_f   (void),
    map_markKd_m   (void),
    map_markKp     (void),
    map_markKp_f   (void),
    map_markKp_m   (void),
    map_traitKd    (void),
    map_traitKd_f  (void),
    map_traitKd_m  (void),
    map_tr_begKd_f (void),
    map_tr_begKd_m (void),
    map_tr_begKd   (void),
    map_tr_endKd_f (void),
    map_tr_endKd_m (void),
    map_tr_endKd   (void),
    map_tr_extKd_f (void),
    map_tr_extKd_m (void),
    map_tr_extKd   (void),
    out_chron      (void),
    out_corr       (void),
    out_covar      (void),
    out_fin_pheno  (void),
    out_mark_seeds (void),
    out_trait_seeds(void),
    out_samp_seeds (void),
    out_perm_seeds (void),
    out_ped_fil    (void),
    out_rw_ped_fil (void), 
    out_ord_fath   (void),
    out_found_labs (void),
    out_ord_moth   (void),
    out_tr_vars    (void),
    out_unobs      (void),
    out_rb_est_fil (void),
    out_resc_fil   (void),
    out_scor_fil   (void),
    out_scor_iter  (void),
    out_seed_fil   (void),
    out_rw_seed_fil(void),
    out_space      (void),
    sampl_by_scan  (void),
    sampl_by_step  (void),
    sela_mark      (void),
    sela_mark_tr   (void),
    sela_cmark     (void),
    sela_cmark_tr  (void),
    sel_cmark      (void),
    sel_cmark_tr   (void),
    sel_mark       (void),
    sel_mark_tr    (void),
    sel_trait      (void),
    sel_tr_eff     (void),
    sel_unlnkd_null(void),
    set_addi_var   (void),
    set_affind_tr  (void),
    set_tstpos_wind(void),
    set_br_conv    (void),
    set_burn_iter  (void),
    set_cloc_wind  (void),
    set_cmark_dat  (void),
    set_cmark_frq  (void),
    set_cmark_nams (void),
    set_affnam     (void),
    set_cp_affnam  (void),
    set_cp_prbgam  (void),
    set_pseudo_pr  (void), 
    set_incmplt_pen(void), 
    set_cs_prbgam  (void),
    set_cs_windpat (void),
    set_cp_windpat (void),
    set_eigen      (void),
    set_geno_mean  (void),
    set_lsmpl_prob (void),
    set_loc_wind   (void),
    set_mark_dat   (void),
    set_mark_frq   (void),
    set_mark_nams  (void),
    set_mark_seeds (void),
    set_mc_iter    (void),
    set_norm_frq   (void),
    set_perm_seeds (void),
    set_prel_iter  (void),
    set_prbgam     (void),
    set_reco_frq   (void),
    set_resi_var   (void),
    set_sampl_seeds(void),
    set_sc_prbgam  (void),
    set_sc_windpat (void),
    set_sqimp_prop (void),
    set_trait_frq  (void),
    set_trait_seeds(void),
    set_trait_disc (void),
    set_trait_liab (void),
    set_trait_geno (void),
    set_trait_quant(void),
    set_windpat    (void),
    sim_cmark      (void),
    sim_cmark_trait(void),
    sim_cmark_uinhr(void),
    sim_cmark_utrt (void),
    sim_mark       (void),
    sim_mark_trait (void),
    sim_mark_uinhr (void),
    sim_mark_utrt  (void),
    sim_unlinked   (void),
    start_add_cor  (void),
    start_add_cov  (void),
    start_add_var  (void),
    start_effect   (void),
    start_mean     (void),
    start_res_cov  (void),
    start_res_cor  (void),
    start_res_var  (void),
    use_full_em    (void),
    use_part_em    (void),
    use_lsmpl_setup(void),
    use_sqimp_setup(void),
    use_sqimp_realz(void),
    set_debugmain   (void), 
    set_debugcmf   (void), 
    set_debuggldisp(void), 
    set_debugmarkrs(void), 
    set_debugnghds (void), 
    set_debugpars  (void), 
    set_debugpedchk(void), 
    set_debugpeel  (void), 
    set_debugquant (void), 
    set_debugrans  (void), 
    set_debugsample(void), 
    set_debugstuff (void); 

/*  prototype for input statement                                             */
/*  proto.key[]:  string containing the first four letters of each            */
/*                keyword in the statement.                                   */
/*  proto.par:    a code for each data value or list of values in the         */
/*                statement.                                                  */
/*                "i" - an integer value             example:   3000          */
/*                "k" - a list of integer values                2 4 5         */
/*                "h" - a hexadecimal value                     0x73331ad7    */
/*                "r" - a real value                            22.33         */
/*                "x" - a list of real values                   .3 .2 .9      */
/*                "s" - a string                                "pedFil3"     */
/*                "n" - a list of names                         A313 8333     */
/*                "g" - alternating names & integers                          */
/*                      (e.g. proband gametes)                  C16 0 D26 1   */
/*                "a" - sets of name plus list of integers,                   */
/*                      with the no. of sets specified by "i"                 */
/*                      (e.g. marker data)                      A1 3 5 3 3    */
/*                                                              B3 3 2 3 3    */

/*  proto.func:   the name of the function which acts upon the data           */
/*                from the statement                                          */
/*  proto.count:  the number of times this statement type occurs in the       */
/*                input file                                                  */

static char *nam_cod = "nga";   /* codes for data lists containing names      */

/* pcods below are:                                                           */
/*       u - univar,     g - unibig,     b - bivar,      m - multivar,        */
/*       p - pedchk,     k - kin,                                             */
/*       d - genedrop,   i - ibddrop,    r - markerdrop,                      */
/*       q - lm_map,     v - lm_pval,    t - lm_ibdtests,a - lm_auto          */
/*       l - lm_lods,    y - lm_bayes,   e - lm_markers, s - lm_schnell       */
/*       w - lm_twoqtl                                                        */

/* contents of table are included from file parsedefs_d.h                     */

static struct _proto {
       char * key;
       char * par;
       char * pcods;
       int (* func) (void);
       int    count;
               } *ppp;

static struct _proto  proto[] = {
#include "parsedefs_d.h"            /* main body of table included here */
{""  ,""  ,""    ,not_impl    ,0}}; /* the last line is the sentinel    */

/*  descriptions of input statements, SAME ORDER and SAME NUMBER        */
/*  as the statement prototypes above                                   */

/* contents of table are included from file parsedefs_f.h               */

static char * input_statement[] = {
#include "parsedefs_f.h"            /* main body of table included here */
 ""                             } ; /* the last line is the sentinel    */
                                                                                                         



/* these words may begin a new input statement, thus delimiting the     */
/* previous statement.                                                  */

static char * verb[] = {
             "allo",                /* allow        */
             "assi",                /* assign       */ 
             "assu",                /* assume       */
             "chec",                /* check        */
             "comp",                /* compute      */
             "disp",                /* display      */
             "fit",                 /* fit          */
             "igno",                /* ignore       */
             "inpu",                /* input        */
             "limi",                /* limit        */
             "map",                 /* map          */
             "outp",                /* output       */
             "samp",                /* sample       */
             "sele",                /* select       */
             "set",                 /* set          */
             "simu",                /* simulate     */
             "star",                /* start        */
             "use",                 /* use          */
             "****",                /* end-of-file  */
             ""     };

/** @} */

#endif  /*PARSEDEFS_H*/

