/* MORGAN: Copyright 1991-2005: Univ. of Washington. All rights reserved. */
/* You may copy, use, modify, but NOT redistribute, modified or otherwise.*/
/* See: http://www.stat.washington.edu/thompson/Genepi/license.shtml      */
/*  vim: sw=3
 *  This is the file schnell2.c, modified from Simon's loki_test.c to include
 *  the multi_point lod estimation from Shi Li's multi point stuff.  Modified
 *  6/99 GLS this version also includes the joint segregation stuff.
 *
 *  $Id: lm_schnell.c,v 1.15 2005/08/14 00:20:32 thompson Exp $
 */

#define PNAM "lm_schnell"
#define SCHNELL

#include "lodscore.h"
#include "setupAutozyg.c"

/* Header files current directory */
#include "mixds2.c"
#include "shnl_chk.c"

void  free_schnell (int);

int main (int argc, char *argv[])
{
   /* Global varibles */
   extern struct Locus *loci;           /* lsample.h */
/* extern double *ym;                  lsample.h */
   /* Centered trait values */

   /* {{{ Local variable */
   FILE *oSeedF;                        /* Output seed file */

   int   i, l, j, k, jj, c, h, t;

   int   trait_loc0[] = { 0, 0, 0 };
   double trait_rec0[] = { 0.5, 0.5, 0.5 };

   /* The positions of the current trait locus.  Lod score is computed
    * of the mid one relative to the left and right.  Hence 3 positions.
    * See set_chromosomes() in Msample/set_chromosome.c. */
   int   sx = 0;
   /*  variables related to list_switch() */
   int   nsw = 0;
   int  *perm_sw = 0;
   int  *perm_loc = 0;                  /* permutation arrays of switches
                                         * and loci  */
   double mu = 0.0;                     /* Global mean       */

   double *additive = NULL;             /* Additive genetic component,
                                           updated when va > 0 */

   int  *scounts;                       /* [nc]         */
   int ***recounts;                     /* [nc][nl][6]  */
   int **lsflag;                        /* [3][nl]      flags for lsampler
                                         *              l_ratio adjustments */
   double ddtmp;
   double **l_ratio;                    /* [nc][3]      now this is the
                                         *              LOG-l-ratio EAT 12/00 */
   double **prob;                       /* [nc][nl]    */
   double **cumulik;                    /* [nc][3]     */
   double **temp_log_prob_seg, **temp2_log_prob_seg; /* [nc][NMarks]  */
   double **tot_log_prob_seg, **tot_log_prob_pen; /* [nc][NMarks]  */
   double **tot_log_left_seg, **tot_log_rght_seg; /* [nc][NMarks]  */
   int   tot_count;

   double ***likeratios = NULL;         /* Matrix for likelihood ratios */
   double leftratio = 0.0;
   double rightratio = 0.0;

   /* .................................................. */
   /* the fourth argument is the number of haplotypes */

   nsw = setupAutozyg (argc, argv, PNAM, 2);
   prob = setupInherit (0);

   /* .................................................. */

   /* Now, the trait data are in x[NTraits][np+1], where np is the no. of
    * individual with complete trait data.  x[][0] are all zeros.
    * (This implies that we are doing a complete-case analysis.)
    */

   /* set up permutation arrays for loci and switches */
   if (Scan)
   {
      perm_sw = malloc (nsw * sizeof (int));
      alloc_chk (WHERE, perm_sw);
      for (i = 0; i < nsw; ++i)
         perm_sw[i] = i;
      perm_loc = malloc ((NMarks + 1) * sizeof (int));
      alloc_chk (WHERE, perm_loc);
      for (i = 0; i <= NMarks; ++i)
         perm_loc[i] = i;
   }

   /* allocate and set ym[], the set of centered trait values   */
   /* This segment removed:  EAT 08/2004                        */

   set_seg_mode (0);

   if (loci[0].va > 0)
   {
      additive = malloc (PedSize * sizeof *additive);
      alloc_chk (WHERE, additive);
      for (i = 0; i < PedSize; ++i)
         additive[i] = 0.0;
   }

   /* If there is a recovery file, skip all the following finding init config
    * and burn-in stuff.  Also we don't have to sample with trait unlinked
    * because that's the first checkpoint we have.  */

   /* init mixd stuff */
   if (loci[0].va > 0)
      smpl_a (loci, NMarks + 1, loci[0].ve, loci[0].va, mu, np, additive);

   /* Allocate memory for likeratios and lods scores */
   likeratios = alloc_dbl3d (n_compnts, NTPos1, NTPos1);

   alloc_loglik (&recounts, &tot_log_prob_seg, &temp_log_prob_seg,
                 &tot_log_prob_pen);
   alloc_lod (&scounts, &lsflag, &l_ratio, &cumulik,
              &temp2_log_prob_seg, &tot_log_left_seg, &tot_log_rght_seg);

   alloc_lpflag (NMarks, 2);
   alloc_logprob ();

   prntStatusBMC ();

   /* try to restart if a rescue file is specified */
   k = 0;
   schnell_restart (likeratios, &k, loci[0].va, additive);

   for (; k < NTPos1; ++k)              /* THE BIG LOOP */
   {
      printf ("\n\nCHROMOSOME SET %d\n\n", k);
      fflush (stdout);
      /* two evaluation sets for each simulation chromo (left/right) */
      printf (" Recombinations:\n");
      for (sx = FEM; sx <= MAL; ++sx)
      {
         trait_loc0[0] = IntVal[k];
         trait_rec0[0] = RecVal[sx][k];
         if (k == 0)
         {
            trait_loc0[1] = NMarks;
            trait_rec0[1] = RecVal[sx][NTPos];
         }
         else
         {
            trait_loc0[1] = IntVal[k - 1];
            trait_rec0[1] = RecVal[sx][k - 1];
         }
         trait_loc0[2] = IntVal[k + 1];
         trait_rec0[2] = RecVal[sx][k + 1];

         set_chromosomes (NMarks, Marek[sx], 2, trait_loc0, trait_rec0, sx);

         set_lpflag (NMarks, 2, sx);

         for (i = 0; i <= 2; ++i)
         {
            if (i == 0)
               printf ("    sim,   ");
            else if (i == 1)
               printf ("    left,  ");
            else
               printf ("    right, ");
            if (sx == MAL)
               printf ("M:");
            if (sx == FEM)
               printf ("F:");
            for (j = 0; j < NMarks; ++j)
            {
               if (!(j % 6) && j)
                  printf ("\n             ");
               printf ("%3d%s", chromo[i][j], dub2nlz (xecomo[sx][i][j], 7, 3));
            }
            printf ("%3d\n", chromo[i][NMarks]);
         }
      }
      printf ("\n");
      fflush (stdout);

      /* L-sampler set-up */

      set_positions (NMarks);

      /* now should do an L-sampler step on the trait locus (0) to reset */
      /* lm_schnell no uses regular do_lsmpl: do_lsmpl_shnl eliminated   */
      /*                                             EAT 2004            */
      ddtmp = do_lsmpl (NMarks, 0, prob, nFGL);

      for (c = 0; c < n_compnts; ++c)
      {
         for (h = 0; h <= 2; ++h)
         {
            l_ratio[c][h] = 0.0;
            cumulik[c][h] = 0.0;
         }

         for (h = 0; h <= NMarks; ++h)
         {
            tot_log_prob_seg[c][h] = 0.0;
            tot_log_left_seg[c][h] = 0.0;
            tot_log_rght_seg[c][h] = 0.0;
            tot_log_prob_pen[c][h] = 0.0;
         }
         scounts[c] = 0;
         for (j = 0; j <= NMarks; ++j)
            for (jj = 0; jj < 6; ++jj)
               recounts[c][j][jj] = 0;
      }
      tot_count = 0;

      /* set initial l_ratio    */

      for (c = 0; c < n_compnts; ++c)
      {
         l_ratio[c][1] = 0.0;
         l_ratio[c][2] = l_ratio[c][1];

         for (h = 1; h <= 2; ++h)
         {
            for (j = 0; j < NMarks; ++j)
               lsflag[h][j] = 1;
            l_ratio[c][h] +=
               diff_seg_prob (NMarks, c, h, lsflag[h], temp_log_prob_seg);
         }
      }

      /* now do MCMC resampling by meiosis or by locus */
      for (t = 0; t < NIt; ++t)
      {
         if (rans () < LSamplProb)
         {
            /* Do L-sampling steps */
            if (!((t + 1) % ChkIt))
            {
               printf (" Starting L-sampler iteration %d\n", t + 1);
               fflush (stdout);
            }
            if (Scan)                   /* permute loci for L-sampler scan */
            {
               permute (perm_loc, NMarks + 1);
               for (j = 0; j <= NMarks; ++j)
               {
                  jj = perm_loc[j];

                  ddtmp = do_lsmpl (NMarks, jj, prob, nFGL);
               }
               for (h = 1; h <= 2; ++h)
                  for (j = 0; j <= NMarks; ++j)
                     lsflag[h][j] = 1;
               for (c = 0; c < n_compnts; ++c)
               {
                  l_ratio[c][1] = 0.0;
                  l_ratio[c][2] = l_ratio[c][1];

                  for (h = 1; h <= 2; ++h)
                  {
                     for (j = 0; j < NMarks; ++j)
                        lsflag[h][j] = 1;
                     l_ratio[c][h] +=
                        diff_seg_prob (NMarks, c, h, lsflag[h],
                                       temp_log_prob_seg);
                     cumulik[c][h] += exp (l_ratio[c][h]);
                  }
                  ++scounts[c];
               }
            }
            else
            {                           /* random locus, by name */
               jj = intran (0, NMarks);
               /* adjust adjacent intervals out of l-ratio */
               /* note: relevant intervals may depend on h */
               for (h = 1; h <= 2; ++h)
                  for (j = 0; j <= NMarks; ++j)
                     lsflag[h][j] = 0;

               for (h = 1; h <= 2; ++h)
               {
                  j = revchro[0][jj];   /* position under sim (0) */
                  if ((((j == 0) && (lpflag[1][h][j] != 0))
                       || (lpflag[0][h][j] != 0))
                      || (((j == NMarks) && (lpflag[1][h][j - 1] != 0))
                          || (lpflag[0][h][j - 1] != 0))
                      || ((lpflag[1][h][j] != 0) || (lpflag[0][h][j] != 0)
                          || (lpflag[1][h][j - 1] != 0)
                          || (lpflag[0][h][j - 1] != 0)))
                  {
                     if (j > 0)
                        lsflag[h][j - 1] = 1;
                     if (j < NMarks)
                        lsflag[h][j] = 1;
                     if (chromo[0][j] != chromo[h][j])
                     {
                        j = revchro[h][jj];
                        if (j > 0)
                           lsflag[h][j - 1] = 1;
                        if (j < NMarks)
                           lsflag[h][j] = 1;
                     }
                  }
                  for (c = 0; c < n_compnts; ++c)
                     l_ratio[c][h] -= diff_seg_prob (NMarks, c, h, lsflag[h],
                                                     temp_log_prob_seg);
               }
               /* Set up transmission probs */

               ddtmp = do_lsmpl (NMarks, jj, prob, nFGL);
               for (h = 1; h <= 2; ++h)
                  for (c = 0; c < n_compnts; ++c)
                     l_ratio[c][h] += diff_seg_prob (NMarks, c, h, lsflag[h],
                                                     temp_log_prob_seg);
               for (c = 0; c < n_compnts; ++c)
               {
                  for (h = 1; h <= 2; ++h)
                     cumulik[c][h] += exp (l_ratio[c][h]);
                  ++scounts[c];
               }
            }
         }
         else
         {                              /* ***    Do M-sample step    *** */
            if (!((t + 1) % ChkIt))
            {
               printf (" Starting M-sampler iteration %d\n", t + 1);
               fflush (stdout);
            }
            if (Scan)                   /* permute meioses for M-sampler scan */
            {
               permute (perm_sw, nsw);
               for (j = 0; j < nsw; ++j)
               {
                  l = perm_sw[j] + 1;

                  i = (meiosis[l].ind).index;
                  sx = meiosis[l].sex;
                  c = nghd[i].component - 1;

                  do_msmpl (NMarks, 0, c, i, sx, prob, nFGL);
               }
               for (h = 1; h <= 2; ++h)
                  for (j = 0; j <= NMarks; ++j)
                     lsflag[h][j] = 1;
               for (c = 0; c < n_compnts; ++c)
               {
                  l_ratio[c][1] = 0.0;
                  l_ratio[c][2] = l_ratio[c][1];

                  for (h = 1; h <= 2; ++h)
                  {
                     for (j = 0; j < NMarks; ++j)
                        lsflag[h][j] = 1;
                     l_ratio[c][h] += diff_seg_prob (NMarks, c, h, lsflag[h],
                                                     temp_log_prob_seg);
                     cumulik[c][h] += exp (l_ratio[c][h]);
                  }
                  ++scounts[c];
               }
            }
            else
            {                           /*choose a random meiosis */
               l = intran (0, nsw - 1) + 1;
               i = (meiosis[l].ind).index;
               sx = meiosis[l].sex;
               c = nghd[i].component - 1;

               /* adjust meiosis to be updated out of l_ratio */

               for (h = 1; h <= 2; ++h)
                  l_ratio[c][h] -= lik_rat (NMarks, i, sx, h);

               /*  do-msmpl */
               do_msmpl (NMarks, 0, c, i, sx, prob, nFGL);

               /* put updated meiosis back in to l_ratio and
                * update the cumulative ratio                */

               for (h = 1; h <= 2; ++h)
               {
                  l_ratio[c][h] += lik_rat (NMarks, i, sx, h);
                  cumulik[c][h] += exp (l_ratio[c][h]);
               }
               ++scounts[c];
            }
         }
         if (loci[0].va > 0)
            smpl_a (loci, NMarks + 1, loci[0].ve, loci[0].va, mu, np, additive);

         if (!((t + 1) % ScorIt))
         {                              /* score tot log-probs & recombs */
            for (c = 0; c < n_compnts; ++c)
            {
               for (j = 0; j <= NMarks; ++j)
                  tot_log_prob_pen[c][j] += log (prob[c][j]);

               full_seg_prob (NMarks, c, recounts, temp_log_prob_seg);
               for (j = 0; j < NMarks; ++j)
                  tot_log_prob_seg[c][j] += temp_log_prob_seg[c][j];

               for (h = 1; h <= 2; ++h)
                  for (j = 0; j < NMarks; ++j)
                     lsflag[h][j] = 1;

               (void) diff_seg_prob (NMarks, c, 1, lsflag[1],
                                     temp2_log_prob_seg);
               for (j = 0; j < NMarks; ++j)
                  tot_log_left_seg[c][j] += temp2_log_prob_seg[c][j] +
                     temp_log_prob_seg[c][j];

               (void) diff_seg_prob (NMarks, c, 2, lsflag[2],
                                     temp2_log_prob_seg);
               for (j = 0; j < NMarks; ++j)
                  tot_log_rght_seg[c][j] += temp2_log_prob_seg[c][j] +
                     temp_log_prob_seg[c][j];
            }
            ++tot_count;
         }
      }
      /* now we are done with this simulation/estimation for this trait locus
         position  */
      /* output results for this simulation/estimation set */

      for (c = 0; c < n_compnts; ++c)
      {
         printf ("\n Component %d:\n\n", c + 1);
         if (scounts[c] == 0)
            printf ("    No likelihood ratios because sampling count = 0\n\n");
         else
         {
            leftratio = cumulik[c][1] / scounts[c];
            rightratio = cumulik[c][2] / scounts[c];
            printf ("    Cumulative likelihood ratios (sampling count %d)\n",
                    scounts[c]);
            printf ("       estimated ratios:   left:sim = %6.4f,"
                    "  right:sim = %6.4f\n\n", leftratio, rightratio);
         }
         /* Fill in the matrices of likehood ratios */
         /* initialization first */

         for (j = 0; j < NTPos1; ++j)
            likeratios[c][k][j] = 0.0;
         if (k == 0)
         {
            likeratios[c][0][NTPos] = leftratio;
            likeratios[c][0][1] = rightratio;
         }
         else if (k == NTPos)
         {
            likeratios[c][k][k - 1] = leftratio;
            likeratios[c][k][0] = rightratio;
         }
         else
         {
            likeratios[c][k][k - 1] = leftratio;
            likeratios[c][k][k + 1] = rightratio;
         }

         printf ("    Average total log-probability of segregations:\n");
         printf ("       first locus (constant) %8.3f, and by map interval:"
                 "\n", -cmplist[c + 1].nmeioses * log (2.0));
         printf ("       interval        left          sim       right\n");
         for (j = 0; j < NMarks; ++j)
            printf ("            %3d    %8.3f     %8.3f    %8.3f\n",
                    j + 1, tot_log_left_seg[c][j] / tot_count,
                    tot_log_prob_seg[c][j] / tot_count,
                    tot_log_rght_seg[c][j] / tot_count);
         printf ("\n");
         printf ("    Average total log-probability of penetrances,"
                 " by locus:");
         for (j = 0; j <= NMarks; ++j)
         {
            if (!(j % 8))
               printf ("\n      ");
            if (chromo[0][j] == 0)
               printf ("  trait  ");
            else
               printf (" mark%2d  ", chromo[0][j]);
         }
         for (j = 0; j <= NMarks; ++j)
         {
            if (!(j % 8))
               printf ("\n    ");
            printf (" %8.3f", tot_log_prob_pen[c][chromo[0][j]] / tot_count);
         }
         printf ("\n\n");
         fflush (stdout);

         /* print out recombination counts  */
         printf ("    Recombination counts for map intervals:\n");
         printf ("       intv   M: total    recomb   non-rec "
                 "  F: total    recomb   non-rec\n");
         for (j = 0; j < NMarks; ++j)
         {
            printf ("       %4d ", j + 1);
            for (i = 0; i < 6; ++i)
            {
               if (i == 3)
                  printf (" ");
               printf ("%10d", recounts[c][j][i]);
            }
            printf ("\n");
         }
         printf ("\n");
         fflush (stdout);
      }

      /* dumping restart information */
      schnell_dump (likeratios, k, loci[0].va, additive);
   }

   /* no need for dumping any more */
   finish_dumping ();

   prntStatusFMC ();

   /* now we are done with simulating for all NMarks+2 simulation sets */
   print_lods (likeratios);

   /* {{{ Clean up */
   if (SamplSeedsOut)
   {
      getSeeds (SAMPL);
      if (!(oSeedF = open_fil (OSEED)))
         give_up (ERR_qit);
      writSeeds (oSeedF);
      fclose (oSeedF);
   }

   free_3d ((void *) &likeratios, n_compnts, NTPos1);

   /* free_schnell (NMarks); */
   free_loglik (&recounts, &tot_log_prob_seg, &temp_log_prob_seg,
                &tot_log_prob_pen);
   free_lod (&scounts, &lsflag, &l_ratio, &cumulik,
             &temp2_log_prob_seg, &tot_log_left_seg, &tot_log_rght_seg);
   free_lpflag (2);
   free_logprob ();
   free_msmpl ();
   free_gen_pen ();
   free_seg_pen (NMarks + 1, &prob);

   /* Free up memory used in peeling routines etc. */
   if (additive)
      free (additive);
   if (perm_sw)
      free (perm_sw);
   if (perm_loc)
      free (perm_loc);

   /* }}} */
   return EXIT_SUCCESS;
}

/* {{{ Log */

/*
 * $Log: lm_schnell.c,v $
 * Revision 1.15  2005/08/14 00:20:32  thompson
 * Adding copyright statement at file head.
 *
 * Revision 1.14  2005/07/15 22:59:00  thompson
 * setupInitInh() call modified is all main programs, to add new flag.
 * lm_markers and lm_multiple modified to not output seeds twice.
 *    lm_markers and lm_multiple have new flag set =1 (golds will change)
 *
 * Revision 1.13  2005/04/22 18:03:52  myrna
 * Setup functions for Autozyg are revised.  setupAutozyg() and setupInherit()
 * are now the same for Autozyg and Lodscore programs.  Functions they call
 * moved to appropriate directories.
 *
 * Revision 1.12  2005/03/28 06:34:45  thompson
 * burn-in iteration counter (arg to sampling()) corrected in lm_markers.
 * do_msmpl() calls in lm_schnell updated for new extra arg2
 *
 * Revision 1.11  2005/03/16 07:32:34  myrna
 * README_newSetup contains pseudo-code for the new setupLodSc() function.
 * This function call replaces set-up code in the lodscore programs.
 * In addition, some redundant variables are replaced, and some function
 * calls are changed.
 *
 * Revision 1.10  2005/01/29 00:39:57  myrna
 * Function calls set_traits() and allocate_loci() no longer include
 * references to the pedigree file.
 *
 * Revision 1.9  2004/11/29 01:23:32  thompson
 * In all programs/routines/headers:
 *   Fix calls to list_switches.
 *   Remove use of old nns array, and old component list structures.
 * Gold2 gold standards checked (all.gold and lm_schnell.gold): all ok
 *
 * Revision 1.8  2004/09/24 22:50:36  myrna
 * Macro constants are renamed.
 *
 * Revision 1.7  2004/09/17 18:51:35  thompson
 * Genotype array y fully eliminated from all routines.
 * Array y removed, and alloc_y and free_y calls removed.
 *
 * Revision 1.6  2004/09/15 04:56:51  thompson
 * Genotype array y removed from alloc_seg_pen, seg_pen2, seg_pen, gen_pen
 *
 * Revision 1.5  2004/09/14 04:26:29  thompson
 * Calls to set_lsmpl() and seq_imp updated to remove genotypic data array y.
 * lm_markers calls to set_lsmpl cleaned up -- need not depend on TraitTyp
 * lm_schnell modified to use set_lsmpl()-- long overdue clean-up
 *
 * Revision 1.4  2004/09/08 19:22:36  myrna
 * Lodscore/ main programs now "#include lodscore.h" for their header files.
 *
 * Revision 1.3  2004/08/20 02:59:24  thompson
 * Schnell is updated to use new nghd trait-data structures, and get rid
 * of old arrays (qdex, ym, x, xtrait, etc.).
 * ALSO IMPORTANT NOTE:
 * schnell was WRONG: failing to sample polygenic values of unobserved individuals
 *           -- a substantive logic bug is fixed in smpl_a in mixd2.c
 *           -- old gold standard checked out (pre bug fix). New golds committed.
 * NOTE ALSO:
 * schnell is still BAD:  polygenic updates are still by single-site updating
 *       -- needs to be fixed (one day) by using polypeel to peel and resample.
 *
 * Revision 1.2  2004/07/06 07:08:18  thompson
 * Major changes in lm_markers for getting lod scores by component
 *   (option not fully implemented yet)
 * Change to locus_mjrpeel call in lm_schnell
 *
 * Revision 1.1  2004/04/22 23:17:50  myrna
 * lm_schnell is added to Lodscores/ directory
 *
 * Revision 1.20  2004/04/20 23:23:38  myrna
 * This version of schnell is caught up to Morgan V2.6+quantitative traits.
 * Setup is now done in proc_auto().  Current versions of Morgan globals are
 * now used.
 *
 * Revision 1.19  2004/01/09 08:06:22  thompson
 * schnell now uses alloc_y()  in Markers, as Autozyg programs,
 * instead of alloc_int3d() in stuff
 *
 * Revision 1.18  2004/01/06 01:37:44  thompson
 * schnell now uses Markers/set_markdat() and shnl_set_markdat code is
 * removed from set_schnell.c  (Code of set_markdat and shnl_set_markdat show
 * null diff: but full test not possible as schnell not yet functional in 2.6)
 *
 * Revision 1.17  2004/01/03 19:50:49  thompson
 * Updated schnell.c for new allocated_loci
 *
 * Revision 1.16  2003/07/31 20:12:23  nali
 * Updated some doxygen comments to make the grouping in 'modules' page
 * more useful.
 *
 * Revision 1.15  2003/07/03 04:36:27  thompson
 * schnell now uses allocate_loci() to set locus structures.
 *
 * Revision 1.14  2003/06/25 06:37:23  thompson
 * pfile replaceed by pedF to standardize. New allocate_loci call implemented.
 * The gold standard is checked.
 *
 * Revision 1.13  2003/06/24 02:21:32  thompson
 * Call to seq_imp() has been updated: Gold standard is checked.
 *
 * Revision 1.12  2002/08/29 21:48:36  nali
 * doxyenizing...
 *
 * Revision 1.11  2002/08/02 18:56:09  myrna
 * Global variables are revised:
 *   NSEQIMP --> NSeqImp
 *   WIt     --> ChkIt
 *   ScIt    --> ScorIt
 *
 * Revision 1.10  2002/05/06 20:00:20  nali
 * fixed a typo SEED -> OSEED
 *
 * Revision 1.9  2002/05/02 18:45:00  myrna
 * allocate() and free_allocate() are replaced with alloc_xxgg() and
 * free_xxgg()
 *
 * Revision 1.8  2002/04/29 23:04:23  nali
 *
 * 1.  New function 'map_position (pos, sex)' gives the map position
 *     (in cM) of the varying trait pos relative to the first marker
 *     (computed using IntVal, RecVal, Marek and rf2dist ()).
 * 2.  New function 'print_lods (likeratios)' computes and prints out
 *     the lod scores together with male and female map positions to
 *     make it easier for plotting.
 *
 * These functions can be used by lm_lods as well.
 *
 * Revision 1.7  2002/04/26 23:51:46  nali
 * M-sampler added to Schnell
 *
 * Revision 1.6  2002/04/19 23:50:43  nali
 * corrected data_index for markers
 *
 * Revision 1.5  2002/04/16 19:08:01  nali
 * remove assignment prob[c][j] = 1.0
 *
 * Revision 1.4  2002/03/15 19:19:39  nali
 * 1.  Uncommented calls to gen_pen2.
 *
 * Revision 1.3  2002/03/07 21:13:29  myrna
 * free_gen_dat() is renamed to free_allefrq() in set_shnl.c
 * C++ style comment delimiter is changed to C style in schnell so program will
 * compile with "cc".  #ifdef DMALLOC" removed so can "make shnell.cc.dml"
 *
 * Revision 1.2  2002/03/06 21:35:57  myrna
 * Call to alloc_chromo() is added
 * Calls to set_chromosomes() and set_lpflag() are revised for gender subscript
 * References to xecomo[][][] and lpflag[][][] have gender subscript changed
 *
 * Revision 1.1  2002/02/17 18:56:54  nali
 *
 * Renamed file.  Commented out computation of log Pr(penetrance).
 *
 * Revision 1.12  2002/02/13 06:43:38  thompson
 * Updated alloc_routines in schnell.c, and founder checking in mixd2.c
 *
 * Revision 1.11  2002/02/06 05:28:50  nali
 * Commented out calls to alloc_gen_pen () that might cause
 * segfault.
 *
 * Revision 1.10  2002/02/05 22:30:04  nali
 * Schnell has been brought up-to-date to fully use
 * MORGAN statements and character names, etc..
 *
 * To Do:
 * 1.  Checking results, comparing with Vitesse.
 * 2.  Fix restart problem
 * 3.  How to print diagnosis info?
 * 4.  Can't compute Pr (penetrance) yet.
 *
 * Revision 1.9  2002/01/28 18:38:52  nali
 *
 * Broke Schnell to use the same parameter file as lm_lods.
 * Some wrong in the set up (segdump in locus_peel_init).
 *
 * Revision 1.8.2.1  2001/11/28 10:09:22  nali
 *
 * Set up to use Morgan statement and command line option parsers so
 * that pedchk is called to set up the key table for use with char
 * names.
 *
 * There is still a malloc bug (?).
 *
 * Revision 1.8  2001/10/12 21:54:15  nali
 * Updated to use comput_lods ()
 *
 * Revision 1.7  2001/09/13 20:00:38  nali
 *
 * 1.  Call 'free_allocate' corresponding to 'allocate_loci'.
 * 2.  Remove schnell
 * 3.  Remove file scope variable 'iobs2' (read_gen_dat2.c) and local nobs
 * (read_gen_dat()).  Use global varibales NObs, and IObs[NOBS]
 * 4.  Call allocate_loci() then do necessary post-processing.  Move
 * allocate()instead.variable)global variable 'gene_idx' which doesn't seem
 * 5.  Introduce a set of functions for allocating arrays, and use them.into
 * read_gen_dat2().)to be used.
 *
 * Revision 1.6  2001/08/29 22:04:01  nali
 * Bug fixed in computing "complete data loglikelihood".
 *
 * Revision 1.5  2001/08/16 19:30:50  nali
 * Added "#define PNAM" so will compile with current headers files.
 *
 * Revision 1.4  2001/07/31 05:30:58  nali
 * Major changes include:
 *
 * 1.  Now use seq_imp(), original method still available by undefining
 * USE_OLD.
 * 2.  Added test data file, schtest.dat.
 * 3.  Can dump debug info every n iteration on request.
 * 4.  Has burn in.
 * 5.  By default, dump checkpoint files after each trait position and is able
 *     to restart (though not able to get the same result as complete run).
 * 6.  Other clean ups.
 *
 * Works with additive genetic component (not crashing).
 *
 * To do: Fix the restart feature.
 *
 */

/* }}} */

void free_schnell (int NMarks)
{
   free_allocate_loci ();
   free_xxgg ();

   /* memory allocated in input_data */
   free_input_data ();

   /* memory allocated in proc_drop */
   if (PTraitPars)
      free (PTraitPars);
   if (PTraitOut)
      free (PTraitOut);

   /* memory allocated in proc_auto */
   free_auto ();

   free_switches ();
   free_marek ();
   locus_peel_free ();
   free_allefrq ();
   free_fnams ();
   freeChkPed ();
   freeKey ();
   free_cmplists ();
   free_nghds ();
   free_ped ();
   free_dub ();
}
