/* MORGAN: Copyright 1991-2005: Univ. of Washington. All rights reserved. */
/* You may copy, use, modify, but NOT redistribute, modified or otherwise.*/
/* See: http://www.stat.washington.edu/thompson/Genepi/license.shtml      */

#include "markers.h"

/* New file to contain all the ibd-measure routines, all called with
 *  same format, and of type Measure.
 * Each function computes an IBD score on component c+1, at locus jj.
 *       First one (max_fgl), rewritten  in Measure format 01/2005, EAT
 *
 *  Saonli moved other measures here, Spring 2005
 *        spair(), robdom(), Saffunaff(), 
 *         Calculate_Likelihood_Lambda()
 *   and supporting routines
 *     calculation (), initialize_flagTable_Lambda (), Weight_Calculation()
 *     Relation(), final_Calculation(),
 *     Peeling_routine_Lambda(), CalPValue(), Weight_Multilambda()
 *  File (and markers.h header) tidied up,  EAT June 2005
 *
 *  For spairs and robdom : Indicator = exreal[0] now indicates whether
 *      cmplist setup is to be used  and
 *  for robdom Base = exreal[1] is the base for robdom powering (=7)
 *     EAT July 2005.
 *  (there is still a hard-wired array in robdom ??).
 */

void  Calculate_Score_Lambda (int comp, Ind * temp, int allele_group[4],
                              int *trt_data, int locus, double *Weight);
void  Peeling_routine_Lambda (int comp_no, Ind * First_Element, int *trt_data,
                              int locus, double *Weight);
void  calculation (int *shipaff, int *shipunaff, int size1, int size2, int
                   loci, int distallele, int zn, int c, int *CountAlleleAff,
                   int *CountAlleleUnAff);

int  *flagTable = NULL;
int  *Relation_Array = NULL;
int  *Tag = NULL;
int **FlagLambda = NULL;
int  *flagTable_Lambda = NULL;
double **Count_Matrix_Lambda = NULL;
double final_Calculation (int SEG_LAMBDA, int Size, int *Param_Array,
                          double *param_val_array_Lambda);

double max_fgl (int c, int jj, int *exint, double *exreal)
                                  /* scores ibd on component c, locus jj */
  /*     for each locus, counts the no. of inheritances of ea label by
   *                                      affected individuals.
   *   NOTE In current version, individual counts only 1 if both
   *          both copies are of the given label : 
   *   alternatively, such an  an individual could count 2, if
   *   if (gg0[jj][cmplist[c+1].ind_aff[i]] == n + k + 1)
   *      ++sumgene[c][jj][k];
   *     if (gg1[jj][cmplist[c+1].ind_aff[i]] == n + k + 1)
   *     ++sumgene[c][jj][k];
   *    (n = fgldex[c] )
   *  (Note this changes the range of histogram values)
   */
{
   int   i, k, mx;

   for (k = 0; k < 2 * cmplist[c + 1].nfou; ++k)
      exint[k] = 0;

   for (i = 0; i < cmplist[c + 1].naff; ++i)
      for (k = 0; k < 2 * cmplist[c + 1].nfou; ++k)
         if ((gg0[jj][cmplist[c + 1].ind_aff[i]] == k + 1)
             || (gg1[jj][cmplist[c + 1].ind_aff[i]] == k + 1))
            ++exint[k];

   mx = 0;
   for (k = 0; k < 2 * cmplist[c + 1].nfou; ++k)
      if (exint[k] > mx)
         mx = exint[k];

   return (mx);
}

double spair (int c, int jj, int *exint, double *exreal)
{
/*  ibd measure to score pair-wise ibd sharing for component c and 
 *  locus jj. For each realization of inheritance vector, the measure 
 *  calculates the extent of ibd sharing between each par of affected 
 *                                         individuals. */
   int   i, j, temp1, temp2;
   double stat1 = 0;
   double sum1 = 0;
   int   n_a;
   int   loci = jj;
   int  *vec1 = NULL;
   double Indicator = exreal[0];
   int **allele = NULL;
   if (Indicator)
   {
      n_a = exint[0];
      allele = (int **) calloc (n_a, sizeof (int *));
      alloc_chk (WHERE, allele);

      vec1 = (int *) calloc (n_a, sizeof (int));
      for (i = 0; i < n_a; i++)
      {
         vec1[i] = exint[i + 2];
      }
   }
   if (!Indicator)
   {
      n_a = cmplist[c + 1].naff;
      allele = (int **) calloc (n_a, sizeof (int *));
      alloc_chk (WHERE, allele);

      vec1 = (int *) calloc (n_a, sizeof (int));
      for (i = 0; i < n_a; i++)
      {
         vec1[i] = cmplist[c + 1].ind_aff[i];
      }
   }

   for (i = 0; i < n_a; i++)
   {
      allele[i] = (int *) calloc (2, sizeof (int));
      alloc_chk (WHERE, allele[i]);
      temp1 = gg0[loci][*(vec1 + i)];
      temp2 = gg1[loci][*(vec1 + i)];
      allele[i][0] = temp1;
      allele[i][1] = temp2;

   }
   sum1 = 0;

   for (i = 0; i < (n_a - 1); i++)
   {
      for (j = (i + 1); j < n_a; j++)
      {
         stat1 = 0;
         if (allele[i][0] == allele[j][0])
            stat1 = stat1 + 1.0;

         if (allele[i][0] == allele[j][1])
            stat1 = stat1 + 1.0;

         if (allele[i][1] == allele[j][0])
            stat1 = stat1 + 1.0;
         if (allele[i][1] == allele[j][1])
            stat1 = stat1 + 1.0;
         sum1 = sum1 + stat1;
      }

   }
   if (allele)
   {
      for (i = 0; i < n_a; i++)
      {
         if (allele[i])
            free (allele[i]);
      }
      free (allele);
      allele = NULL;
   }
   if (vec1)
   {
      free (vec1);
   }
   return sum1;
}

double robdom (int c, int jj, int *exint, double *exreal)
/*** Calculation of S_{rob dom} ***/
{
/* This ibd measure scores joint ibd sharing among a set of affected 
 * individuals in component c for locus jj. For every realization of 
 * inheritance vector, the ibd measure first computes the number of affected 
 * individuals(a_j) in component c sharing each founder genome label j. The 
 *   final measure is sum_j(7^(a_j -1). */
   int   i, j, k = 0, k1 = 0, l = 0, temp1, temp2;
   int   temp8[40];
   int   tag = 0, count, n_a;
   double robdom;
   int  *vec1 = NULL;
   int **allele = NULL;
   double Indicator = exreal[0];
   double Base = exreal[1];
   int   loci = jj;
   if (Indicator)
   {
      n_a = exint[0];
      vec1 = (int *) calloc (n_a, sizeof (int));
      for (i = 0; i < n_a; i++)
      {
         vec1[i] = exint[i + 2];
      }
   }

   if (!Indicator)
   {
      n_a = cmplist[c + 1].naff;
      vec1 = (int *) calloc (n_a, sizeof (int));
      for (i = 0; i < n_a; i++)
      {
         vec1[i] = cmplist[c + 1].ind_aff[i];
      }
   }

   allele = (int **) calloc (n_a, sizeof (int *));
   alloc_chk (WHERE, allele);
   for (i = 0; i < n_a; i++)
   {
      allele[i] = (int *) calloc (2, sizeof (int));
      alloc_chk (WHERE, allele[i]);
      temp1 = gg0[loci][vec1[i]];
      temp2 = gg1[loci][vec1[i]];
      allele[i][0] = temp1;
      allele[i][1] = temp2;
   }
   if (n_a > 0)
   {
      temp8[0] = allele[0][0];

      tag = 0;
      k1 = 1;
   }
   for (i = 1; i < n_a; i++)
   {
      tag = 0;
      for (k = 0; k < k1; k++)
      {
         if (allele[i][0] == temp8[k])
            tag = 1;
      }
      if (tag == 0)
      {
         temp8[k1] = allele[i][0];
         k1 = k1 + 1;
      }
   }
   if (n_a > 0)
      l = k1;
   for (j = 0; j < n_a; j++)
   {
      tag = 0;
      for (k = 0; k < l; k++)
      {
         if (allele[j][1] == temp8[k])
            tag = 1;
      }
      if (tag == 0)
      {
         temp8[l] = allele[j][1];
         l = l + 1;
      }
   }
   robdom = 0;
   for (j = 0; j < l; j++)
   {
      count = 0;
      for (i = 0; i < n_a; i++)
      {
         if ((allele[i][0] == temp8[j]) || (allele[i][1] == temp8[j]))
            count = count + 1;
      }
      robdom = robdom + pow (Base, (double) (count - 1));

   }
   for (i = 0; i < n_a; i++)
   {
      if (allele[i])
         free (allele[i]);
   }
   if (allele)
      free (allele);
   allele = NULL;
   if (vec1)
   {
      free (vec1);
   }

   if (robdom < 0)
   {
      return 0;
   }
   else
      return robdom;
}

double Saffunaff (int c, int jj, int *exint, double *extreal)
{                                       /* This ibd measure scores extent of ibd sharing among affected 
                                           individuals. The measure also incorporates the lack of ibd sharing between 
                                           affected and unaffected within each sibship of component c for locus j 
                                           for each realization of inheritance vector. */
   double newstat = 0;

   int   i, j, zn, countv = 0;
   int **sibship_aff = NULL;
   int **sibship_unaff = NULL;
   int  *CountAlleleAff = NULL;
   int  *CountAlleleUnAff = NULL;
   int  *Group_Sizeaff = NULL;
   int  *Group_Sizeunaff = NULL;
   int   Group = exint[0];
   int   locus = jj;
   if (Group > 0)
   {
      sibship_aff = (int **) calloc (Group, sizeof (int *));
      alloc_chk (WHERE, sibship_aff);
      sibship_unaff = (int **) calloc (Group, sizeof (int *));
      alloc_chk (WHERE, sibship_unaff);
      Group_Sizeaff = (int *) calloc (Group, sizeof (int));
      alloc_chk (WHERE, Group_Sizeaff);
      Group_Sizeunaff = (int *) calloc (Group, sizeof (int));
      alloc_chk (WHERE, Group_Sizeunaff);
      for (i = 0; i < Group; i++)
      {
         Group_Sizeaff[i] = exint[i + 1];
      }
      for (i = 0; i < Group; i++)
      {
         Group_Sizeunaff[i] = exint[Group + i + 1];
      }
      for (i = 0; i < Group; i++)
      {

         sibship_aff[i] = (int *) calloc (Group_Sizeaff[i], sizeof (int));
         alloc_chk (WHERE, sibship_aff[i]);
         sibship_unaff[i] = (int *) calloc (Group_Sizeunaff[i], sizeof (int));
         alloc_chk (WHERE, sibship_unaff[i]);

      }
   }

   CountAlleleAff = (int *) calloc (2 * cmplist[c + 1].nfou, sizeof (int));
   alloc_chk (WHERE, CountAlleleAff);

   CountAlleleUnAff = (int *) calloc (2 * cmplist[c + 1].nfou, sizeof (int));
   alloc_chk (WHERE, CountAlleleUnAff);

   countv = 0;

   for (i = 0; i < Group; i++)
   {
      for (j = 0; j < Group_Sizeaff[i]; j++)
      {
         sibship_aff[i][j] = exint[(2 * Group + 1) + countv];

         countv = countv + 1;
      }
   }

   for (i = 0; i < Group; i++)
   {
      for (j = 0; j < Group_Sizeunaff[i]; j++)
      {
         sibship_unaff[i][j] = exint[(2 * Group + 1) + countv + 1];
         countv = countv + 1;
      }

   }

   for (zn = 0; zn < 2 * cmplist[c + 1].nfou; zn++)
   {
      double Value = 0;
      CountAlleleAff[zn] = 0;

      CountAlleleUnAff[zn] = 0;
      for (i = 0; i < Group; i++)
      {
         int   a1 = 0;
         for (j = 0; j < Group_Sizeaff[i]; j++)
         {
            if (((gg0[locus][sibship_aff[i][j]] ==
                  zn + 1)
                 || (gg1[locus][sibship_aff[i][j]] == zn + 1)) && (a1 == 0))
            {
               calculation (sibship_aff[i],
                            sibship_unaff[i],
                            Group_Sizeaff[i], Group_Sizeunaff[i],
                            locus,
                            zn + 1, zn, c, CountAlleleAff, CountAlleleUnAff);
               a1 = 1;
            }
         }
      }
      Value =
         (double) (CountAlleleAff[zn] * pow (2, CountAlleleAff[zn])) +
         (double) (CountAlleleUnAff[zn] * pow (2, CountAlleleUnAff[zn]));
      newstat = newstat + Value;

   }
   if (Group != 0)
   {
      if (sibship_aff != NULL)
      {
         for (j = 0; j < Group; j++)
         {
            if (sibship_aff[j] != NULL)
            {
               free (sibship_aff[j]);
            }
         }
         free (sibship_aff);
      }
      if (sibship_unaff != NULL)
      {
         for (j = 0; j < Group; j++)
         {
            if (sibship_unaff[j] != NULL)
            {
               free (sibship_unaff[j]);
            }
         }
         free (sibship_unaff);
      }
   }
   free (CountAlleleAff);
   free (CountAlleleUnAff);
   free (Group_Sizeaff);
   free (Group_Sizeunaff);
   return newstat;
}

void calculation (int *shipaff, int *shipunaff, int size1, int size2, int
                  loci, int distallele, int zn, int c, int *CountAlleleAff,
                  int *CountAlleleUnAff)
{
   double newcountaff1 = 0;
   double newcount1 = 0;
   int   v, v1;
   int   i;
   if (size1 != 0)
   {
      for (i = 0; i < size1; i++)
      {
         v = *(shipaff + i);

         if ((gg0[loci][v] == distallele) || (gg1[loci][v] == distallele))

            newcountaff1 = newcountaff1 + 1.0;
      }
   }
   if (size1 != 0)
   {

      CountAlleleAff[zn] = CountAlleleAff[zn] + (int) newcountaff1;
   }
   if (size2 != 0)
   {
      for (i = 0; i < size2; i++)
      {
         v1 = *(shipunaff + i);
         if ((gg0[loci][v1] == distallele) || (gg1[loci][v1] == distallele))
            newcount1 = newcount1 + 1.0;

      }
   }

   if (size2 != 0)
   {
      CountAlleleUnAff[zn] = CountAlleleUnAff[zn] + (size2 - (int) newcount1);
   }

   return;

}

void initialize_flagTable_Lambda ()
{
   int   i;

   flagTable_Lambda = (int *) calloc (pedsize, sizeof (int));
   alloc_chk (WHERE, flagTable_Lambda);

   for (i = 0; i < pedsize; i++)
   {
      flagTable_Lambda[i] = 0;
   }

}
void Weight_Calculation (Ind * Element, Vnode ** Weight)
{
   Indlist *kidsPtr = NULL;
   Mar  *marCurrentPtr = NULL;
   Marlist *marListPtr = NULL;
   Ind  *kidsCurrentPtr = NULL;

   Vnode *HeadPtr = *Weight;
   if (Element == NULL)
   {
      printf ("The pedigree element is NULL\n");
      return;
   }
   else if ((Element->marriages == NULL))
   {
      HeadPtr->count_aff = HeadPtr->count_aff + 1;
      HeadPtr->count_unaff = HeadPtr->count_unaff + 1;

   }
   else if (Element->marriages != NULL)
   {
      Vnode *temp = NULL;
      marListPtr = Element->marriages;
      while (marListPtr != NULL)
      {
         marCurrentPtr = marListPtr->current;
         if (marCurrentPtr == NULL)
         {
            printf ("There is some error\n");
         }
         else
         {

            kidsPtr = marCurrentPtr->kids;
            while (kidsPtr != NULL)
            {

               kidsCurrentPtr = kidsPtr->current;
               if (kidsCurrentPtr->d_dat != NULL)
               {
                  if (kidsCurrentPtr->d_dat[0] == 2)
                  {
                     HeadPtr->count_aff = HeadPtr->count_aff + 1;
                  }
                  else if (kidsCurrentPtr->d_dat[0] == 1)
                  {
                     HeadPtr->count_unaff = HeadPtr->count_unaff + 1;
                  }
               }

               else if (kidsCurrentPtr->d_dat == NULL)
               {

                  if (temp == NULL)
                  {
                     temp = (Vnode *) calloc (1, sizeof (Vnode));
                     alloc_chk (WHERE, temp);
                     temp->level = HeadPtr->level + 1;

                  }
                  Weight_Calculation (kidsCurrentPtr, &temp);
               }
               kidsPtr = kidsPtr->next;
            }

         }

         marListPtr = marListPtr->next;

      }
      if (temp != NULL)
      {
         temp->next = HeadPtr;
         HeadPtr = temp;
      }
   }
   *Weight = HeadPtr;
   return;
}
void Relation (int i, int nfou, int original, int *cnt)
{
   int   j = 0;
   while (j < (2 * nfou))
   {

      if (Relation_Array[j] == 1)
      {
         j = j + 1;
         continue;

      }
      else if (((j != original) && (Count_Matrix_Lambda[i][j] > 0))
               || ((Count_Matrix_Lambda[j][i] > 0) && (FlagLambda[i][j] == 0)))
      {
         FlagLambda[i][j] = -1;
         FlagLambda[j][i] = -1;
         Relation_Array[j] = 1;
         *cnt = *cnt + 1;
         Tag[j] = *cnt;

         Relation (j, nfou, original, cnt);
      }
      j = j + 1;
   }

}

double Calculate_Likelihood_Lambda (int c, int locus, int *exint,
                                    double *exreal)
{

   int   i, j, k;
   double likelihood = 0;
   double logfinalLikelihood = 0;
   double MaxValue = 0;
   int  *CntArray = NULL;
   int  *lambda_array = NULL;
   Ind  *First_Element = NULL;
   int  *trt_data = NULL;
   double *Weight = NULL;
   int   SEG_LAMBDA = 0;
   double Indicator = exreal[0];
   double *param_val_array_Lambda = NULL;
   Weight = (double *) calloc (pedsize, sizeof (double));
   alloc_chk (WHERE, Weight);
   SEG_LAMBDA = exreal[cmplist[c + 1].size] + 1;
   param_val_array_Lambda = (double *) calloc (SEG_LAMBDA, sizeof (double));
   for (i = 0; i < SEG_LAMBDA; i++)
   {
      param_val_array_Lambda[i] = exreal[cmplist[c + 1].size + i + 2];
   }

   trt_data = (int *) calloc (pedsize, sizeof (int));
   alloc_chk (WHERE, trt_data);
   if (!Indicator)
   {
      for (i = 0; i < cmplist[c + 1].naff; i++)
      {
         trt_data[cmplist[c + 1].ind_aff[i]] = 2;
      }
      for (i = 0; i < cmplist[c + 1].nunaff; i++)
      {
         trt_data[cmplist[c + 1].ind_unaff[i]] = 1;
      }
   }
   if (Indicator)
   {
      for (i = 0; i < exint[0]; i++)
      {
         trt_data[exint[i + 2]] = 2;
      }
      for (i = 0; i < exint[1]; i++)
      {
         trt_data[exint[exint[0] + i + 2]] = 1;
      }
   }

   initialize_flagTable_Lambda ();

   Relation_Array = (int *) calloc (2 * cmplist[c + 1].nfou, sizeof (int));
   alloc_chk (WHERE, Relation_Array);
   Tag = (int *) calloc (2 * cmplist[c + 1].nfou, sizeof (int));
   alloc_chk (WHERE, Tag);

   Count_Matrix_Lambda = (double **) calloc (2 * cmplist[c + 1].nfou, sizeof
                                             (double *));
   alloc_chk (WHERE, Count_Matrix_Lambda);

   for (i = 0; i < (2 * cmplist[c + 1].nfou); i++)
   {
      Count_Matrix_Lambda[i] = (double *) calloc (2 * cmplist[c + 1].nfou,
                                                  sizeof (double));
      alloc_chk (WHERE, Count_Matrix_Lambda[i]);

   }
   FlagLambda = (int **) calloc (2 * cmplist[c + 1].nfou, sizeof (int *));
   alloc_chk (WHERE, FlagLambda);
   CntArray = (int *) calloc (2 * cmplist[c + 1].nfou, sizeof (int));
   alloc_chk (WHERE, CntArray);

   for (i = 0; i < (2 * cmplist[c + 1].nfou); i++)
   {
      FlagLambda[i] = (int *) calloc (2 * cmplist[c + 1].nfou, sizeof (int));
      alloc_chk (WHERE, FlagLambda[i]);

   }

   for (i = 0; i < pedsize; i++)
   {
      if (nghd[i].component == (c + 1))
      {
         First_Element = &nghd[i];

         Peeling_routine_Lambda (c, First_Element, trt_data, locus, Weight);
      }

   }
   MaxValue = -999;

   for (i = 0; i < (2 * cmplist[c + 1].nfou - 1); i++)
   {

      CntArray[i] = 0;
      if (Relation_Array[i] == 0)
      {
         Relation_Array[i] = 1;
         Relation (i, cmplist[c + 1].nfou, i, &CntArray[i]);

      }

      if (CntArray[i] > 0)
      {

         lambda_array = (int *) calloc (CntArray[i] + 1, sizeof (int));
         alloc_chk (WHERE, lambda_array);

         lambda_array[0] = i;

         j = 0;
         k = 1;

         while (j < (2 * cmplist[c + 1].nfou))
         {
            if ((Tag[j] > 0) && (j != i))
            {

               lambda_array[k] = j;
               k = k + 1;
            }
            j = j + 1;

         }

         likelihood = final_Calculation (SEG_LAMBDA, CntArray[i] + 1,
                                         lambda_array, param_val_array_Lambda);
         if (lambda_array)
         {
            free (lambda_array);
         }

         logfinalLikelihood = logfinalLikelihood + likelihood;

      }
      for (j = 0; j < 2 * cmplist[c + 1].nfou; j++)
      {

         Tag[j] = 0;
      }

   }

   if (Count_Matrix_Lambda != NULL)
   {
      for (i = 0; i < (2 * cmplist[c + 1].nfou); i++)
      {
         if (Count_Matrix_Lambda[i])
         {
            free (Count_Matrix_Lambda[i]);
         }
      }

      free (Count_Matrix_Lambda);

      Count_Matrix_Lambda = NULL;
   }
   if (FlagLambda != NULL)
   {
      for (i = 0; i < (2 * cmplist[c + 1].nfou); i++)
      {
         if (FlagLambda[i])
         {
            free (FlagLambda[i]);
         }
      }
      free (FlagLambda);
      FlagLambda = NULL;
   }

   if (flagTable_Lambda)
   {
      free (flagTable_Lambda);
      flagTable_Lambda = NULL;
   }
   if (CntArray)
      free (CntArray);
   if (Relation_Array)
   {
      free (Relation_Array);
      Relation_Array = NULL;
   }
   if (Tag)
   {
      free (Tag);
      Tag = NULL;
   }
   if (trt_data)
   {
      free (trt_data);
      trt_data = NULL;
   }
   if (Weight)
   {
      free (Weight);
      Weight = NULL;
   }
   if (param_val_array_Lambda)
   {
      free (param_val_array_Lambda);
   }

   return logfinalLikelihood;

}

double final_Calculation (int SEG_LAMBDA, int Size, int *Param_Array,
                          double *param_val_array_Lambda)
{
   double likelihood = 0;
   double oldVal = 0;
   double *lambda = NULL;
   double *TotalParam = NULL;
   int   Config = 0;
   int   j;
   int   v, v1 = 0;
   int   total_comb = (int) pow (SEG_LAMBDA, Size);

   TotalParam = NULL;
   lambda = NULL;
   TotalParam = (double *) calloc ((Size), sizeof (double));
   alloc_chk (WHERE, TotalParam);
   lambda = (double *) calloc ((Size), sizeof (double));
   alloc_chk (WHERE, lambda);

   likelihood = 0;
   if (Size == 2)
   {
      v = 0;
      v1 = 1;
      if ((Count_Matrix_Lambda[Param_Array[v]][Param_Array[v1]] == 0)
          || (Count_Matrix_Lambda[Param_Array[v1]][Param_Array[v]] == 0))
      {
         likelihood = 0;
      }
      else
      {
         likelihood =
            (double) (Count_Matrix_Lambda[Param_Array[v]][Param_Array[v1]]) *
            log ((double) Count_Matrix_Lambda[Param_Array[v]][Param_Array[v1]])
            +
            (double) (Count_Matrix_Lambda[Param_Array[v1]][Param_Array[v]]) *
            log ((double) Count_Matrix_Lambda[Param_Array[v]][Param_Array[v1]])
            - (double) (Count_Matrix_Lambda[Param_Array[v]][Param_Array[v1]] +
                        Count_Matrix_Lambda[Param_Array[v1]][Param_Array[v]]) *
            (log
             ((double)
              (Count_Matrix_Lambda[Param_Array[v]][Param_Array[v1]] +
               Count_Matrix_Lambda[Param_Array[v1]][Param_Array[v]])));
      }
      oldVal = likelihood;
   }
   else if (Size > 2)
   {
      oldVal = -999;
      Config = 0;
      while (Config < total_comb)
      {

         likelihood = 0;

         for (j = 0; j < (Size); j++)
         {
            int   gallop = (int) pow (SEG_LAMBDA, j);
            int   index = (int) Config / gallop;
            index = index % SEG_LAMBDA;
            TotalParam[Size - j - 1] = param_val_array_Lambda[index];

         }
         v = 0;
         while (v < Size)
         {
            lambda[v] = TotalParam[v];
            v = v + 1;
         }

         v = 0;
         while (v < (Size - 1))
         {
            v1 = v + 1;
            while (v1 < Size)
            {

               if ((Count_Matrix_Lambda[Param_Array[v]][Param_Array[v1]] > 0)
                   || (Count_Matrix_Lambda[Param_Array[v1]][Param_Array[v]] >
                       0))
               {

                  likelihood =
                     likelihood +
                     (double) (Count_Matrix_Lambda[Param_Array[v]]
                               [Param_Array[v1]]) * log (lambda[v]) +
                     (double) (Count_Matrix_Lambda[Param_Array[v1]]
                               [Param_Array[v]]) * log (lambda[v1]) -
                     (double) (Count_Matrix_Lambda[Param_Array[v]]
                               [Param_Array[v1]] +
                               Count_Matrix_Lambda[Param_Array[v1]][Param_Array
                                                                    [v]]) *
                     (log (lambda[v] + lambda[v1]));
               }
               v1 = v1 + 1;
            }
            v = v + 1;
         }

         if (likelihood > oldVal)
         {
            oldVal = likelihood;
         }
         Config = Config + 1;
      }

   }

   if (TotalParam)
   {
      free (TotalParam);
      TotalParam = NULL;
   }
   if (lambda)
      free (lambda);

   return oldVal;
}

void Peeling_routine_Lambda (int comp_no, Ind * First_Element, int *trt_data,
                             int locus, double *Weight)
{
   Ind  *spousePtr;
   Indlist *kidsPtr = NULL;
   Mar  *marCurrentPtr = NULL;
   Marlist *marListPtr = NULL;
   int   allele_group[4];
   if (First_Element == &nullnode)
   {
      return;
   }
   else if (flagTable_Lambda[First_Element->index] == 1)
   {
      return;
   }
   else
   {
      marListPtr = First_Element->marriages;
      if (marListPtr == NULL)
      {
         flagTable_Lambda[First_Element->index] = 1;
         return;
      }
      else
      {

         marCurrentPtr = marListPtr->current;
         if (marCurrentPtr != NULL)
         {

            spousePtr = marCurrentPtr->spouse;
            kidsPtr = marCurrentPtr->kids;
         }
      }

      if (First_Element->sex == 1)
      {
         allele_group[0] = gg0[locus][First_Element->index];
         allele_group[1] = gg1[locus][First_Element->index];
         allele_group[2] = gg0[locus][spousePtr->index];
         allele_group[3] = gg1[locus][spousePtr->index];
      }

      else if (First_Element->sex == 2)
      {
         allele_group[2] = gg0[locus][First_Element->index];
         allele_group[3] = gg1[locus][First_Element->index];
         allele_group[0] = gg0[locus][spousePtr->index];
         allele_group[1] = gg1[locus][spousePtr->index];
      }

      while (kidsPtr != NULL)
      {

         Calculate_Score_Lambda (comp_no, kidsPtr->current, allele_group,
                                 trt_data, locus, Weight);

         kidsPtr = kidsPtr->next;
      }

      flagTable_Lambda[First_Element->index] = 1;
      flagTable_Lambda[spousePtr->index] = 1;

      return;

   }
}

void Calculate_Score_Lambda (int c, Ind * temp, int allele_group[4],
                             int *trt_data, int locus, double *Weight)
{

   int   allele1, allele2;
   int   nfou = cmplist[c + 1].nfou;
   allele1 = gg0[locus][temp->index];
   allele2 = gg1[locus][temp->index];
   fflush (stdout);

   fflush (stdout);
   if (allele2 == allele_group[0])
   {
      if (trt_data[temp->index] == 2)
      {
         int   val1, val2;

         val1 = ((allele_group[0] - 1) % (2 * nfou));
         val2 = ((allele_group[1] - 1) % (2 * nfou));
         if ((val1 < 0) || (val2 < 0) || (val1 >= (2 * nfou))
             || (val2 >= (2 * nfou)))
         {
            printf ("*** SOMETHING WROUNG HERE *************(%d)\n", __LINE__);
            return;
         }
         Count_Matrix_Lambda[val1][val2] = Count_Matrix_Lambda[val1][val2] + 1;

      }
      else if (trt_data[temp->index] == 1)
      {
         int   val1, val2;
         val1 = ((allele_group[1] - 1) % (2 * nfou));
         val2 = ((allele_group[0] - 1) % (2 * nfou));
         if ((val1 < 0) || (val2 < 0) || (val1 >= (2 * nfou))
             || (val2 >= (2 * nfou)))
         {
            printf ("*** SOMETHING WROUNG HERE *************(%d)\n", __LINE__);
            return;
         }

         Count_Matrix_Lambda[val1][val2] = Count_Matrix_Lambda[val1][val2] + 1;

      }
      else if (trt_data[temp->index] == 0)
      {
         int   val1, val2;
         val1 = ((allele_group[0] - 1) % (2 * nfou));
         val2 = ((allele_group[1] - 1) % (2 * nfou));
         if ((val1 < 0) || (val2 < 0) || (val1 >= (2 * nfou))
             || (val2 >= (2 * nfou)))
         {
            printf ("*** SOMETHING WROUNG HERE *************(%d)\n", __LINE__);
            return;
         }

         Count_Matrix_Lambda[val1][val2] =
            Count_Matrix_Lambda[val1][val2] + Weight[temp->index];
         Count_Matrix_Lambda[val2][val1] =
            Count_Matrix_Lambda[val2][val1] + (1 - Weight[temp->index]);

      }
   }
   else if (allele2 == allele_group[1])
   {
      if (trt_data[temp->index] == 2)
      {
         int   val1, val2;
         val1 = ((allele_group[1] - 1) % (2 * nfou));
         val2 = ((allele_group[0] - 1) % (2 * nfou));
         if ((val1 < 0) || (val2 < 0) || (val1 >= (2 * nfou))
             || (val2 >= (2 * nfou)))
         {
            printf ("*** SOMETHING WROUNG HERE *************(%d)\n", __LINE__);
            return;
         }
         Count_Matrix_Lambda[val1][val2] = Count_Matrix_Lambda[val1][val2] + 1;
      }
      else if (trt_data[temp->index] == 1)
      {
         int   val1, val2;
         val1 = ((allele_group[0] - 1) % (2 * nfou));
         val2 = ((allele_group[1] - 1) % (2 * nfou));
         if ((val1 < 0) || (val2 < 0) || (val1 >= (2 * nfou))
             || (val2 >= (2 * nfou)))
         {
            printf ("*** SOMETHING WROUNG HERE *************(%d)\n", __LINE__);
            return;
         }
         Count_Matrix_Lambda[val1][val2] = Count_Matrix_Lambda[val1][val2] + 1;

      }
      else if (trt_data[temp->index] == 0)
      {
         int   val1, val2;
         val1 = ((allele_group[0] - 1) % (2 * nfou));
         val2 = ((allele_group[1] - 1) % (2 * nfou));
         if ((val1 < 0) || (val2 < 0) || (val1 >= (2 * nfou))
             || (val2 >= (2 * nfou)))
         {
            printf ("*** SOMETHING WROUNG HERE *************(%d)\n", __LINE__);
            return;
         }

         Count_Matrix_Lambda[val1][val2] =
            Count_Matrix_Lambda[val1][val2] + (1 - Weight[temp->index]);
         Count_Matrix_Lambda[val2][val1] =
            Count_Matrix_Lambda[val2][val1] + Weight[temp->index];

      }
   }
   if (allele1 == allele_group[2])
   {
      if (trt_data[temp->index] == 2)
      {
         int   val1, val2;
         val1 = ((allele_group[2] - 1) % (2 * nfou));
         val2 = ((allele_group[3] - 1) % (2 * nfou));
         if ((val1 < 0) || (val2 < 0) || (val1 >= (2 * nfou))
             || (val2 >= (2 * nfou)))
         {
            printf ("*** SOMETHING WROUNG HERE *************(%d)\n", __LINE__);
            return;
         }
         Count_Matrix_Lambda[val1][val2] = Count_Matrix_Lambda[val1][val2] + 1;

      }
      else if (trt_data[temp->index] == 1)
      {
         int   val1, val2;
         val1 = ((allele_group[3] - 1) % (2 * nfou));
         val2 = ((allele_group[2] - 1) % (2 * nfou));
         if ((val1 < 0) || (val2 < 0) || (val1 >= (2 * nfou))
             || (val2 >= (2 * nfou)))
         {
            printf ("*** SOMETHING WROUNG HERE *************(%d)\n", __LINE__);
            return;
         }
         Count_Matrix_Lambda[val1][val2] = Count_Matrix_Lambda[val1][val2] + 1;

      }
      else if (trt_data[temp->index] == 0)
      {
         int   val1, val2;
         val1 = ((allele_group[3] - 1) % (2 * nfou));
         val2 = ((allele_group[2] - 1) % (2 * nfou));
         if ((val1 < 0) || (val2 < 0) || (val1 >= (2 * nfou))
             || (val2 >= (2 * nfou)))
         {
            printf ("*** SOMETHING WROUNG HERE *************(%d)\n", __LINE__);
            return;
         }
         Count_Matrix_Lambda[val1][val2] =
            Count_Matrix_Lambda[val1][val2] + 1 - Weight[temp->index];
         Count_Matrix_Lambda[val2][val1] =
            Count_Matrix_Lambda[val2][val1] + (Weight[temp->index]);

      }
   }
   else if (allele1 == allele_group[3])
   {
      if (trt_data[temp->index] == 2)
      {
         int   val1, val2;
         val1 = ((allele_group[3] - 1) % (2 * nfou));
         val2 = ((allele_group[2] - 1) % (2 * nfou));
         if ((val1 < 0) || (val2 < 0) || (val1 >= (2 * nfou))
             || (val2 >= (2 * nfou)))
         {
            printf ("*** SOMETHING WROUNG HERE *************(%d)\n", __LINE__);
            return;
         }
         Count_Matrix_Lambda[val1][val2] = Count_Matrix_Lambda[val1][val2] + 1;
      }
      else if (trt_data[temp->index] == 1)
      {
         int   val1, val2;
         val1 = ((allele_group[3] - 1) % (2 * nfou));
         val2 = ((allele_group[2] - 1) % (2 * nfou));
         if ((val1 < 0) || (val2 < 0) || (val1 >= (2 * nfou))
             || (val2 >= (2 * nfou)))
         {
            printf ("*** SOMETHING WROUNG HERE *************(%d)\n", __LINE__);
            return;
         }
         Count_Matrix_Lambda[val2][val1] = Count_Matrix_Lambda[val2][val1] + 1;
      }
      else if (trt_data[temp->index] == 0)
      {
         int   val1, val2;
         val1 = ((allele_group[3] - 1) % (2 * nfou));
         val2 = ((allele_group[2] - 1) % (2 * nfou));
         if ((val1 < 0) || (val2 < 0) || (val1 >= (2 * nfou))
             || (val2 >= (2 * nfou)))
         {
            printf ("*** SOMETHING WROUNG HERE *************(%d)\n", __LINE__);
            return;
         }

         Count_Matrix_Lambda[val2][val1] =
            Count_Matrix_Lambda[val2][val1] + 1 - Weight[temp->index];
         Count_Matrix_Lambda[val1][val2] =
            Count_Matrix_Lambda[val1][val2] + (Weight[temp->index]);
      }

   }
   else
   {
      printf ("*********Default allele..mismatch at line(%d)\n", __LINE__);
   }
}

double CalPValue (FILE * read_fp, int No_Permutation, int No_locus, int locus)
{
   double obsval = 0;
   int   p_cnt = 0;
   double p_val = 0;
   double val = 0;
   char  line[100];
   int   tag = 0;

   if (read_fp == NULL)
   {
      printf ("Could not open the file\n");
      return 0;
   }
   else
   {

      p_cnt = 0;

      while (fgets (line, 1000, read_fp) != NULL)
      {
         tag = tag + 1;

         val = 0;
         if ((tag == (locus + 1)) && (locus == 0))
         {
            sscanf (line, "%lf\n", &val);

            obsval = val;

         }

         else if ((tag == (locus + 1)) && (locus > 0))
         {

            sscanf (line, "%lf\n", &val);

            obsval = val;

         }
         else if ((tag > No_locus) && ((locus + 1) != No_locus)
                  && ((tag % No_locus) == (locus + 1)))
         {
            sscanf (line, "%lf\n", &val);

         }
         else if ((tag > No_locus) && ((tag % No_locus) == 0)
                  && (locus + 1 == No_locus))
         {

            sscanf (line, "%lf\n", &val);

         }

         if ((val >= obsval) && (obsval > 0))
         {

            p_cnt = p_cnt + 1;
         }
      }

      p_val = (double) p_cnt / No_Permutation;

      return p_val;
   }
}

void Weight_Multilambda (double *paff)
{
   int   i;
   double Total = 0;
   printf ("I am here %d\n", __LINE__);
   for (i = 0; i < pedsize; i++)
   {
      Ind  *First_Element = &nghd[i];
      if (First_Element->d_dat != NULL)
      {
         if ((First_Element->d_dat != NULL)
             && ((First_Element->pa != &nullnode)
                 && (First_Element->ma != &nullnode)))
         {

            Vnode *HeadPtr = NULL;
            Vnode *Weight = (Vnode *) calloc (1, sizeof (Vnode));
            printf ("I am here %d\n", __LINE__);
            Weight_Calculation (First_Element, &Weight);
            printf ("I am here %d\n", __LINE__);
            HeadPtr = Weight;
            Total = 0;
            while (HeadPtr != NULL)
            {

               Total = (double) (1 / (double) (HeadPtr->level + 1)) + Total;
               paff[First_Element->index] =
                  paff[First_Element->index] +
                  (double) (1 / (double) (HeadPtr->level + 1)) *
                  ((double) HeadPtr->count_aff /
                   (HeadPtr->count_aff + HeadPtr->count_unaff));
               HeadPtr = HeadPtr->next;
            }
            paff[First_Element->index] = paff[First_Element->index] / Total;
            while (Weight != NULL)
            {
               Vnode *temp = Weight;
               Weight = Weight->next;
               free (temp);
            }
         }
      }
   }

}
