/* MORGAN: Copyright 1991-2005: Univ. of Washington. All rights reserved. */
/* You may copy, use, modify, but NOT redistribute, modified or otherwise.*/
/* See: http://www.stat.washington.edu/thompson/Genepi/license.shtml      */
/**
 * this function builds global data structures used by Genedrop and
 * Autozyg programs.
 * proc_drop() has as its input the results of parsing the parameter
 * statements by proc_pars() for genedrop.  It checks this data for 
 * consistency and completeness and, if there are no errors, builds data
 * structures used by genedrop.
 *                                                           MAJ Jun 1999 
 */
int proc_drop (void)
{
   int   entry, entry2, entry3, sex;
   int   lnkd, m_no, loc_no, t_no;
   int   max_alle;
   SIM_PTR p_s;
   DIST_PTR p_d;
   UNLNKD_PTR p_u;
   TLIST_PTR p_t;
   PRNTMAP_PTR p_m;
   PRNTLOC_PTR p_l;
   int   sxsp_maps = FALSE;

   if (!test_sim ())                    /* checks for simulation             */
      if (proc_err ())                  /* statement(s)                      */
         return FALSE;

   if (n_err)
      return proc_end ();
   /*                                    * for each simulated chromosome,    */
   entry = FALSE;                       /* checks marker map and trait map   */
   while (p_s = nxt_sim_chrom (&entry))
   {
      if (!test_mark_map (p_s, &sxsp_maps))
         if (proc_err ())
            return FALSE;
      if (!test_trait_map (p_s, &sxsp_maps))
         if (proc_err ())
            return FALSE;
   }
   /*                                    * for each simulated chromosome,    */
   /*                                    *  for each trait,                  */
   entry = FALSE;                       /* checks for duplications           */
   lnkd = TRUE;
   while (p_s = nxt_sim_chrom (&entry))
   {
      entry2 = FALSE;
      while (p_d = nxt_mapd_trait (&entry2, AVG, p_s))
         if (!test_trait_list (lnkd, p_d->no))
            if (proc_err ())
               return FALSE;
   }
   /*                                    * for each unlinked trait,          */
   entry = FALSE;                       /* checks for duplications           */
   lnkd = FALSE;
   while (p_u = nxt_unlnkd_trait (&entry))
      if (!test_trait_list (lnkd, p_u->no))
         if (proc_err ())
            return FALSE;
   /*                                    * for each chromosome,              */
   /*                                    * for each marker,                  */
   entry = FALSE;                       /* checks for allele frequencies     */
   while (p_s = nxt_sim_chrom (&entry))
   {
      entry2 = FALSE;
      while (m_no = nxt_mark_no (&entry2, p_s))
         if (!test_mark_loci (p_s, m_no))
            if (proc_err ())
               return FALSE;
   }

   /*                                    * for each trait,                   */
   /*                                    * checks variances, genotype        */
   entry = FALSE;                       /* means and allele frequencies      */
   while (p_t = nxt_trait (&entry))
   {
      if (!test_trait_vars (p_t->no))
         if (proc_err ())
            return FALSE;
      if (!test_trait_genotyp (p_t->no))
         if (proc_err ())
            return FALSE;
      if (!test_trait_loci (p_t->no))
         if (proc_err ())
            return FALSE;
   }
   if (n_err)
      return proc_end ();
   /*                                    * for each simulated chromosome,    */
   /*                                    * if no errors, prints out          */
   /*                                    * simulation parameters             */
   /*                                    * for each chromosome, for each     */
   /*                                    * sex, puts markers into "print     */
   /*                                    * map"                              */
   /*                                    * maps not present default to       */
   /*                                    * sex-averaged map, then F map      */
   entry = FALSE;

   while (p_s = nxt_sim_chrom (&entry))
   {
      for (sex = FEM; sex <= AVG; ++sex)
      {                                 /* adds marker to map for sex        */
         p_m = add_prntmap (p_s, sex);
         entry2 = FALSE;
         entry3 = FALSE;

         if (p_s->no_mark_map == 1)
         {
            p_l = add_only_mark_locus (p_m);
            sel_mark_locus (p_s, p_l, &entry3); /* FIXME:  to set selectd   */
         }
         else
            while (p_d = nxt_mapd_mark (&entry2, sex, p_s))
            {
               p_l = add_mark_locus (p_m, p_d);
               sel_mark_locus (p_s, p_l, &entry3); /* FIXME: to set selectd */
            }
         /*                              * adds traits to map for sex        */
         entry2 = FALSE;
         while (p_d = nxt_mapd_trait (&entry2, sex, p_s))
            add_trait_locus (p_m, p_d);

         /*                              * finds distances between           */
         entry2 = FALSE;                /* adjacent loci                     */
         while (p_l = nxt_prntmap_locus (&entry2, p_m))
            fix_locus_dist (p_m, p_l);

         if ((sxsp_maps && (sex == FEM || sex == MAL))
             || (!sxsp_maps && sex == AVG))
         {
            prnt_map (p_m, sex);        /* prints the map                    */

            entry2 = FALSE;             /* checks trait distances            */
            while (p_l = nxt_prntmap_locus (&entry2, p_m))
               if (!test_trait_dist (p_m, p_l))
                  if (proc_err ())
                     return FALSE;
         }
      }
      prnt_mark_freqs (p_s);            /* prints marker parameters          */

      prnt_trait_freqs (p_m);           /* prints trait parameters           */
      prnt_trait_pars (p_m);
   }
   prnt_unlnkd_freqs ();
   prnt_unlnkd_pars ();
   fprintf (stdout, "\n");

   if (n_err)
      return proc_end ();

   NLoci = 0;                           /* counts loci and allocates         */
   NMarks = 0;                          /* recombination table               */
   NTraits = 0;
   NUnlnkd = 0;
   entry = FALSE;
   while (p_m = nxt_prntmap (&entry, AVG))
   {
      NMarks += p_m->no_mark_sel;
      NTraits += p_m->no_traits;
   }
   entry = FALSE;
   while (p_u = nxt_unlnkd_trait (&entry))
   {
      ++NTraits;
      ++NUnlnkd;
   }
   NLoci = NMarks + NTraits;

   alloc_recomb ();
   /*                                    * for F and M recomb tables, adds   */
   /*                                    * entry for each locus to table     */
   for (sex = FEM; sex <= MAL; ++sex)
   {
      loc_no = 0;
      entry = FALSE;
      while (p_m = nxt_prntmap (&entry, sex))
      {
         entry2 = FALSE;
         while (p_l = nxt_prntmap_locus (&entry2, p_m))
         {
            mak_recomb (p_l, loc_no, sex);
            ++loc_no;
         }
      }
      /*                                 * for each unlinked trait           */
      entry = FALSE;                    /* adds entry to recomb table        */
      while (p_u = nxt_unlnkd_trait (&entry))
      {
         mak_no_recomb (p_u, loc_no, sex);
         ++loc_no;
      }
   }

   if (NTraits)
   {
      alloc_trait ();                   /* for each trait                    */
      /*                                 * stores trait computational        */
      loc_no = 0;                       /* parameters                        */
      t_no = 0;
      entry = FALSE;
      while (p_m = nxt_prntmap (&entry, AVG))
      {
         entry2 = FALSE;
         while (p_l = nxt_prntmap_locus (&entry2, p_m))
         {
            if (p_l->locus_typ == TD)
            {
               mak_trait (p_l->no, loc_no, t_no);
               ++t_no;
            }
            ++loc_no;
         }
      }
      entry = FALSE;
      while (p_u = nxt_unlnkd_trait (&entry))
      {
         mak_trait (p_u->no, loc_no, t_no);
         ++t_no;
         ++loc_no;
      }
   }
   /*                                    * for each chromosome               */
   loc_no = 0;
   entry = FALSE;
   while (p_m = nxt_prntmap (&entry, AVG))
   {                                    /* for each locus                    */
      entry2 = FALSE;                   /* stores allele frequencies         */
      while (p_l = nxt_prntmap_locus (&entry2, p_m))
      {
         mak_freq (p_m, p_l, p_l->no, loc_no);
         ++loc_no;
      }
   }
   entry = FALSE;
   while (p_u = nxt_unlnkd_trait (&entry))
   {
      mak_freq (NULL, NULL, p_u->no, loc_no);
      ++loc_no;
   }

   max_alle = 0;
   loc_no = 0;
   entry = FALSE;
   while (p_m = nxt_prntmap (&entry, AVG))
   {                                    /* finds the maximum no. of alleles  */
      entry2 = FALSE;
      while (p_l = nxt_prntmap_locus (&entry2, p_m))
      {
         if ((PAlleleFreq + loc_no)->nAlleles > max_alle)
            max_alle = (PAlleleFreq + loc_no)->nAlleles;
         ++loc_no;
      }
   }
   WidAlle = intWid (0, max_alle);

   return TRUE;
}

/*------------------------ functions follow ---------------------------------*/

/**
 * gets the next simulated chromsome.
 * returns a pointer to the next sim node.
 */
SIM_PTR nxt_sim_chrom (int *p_entry)
{
   static SIM_PTR p_sav;
   if (!*p_entry)
   {
      *p_entry = TRUE;
      return (p_sav = first_sim_node);
   }
   else
      return (p_sav = p_sav->p_next);
}

/**
 * gets the next trait.
 * returns a pointer to the entry for the next trait in the distance list
 * uses defaults if specified sex is not present 
 */
DIST_PTR nxt_mapd_trait (int *p_entry, int sx, SIM_PTR p_sim)
{
   int   i, j;
   static int no_loci;
   MAP_PTR p, p_map;
   static DIST_PTR p_sav;

   if (!*p_entry)                       /* tests for next chromosome         */
   {
      *p_entry = TRUE;
      if (p_sim->no_traits == 0)
         return NULL;

      p_map = NULL;

      for (i = 1; !p_map && i <= 4; ++i)
      {
         if (i == 1)                    /* order to check for map presnt     */
            j = sx;
         else if (i == 2)
            j = AVG;
         else if (i == 3)
            j = FEM;
         else if (i == 4)
            j = MAL;

         if (p = first_trait_map_node[j])
         {
            while (p->p_next && p->chrom_no < p_sim->chrom_no)
               p = p->p_next;
            if (p->chrom_no == p_sim->chrom_no)
            {
               no_loci = p->no_loci;
               p_map = p;
            }
         }
      }
      if (!p_map)
         return NULL;
      else
         return (p_sav = p_map->p_dist);
   }
   if (--no_loci)                       /* returns ptr to next trait         */
      return ++p_sav;                   /* (if any)                          */
   else
      return NULL;
}

/**
 * gets the next marker.
 * returns a pointer to the entry for the next marker in the dist list.
 * uses defaults if specified sex not present.
 */
DIST_PTR nxt_mapd_mark (int *p_entry, int sx, SIM_PTR p_sim)
{
   int   i, j;
   static int no_loci;
   MAP_PTR p, p_map;
   static DIST_PTR p_sav;

   if (!*p_entry)
   {
      *p_entry = TRUE;

      if (p_sim->no_mark_map == 1)
         return NULL;

      p_map = NULL;

      for (i = 1; !p_map && i <= 4; ++i)
      {
         if (i == 1)                    /* order to check for map presnt      */
            j = sx;
         else if (i == 2)
            j = AVG;
         else if (i == 3)
            j = FEM;
         else if (i == 4)
            j = MAL;

         if (p = first_mark_map_node[j])
         {
            while (p->p_next && p->chrom_no < p_sim->chrom_no)
               p = p->p_next;
            if (p->chrom_no == p_sim->chrom_no)
            {
               no_loci = p->no_loci;
               p_map = p;
            }
         }
      }
      if (!p_map)
         return NULL;
      else
         return (p_sav = p_map->p_dist);
   }
   if (--no_loci)
      return ++p_sav;
   else
      return NULL;
}

UNLNKD_PTR nxt_unlnkd_trait (int *p_entry)
{
   static UNLNKD_PTR p_sav;

   if (!*p_entry)
   {
      *p_entry = TRUE;                  /* returns ptr to first unlinked     */
      return (p_sav = first_unlnkd_node); /* trait                          */
   }
   else
      return (p_sav = p_sav->p_next);   /* returns ptr to next               */
}

int nxt_mark_no (int *p_entry, SIM_PTR p_sim)
{
   static int mark_no;

   if (!*p_entry)
   {
      *p_entry = TRUE;
      return (mark_no = 1);
   }
   else
   {
      if (mark_no >= p_sim->no_mark_map)
         return 0;
      else
         return ++mark_no;
   }
}

TLIST_PTR nxt_trait (int *p_entry)
{
   static TLIST_PTR p_sav;

   if (!*p_entry)
   {
      *p_entry = TRUE;
      return (p_sav = first_tlist_node);
   }
   else
      return (p_sav = p_sav->p_next);
}

PRNTLOC_PTR nxt_prntmap_locus (int *p_entry, PRNTMAP_PTR p_pmap)
{
   static PRNTLOC_PTR p_loc;

   if (!*p_entry)
   {
      *p_entry = TRUE;
      p_loc = p_pmap->p_locus;
   }
   else
      p_loc = p_loc->p_next;

   return p_loc;
}

/**
 * returns a pointer to the next printmap node.
 */
PRNTMAP_PTR nxt_prntmap (int *p_entry, int sx)
{
   static PRNTMAP_PTR p_sav;
   if (!*p_entry)
   {
      *p_entry = TRUE;
      return (p_sav = first_prntmap_node[sx]);
   }
   else
      return (p_sav = p_sav->p_next);
}

int test_sim (void)
{
   if (!first_sim_node && !first_unlnkd_node)
   {                                    /* tests for simulate statement      */
      err_typ = ERD_SIM;
      return FALSE;
   }
   if (first_sim_node)                  /* checks for both names and         */
      /*                                 * unnamed chromosomes               */
      if (first_sim_node->chrom_no == 0 && first_sim_node->p_next)
      {
         err_typ = ERD_CHR;
         return FALSE;
      }
   return TRUE;
}

/**
 * checks the marker map for the chromosome.
 * tests for marker map present for this chromosome.
 * adds the no. of mapped markers to the sim node.                     
 * tests for consistent sex-specific marker maps.                    
 * if marker map is missing, checks that the map is not needed.            
 * for simulation, checks that no. of markers is same as no. mapped.     
 * for selection of all markers, sets the number of markers to the 
 * no. mapped.                        
 * for list of markers, checks marker no.'s against no. mapped.          
 */
int test_mark_map (SIM_PTR p_sim, int *ss_map)
{
   int   sx;
   MAP_PTR p_map;
   MAP_PTR p_m[3] = { NULL, NULL, NULL };

   for (sx = FEM; sx <= AVG; ++sx)      /* check for marker maps present      */
      if (p_map = first_mark_map_node[sx])
      {
         while (p_map->p_next && p_map->chrom_no < p_sim->chrom_no)
            p_map = p_map->p_next;
         if (p_map->chrom_no == p_sim->chrom_no) /* if found, adds the no.   */
         {
            p_sim->no_mark_map = p_map->no_loci; /* marks mapped to sim_node */
            p_m[sx] = p_map;
            if (sx != AVG)              /* checks for sex-specific map        */
               *ss_map = TRUE;
         }
      }
   if (p_m[FEM] && p_m[MAL] && (p_m[FEM]->no_loci != p_m[MAL]->no_loci))
      err_typ = ERD_NFM;
   if (p_m[MAL] && p_m[AVG] && (p_m[MAL]->no_loci != p_m[AVG]->no_loci))
      err_typ = ERD_NMA;
   if (p_m[AVG] && p_m[FEM] && (p_m[AVG]->no_loci != p_m[FEM]->no_loci))
      err_typ = ERD_NFA;
   if (err_typ)                         /* check for same no. markers for     */
   {                                    /* ea gender                          */
      for_chrom (p_sim->chrom_no);
      return FALSE;
   }

   if (!p_sim->no_mark_map)             /* **** no map is present        **** */
   {
      if (MK == p_sim->sim_typ && p_sim->no_mark_sel != 1) /* ..and simulatn */
         err_typ = ERD_MMM;             /* checks for only one marker         */

      else if (MKL == p_sim->sim_typ)   /* ..and selection of markers         */
         if (p_sim->no_mark_sel != 1)   /* checks no. selected                */
            err_typ = ERD_MMM;
         else if (*(p_sim->p_mark) != 1) /* and for marker no. 1             */
            err_typ = ERA_MN1;

      if (err_typ)
      {
         for_chrom (p_sim->chrom_no);
         return FALSE;
      }

      if (AMK == p_sim->sim_typ)        /* if all markers are selected        */
      {                                 /* assumes 1 marker                   */
         p_sim->no_mark_sel = 1;
         err_typ = ERA_1MA;             /* issues a warning                   */
         for_chrom (p_sim->chrom_no);
         return FALSE;
      }
      return TRUE;
   }

   if (MK == p_sim->sim_typ)            /* **** a map is present        * *** */
   {
      if (p_sim->no_mark_sel == 1)      /* for simulation, checks for 1 mark; */
         err_typ = ERD_NME;             /* there should be no map             */
      if (p_sim->no_mark_sel != p_sim->no_mark_map) /* checks for no. marks  */
         err_typ = ERD_NAM;             /* simulated same as no. mapped       */
   }
   else if (AMK == p_sim->sim_typ)      /* for selection of all,              */
      p_sim->no_mark_sel = p_sim->no_mark_map; /* set no. marks to no. mappd */

   else if (MKL == p_sim->sim_typ)      /* for marker list, checks range      */
      if (*(p_sim->p_mark + p_sim->no_mark_sel - 1) > p_sim->no_mark_map)
         err_typ = ERA_MNL;

   if (err_typ)
   {
      for_chrom (p_sim->chrom_no);
      return FALSE;
   }
   else
      return TRUE;
}

/**
 * checks the trait map for the chromosome.
 * tests for trait map present for this chromosome.                
 * checks sex-specific maps for consistency.                  
 * checks for correct no. of traits.
 * checks for marker no.'s within range.
 */
int test_trait_map (SIM_PTR p_sim, int *ss_map)
{
   int   sx, nt, j;
   MAP_PTR p_map, p_sm[3] = { NULL, NULL, NULL };
   DIST_PTR p_d1, p_d2;

   if (!p_sim->no_traits)               /* checks for linked trait            */
      return TRUE;

   for (sx = FEM; sx <= AVG; ++sx)      /* checks for trait map present       */
      if (p_map = first_trait_map_node[sx])
      {
         while (p_map->p_next && p_map->chrom_no < p_sim->chrom_no)
            p_map = p_map->p_next;
         if (p_map->chrom_no == p_sim->chrom_no)
         {
            nt = p_map->no_loci;
            p_sm[sx] = p_map;           /* if found, checks for sex-spec map  */
            if (sx != AVG)
               *ss_map = TRUE;
         }
      }
   if (!p_sm[AVG] && !p_sm[MAL] && !p_sm[FEM]) /* checks for at least 1      */
   {                                    /* trait map present                  */
      err_typ = ERD_MTM;
      for_chrom (p_sim->chrom_no);
      return FALSE;
   }
   if (p_sm[FEM] && p_sm[MAL] && (p_sm[FEM]->no_loci != p_sm[MAL]->no_loci))
      err_typ = ERD_TFM;
   if (p_sm[MAL] && p_sm[AVG] && (p_sm[MAL]->no_loci != p_sm[AVG]->no_loci))
      err_typ = ERD_TMA;
   if (p_sm[AVG] && p_sm[FEM] && (p_sm[AVG]->no_loci != p_sm[FEM]->no_loci))
      err_typ = ERD_TFA;
   if (err_typ)                         /* checks for same no. traits for ea  */
   {                                    /* gender                             */
      for_chrom (p_sim->chrom_no);
      return FALSE;
   }

   for (sx = FEM; sx <= AVG; ++sx)      /* checks sex-specific maps for       */
   {                                    /* consistency                        */
      if (p_sm[sx])
      {
         p_d1 = p_sm[sx]->p_dist;
         p_d2 = NULL;
         if (p_sm[(sx + 1) % (AVG + 1)])
            p_d2 = p_sm[(sx + 1) % (AVG + 1)]->p_dist;

         for (j = 0; j < p_sm[sx]->no_loci; ++j)
         {
            if (p_sim->no_mark_map == 0) /* no marker map                    */
            {
               if (p_d1->map_mark > 1)  /* checks for marker no. 0 or 1       */
                  err_typ = ERD_MMX;
            }
            else if (p_d1->map_mark > p_sim->no_mark_map) /* checks for mark */
               err_typ = ERD_MMX;       /* no. in trait map in range          */
            else if (p_d2)              /* checks for trait no. the same in   */
            {                           /* in sex-specific maps               */
               if (p_d1->no != p_d2->no)
                  err_typ = ERD_TID;
               else if (p_d1->map_mark != p_d2->map_mark) /* checks trait    */
                  err_typ = ERD_MID;    /* locus the same in sex-spec maps    */
               ++p_d2;
            }
            ++p_d1;
         }
      }
   }
   if (err_typ)
   {
      for_chrom (p_sim->chrom_no);
      return FALSE;
   }
   else
      return TRUE;
}

int test_trait_list (int linked, int t_no)
{
   TLIST_PTR cur_node, new_node;

   if (first_tlist_node)
   {
      cur_node = first_tlist_node;
      while (cur_node->p_next && cur_node->p_next->no <= t_no)
         cur_node = cur_node->p_next;

      if (cur_node->no == t_no)         /* checks for trait already in list   */
      {
         sprintf (errnumb, "%d", t_no);
         err_txt = errnumb;
         if (linked && cur_node->lnkd)
            err_typ = ERD_LLL;
         else
            err_typ = ERD_LUL;
      }
   }
   if (err_typ)
      return FALSE;

   new_node = malloc (sizeof (TLIST));
   alloc_chk (WHERE, new_node);

   new_node->no = t_no;
   new_node->lnkd = linked;

   if (!first_tlist_node || first_tlist_node->no > t_no)
   {
      new_node->p_next = first_tlist_node;
      first_tlist_node = new_node;
   }
   else
   {
      new_node->p_next = cur_node->p_next;
      cur_node->p_next = new_node;
   }
   return TRUE;
}

int test_mark_loci (SIM_PTR p_sim, int m_no)
{
   LOCUS_PTR p_locus;
   FREQ_PTR p_freq;

/* add code to look at p_sim->p_mark no's  for selection ????????? *******   */

   p_locus = first_mark_locus_node;

   if (!p_locus)
      err_typ = ERD_NML;
   else
   {
      while (p_locus->p_next && (p_locus->chrom_no < p_sim->chrom_no
                                 || (p_locus->chrom_no == p_sim->chrom_no
                                     && p_locus->no < m_no)))
         p_locus = p_locus->p_next;

      if (p_locus->chrom_no != p_sim->chrom_no || (p_locus->chrom_no == p_sim->chrom_no && p_locus->no != m_no)) /* test for allele freqs for marker  */
         err_typ = ERD_NML;
   }
   if (err_typ)
   {
      for_chrom_mark (p_sim->chrom_no, m_no);
      /*                                 * stores default values             */
      p_freq = alloc_dflt_frq ();
      stor_dflt_frq (p_freq);
      p_locus = alloc_dflt_locus_node (p_sim->chrom_no, m_no, MD);
      stor_dflt_locus_data (p_sim->chrom_no, m_no, p_locus, p_freq);
   }
   else if (norm_frq (norm_frq_spec, p_locus->p_freq_node->sum))
   {
      for_chrom_mark (p_sim->chrom_no, m_no);
      err_typ = ERD_MAN;
   }
   if (err_typ)
      return FALSE;
   else
      return TRUE;
}

int test_trait_vars (int t_no)
{
   int   no_addi = FALSE, no_resi = FALSE;
   double sum = 0.;
   VARI_PTR p_addi, p_resi;

   p_addi = first_addi_node;
   p_resi = first_resi_node;

   if (!p_addi)
      no_addi = TRUE;
   else
   {
      while (p_addi->p_next && p_addi->no < t_no)
         p_addi = p_addi->p_next;
      if (p_addi->no == t_no)
         sum = p_addi->vari;
      else
         no_addi = TRUE;
   }
   if (!p_resi)
      no_resi = TRUE;
   else
   {
      while (p_resi->p_next && p_resi->no < t_no)
         p_resi = p_resi->p_next;
      if (p_resi->no == t_no)
         sum = sum + p_resi->vari;
      else
         no_resi = TRUE;
   }
   /*                                    * stores default values             */
   if (no_addi)
   {
      p_addi = alloc_vari_node (t_no, AV);
      stor_dflt_vari_data (t_no, p_addi);
   }
   if (no_resi)
   {
      p_resi = alloc_vari_node (t_no, RV);
      stor_dflt_vari_data (t_no, p_resi);
   }
   if (no_addi && no_resi)
      err_typ = ERD_NAR;
   else if (sum == 0.)
      err_typ = ERD_SVR;
   else if (no_addi)
      err_typ = ERD_TAV;
   else if (no_resi)
      err_typ = ERD_TRV;

   if (err_typ)
   {
      sprintf (errnumb, "%d", t_no);
      err_txt = errnumb;
      return FALSE;
   }
   else
      return TRUE;
}

int test_trait_genotyp (int t_no)
{
   MEANS_PTR p_means;

   p_means = first_means_node;

   if (!p_means)
      err_typ = ERD_NGM;
   else
   {
      while (p_means->p_next && p_means->no < t_no)
         p_means = p_means->p_next;

      if (p_means->no != t_no)
         err_typ = ERD_NGM;
   }
   if (err_typ)
   {
      sprintf (errnumb, "%d", t_no);
      err_txt = errnumb;
      /*                                 * stores default values             */
      p_means = alloc_means_node (t_no);
      stor_dflt_mean_data (t_no, p_means);

      return FALSE;
   }
   else
      return TRUE;
}

int test_trait_loci (int t_no)
{
   LOCUS_PTR p_locus;
   FREQ_PTR p_freq;

   p_locus = first_trait_locus_node;

   if (!p_locus)
      err_typ = ERD_NTL;
   else
   {
      while (p_locus->p_next && p_locus->no < t_no)
         p_locus = p_locus->p_next;

      if (p_locus->no != t_no)
         err_typ = ERD_NTL;
   }
   if (err_typ)
   {
      sprintf (errnumb, "%d", t_no);
      err_txt = errnumb;
      /*                                 * stores default values             */
      p_freq = alloc_dflt_frq ();
      stor_dflt_frq (p_freq);
      p_locus = alloc_dflt_locus_node (NO_CH_NO, t_no, TD);
      stor_dflt_locus_data (NO_CH_NO, t_no, p_locus, p_freq);
   }
   else if (norm_frq (norm_frq_spec, p_locus->p_freq_node->sum))
   {
      err_typ = ERD_TAN;
      sprintf (errnumb, "%d", t_no);
      err_txt = errnumb;
   }
   if (err_typ)
      return FALSE;
   else
      return TRUE;
}

void for_chrom (int chr_no)
{
   if (chr_no > 0)
      sprintf (errnumb, "for chromosome %d", chr_no);
   else
      errnumb[0] = '\0';

   err_txt = errnumb;
   return;
}

void on_chrom (int chr_no)
{
   if (chr_no > 0)
      sprintf (errnumb, "on chromosome %d", chr_no);
   else
      errnumb[0] = '\0';

   err_txt = errnumb;
   return;
}

void for_chrom_mark (int chr_no, int m_no)
{
   if (chr_no > 0)
      sprintf (errnumb, "chromosome %d marker %d", chr_no, m_no);
   else
      sprintf (errnumb, "marker %d", m_no);

   err_txt = errnumb;
   return;
}

PRNTMAP_PTR add_prntmap (SIM_PTR p_sim, int sx)
{
   PRNTMAP_PTR p_pmap, p_cur;

   p_pmap = malloc (sizeof (PRNTMAP));  /* makes prntmap node for this       */
   alloc_chk (WHERE, p_pmap);           /* chromosome                        */

   p_pmap->chrom_no = p_sim->chrom_no;
   p_pmap->no_mark_sel = p_sim->no_mark_sel;
   p_pmap->no_mark_map = p_sim->no_mark_map;
   p_pmap->no_traits = 0;
   p_pmap->p_locus = NULL;
   p_pmap->p_next = NULL;

   if (!first_prntmap_node[sx])         /* puts node as first in list        */
      first_prntmap_node[sx] = p_pmap;
   else
   {
      p_cur = first_prntmap_node[sx];   /* or adds node to end of list       */
      while (p_cur->p_next)
         p_cur = p_cur->p_next;
      p_cur->p_next = p_pmap;
   }
   return p_pmap;
}

PRNTLOC_PTR add_locus_node (void)
{
   PRNTLOC_PTR p_new;

   p_new = malloc (sizeof (PRNTLOC));   /* allocs and inits prntmap_locus     */
   alloc_chk (WHERE, p_new);            /* node                               */

   p_new->locus_typ = 0;
   p_new->no = 0;
   p_new->map_mark = 0;
   p_new->selectd = 0;
   p_new->sel_map_mark = 0;
   p_new->dist = 0.;
   p_new->sel_dist = 0.;
   p_new->loc_dist = 0.;
   p_new->p_prev = NULL;
   p_new->p_next = NULL;

   return p_new;
}

PRNTLOC_PTR add_only_mark_locus (PRNTMAP_PTR p_pmap)
{
   PRNTLOC_PTR p_new;

   p_new = add_locus_node ();           /* makes prntmap_locus node for mark  */

   p_new->no = 1;                       /* the marker is unmapped             */
   p_new->locus_typ = MD;

   p_pmap->p_locus = p_new;

   return p_new;
}

PRNTLOC_PTR add_mark_locus (PRNTMAP_PTR p_pmap, DIST_PTR p_dist)
{
   PRNTLOC_PTR p_new, p_cur;

   p_new = add_locus_node ();           /* makes prntmap_locus node for mark  */

   p_new->locus_typ = MD;
   p_new->no = p_dist->no;
   p_new->map_mark = p_dist->map_mark;
   p_new->dist = p_dist->dist;

   if (!p_pmap->p_locus)                /* puts node at start of list         */
      p_pmap->p_locus = p_new;
   else
   {
      p_cur = p_pmap->p_locus;          /* or adds node to end  of list       */
      while (p_cur->p_next)
         p_cur = p_cur->p_next;
      p_cur->p_next = p_new;
      p_new->p_prev = p_cur;
   }

   return p_new;
}

PRNTLOC_PTR add_trait_locus (PRNTMAP_PTR p_pmap, DIST_PTR p_dist)
{
   PRNTLOC_PTR p_new, p_cur;

   p_new = add_locus_node ();           /* makes prntmap_locus node for trait */

   p_new->no = p_dist->no;
   p_new->map_mark = p_dist->map_mark;
   p_new->locus_typ = TD;
   p_new->dist = p_dist->dist;

   if (p_dist->map_mark == 0)           /* trait precedes 1st marker          */
   {
      if (p_pmap->p_locus->locus_typ == MD
          || (p_pmap->p_locus->locus_typ == TD
              && p_pmap->p_locus->dist < p_dist->dist))
      {
         p_new->p_next = p_pmap->p_locus; /* puts trait at start of chrom    */
         p_pmap->p_locus->p_prev = p_new;
         p_pmap->p_locus = p_new;
      }
      else
      {
         p_cur = p_pmap->p_locus;
         while (p_cur->p_next
                && p_cur->p_next->locus_typ == TD
                && p_cur->p_next->dist > p_dist->dist)
            p_cur = p_cur->p_next;

         p_cur->p_next->p_prev = p_new; /* puts trait after start             */
         p_new->p_next = p_cur->p_next;
         p_new->p_prev = p_cur;
         p_cur->p_next = p_new;
      }
   }
   else
   {                                    /* trait follows 1st marker           */
      p_cur = p_pmap->p_locus;
      while (p_cur->p_next
             && (p_cur->p_next->locus_typ == TD
                 || (p_cur->p_next->locus_typ == MD
                     && p_cur->p_next->no <= p_dist->map_mark)))
         p_cur = p_cur->p_next;

      if (p_cur->p_next)
         p_cur->p_next->p_prev = p_new;
      p_new->p_next = p_cur->p_next;    /* inserts trait in list or adds      */
      p_new->p_prev = p_cur;            /* to end                             */
      p_cur->p_next = p_new;
   }

   ++p_pmap->no_traits;

   return p_new;
}

/**
 * checks marker to trait distance.
 * should not be > marker to marker distance.
 */
int test_trait_dist (PRNTMAP_PTR p_pmap, PRNTLOC_PTR p_loc)
{
   PRNTLOC_PTR p_cur;

   if (p_loc == p_pmap->p_locus)        /*checks for 1st locus               */
      return TRUE;

   else if (p_loc->locus_typ == MD)     /*checks for marker locus            */
      return TRUE;

   else if (p_loc->map_mark == 0)       /*checks for no preceding            */
      return TRUE;                      /*marker locus                       */

   else if (!p_loc->p_next)             /*checks for no following            */
      return TRUE;                      /*marker locus                       */

   else
   {
      p_cur = p_loc->p_next;

      while (p_cur->p_next && p_cur->locus_typ != MD)
         p_cur = p_cur->p_next;

      if (p_cur->locus_typ == MD        /*tests marker to trait dist         */
          && p_cur->dist < p_loc->dist)
      {
         err_typ = ERD_MTD;
         sprintf (errnumb, "%d", p_loc->no);
         err_txt = errnumb;
      }
   }
   if (err_typ)
      return FALSE;
   else
      return TRUE;
}

/**
 * finds distance between preceding locus & current locus.
 */
void fix_locus_dist (PRNTMAP_PTR p_pmap, PRNTLOC_PTR p_loc)
{
   if (p_loc == p_pmap->p_locus)        /*if first locus, dist is 0.         */
      p_loc->loc_dist = 0.;

   /*                                    * if trait precedes 1st marker,     */
   /*                                    * dist is preceding trait to marker */
   /*                                    * less trait to marker dist         */
   else if (p_loc->locus_typ == TD && p_loc->map_mark == 0)
      p_loc->loc_dist = p_loc->p_prev->dist - p_loc->dist;

   /*                                    * if marker is 1st marker,          */
   /*                                    * dist is preceding trait to marker */
   /*                                    * dist                              */
   else if (p_loc->locus_typ == MD && p_loc->selectd == 1)
      p_loc->loc_dist = p_loc->p_prev->dist;

   /*                                    * if marker or trait follows marker */
   /*                                    * dist is ok                        */
   else if (p_loc->p_prev->locus_typ == MD)
      p_loc->loc_dist = p_loc->dist;

   /*                                    * if marker follows trait,          */
   /*                                    * dist is preceding marker to       */
   /*                                    * marker dist less preceding        */
   /*                                    * marker to trait dist              */
   else if (p_loc->locus_typ == MD && p_loc->p_prev->locus_typ == TD)
      p_loc->loc_dist = p_loc->dist - p_loc->p_prev->dist;

   /*                                    * if trait follows trait,           */
   /*                                    * dist is preceding marker to trait */
   /*                                    * dist less preceding marker to     */
   /*                                    * preceding trait dist              */
   else if (p_loc->locus_typ == TD && p_loc->p_prev->locus_typ == TD)
      p_loc->loc_dist = p_loc->dist - p_loc->p_prev->dist;

   return;
}

/**
 * prints the chromosome map.
 * ea chromosome segment shows the marker and trait positions, and
 * distances between the adjacent loci (cM)
 * (a segment consists of as many loci as will fit on the page)
 */
void prnt_map (PRNTMAP_PTR p_pmap, int sex)
{
   PRNTLOC_PTR p_cur;
   int   no_loci;
   static int entry = 0;
/*                                       * first prints the heading           */

   if (p_pmap->no_mark_sel <= 1 && p_pmap->no_traits <= 0)
   {
      if (p_pmap->chrom_no != 0)
         fprintf (stdout, "\n No map for chromosome %d\n\n\n",
                  p_pmap->chrom_no);
      else
         fprintf (stdout, "\n No chromosome map\n\n\n");

      return;
   }

   if (!entry)
   {
      entry = TRUE;
      printf (" \n Inter-locus distances in cM, using Haldane map function:\n");
   }

   if (p_pmap->chrom_no != 0)
   {
      fprintf (stdout, "\n Chromosome %d", p_pmap->chrom_no);
      switch (sex)
      {
          case MAL:
             fprintf (stdout, " male map:\n");
             break;
          case FEM:
             fprintf (stdout, " female map:\n");
             break;
          default:
             fprintf (stdout, "\n");
             break;
      }
   }
   else
   {
      switch (sex)
      {
          case MAL:
             fprintf (stdout, "\n Male map:\n");
             break;
          case FEM:
             fprintf (stdout, "\n Female map:\n");
             break;
          default:
             break;
      }
   }
   fprintf (stdout, "\n");

   p_cur = p_pmap->p_locus;

   while (p_cur && p_cur->p_next)
   {                                    /* inits the next chrom segment lines */
      while (p_cur && p_cur->locus_typ != TD
             && p_cur->locus_typ == MD && !p_cur->selectd)
         p_cur = p_cur->p_next;

      if (p_cur->locus_typ == TD)
         prntmap_tr0 (p_cur->no);
      else
         prntmap_mrk0 (p_cur->no);
      sprintf (dist_line, "   ");

      p_cur = p_cur->p_next;
      no_loci = 1;

      while (p_cur && no_loci < max_line_loc)
      {                                 /* adds loci and dists to the segment */
         if (p_cur->locus_typ == TD)
            prntmap_tr (no_loci, p_cur->no);
         else
            prntmap_mrk (no_loci, p_cur->no);

         prntmap_dist (no_loci, p_cur->loc_dist);

         ++no_loci;
         p_cur = p_cur->p_next;
         while (p_cur && p_cur->locus_typ != TD
                && p_cur->locus_typ == MD && !p_cur->selectd)
            p_cur = p_cur->p_next;
      }
      prntmap_line (no_loci);           /* prints out this map segment        */

      if (p_cur)
         p_cur = p_cur->p_prev;
      while (p_cur && p_cur->locus_typ != TD
             && p_cur->locus_typ == MD && !p_cur->selectd)
         p_cur = p_cur->p_prev;
   }
   return;
}

void prntmap_mrk0 (int m)
{
   sprintf (trait_line, "    ");
   sprintf (trait_tic_marks, " -");
   sprintf (dist_line, "  ");
   sprintf (mark_tic_marks, " +");
   sprintf (mark_line, "M%d  ", m);

   return;
}
void prntmap_tr0 (int t)
{
   sprintf (trait_line, "T%d  ", t);
   sprintf (trait_tic_marks, " +");
   sprintf (dist_line, "  ");
   sprintf (mark_tic_marks, " -");
   sprintf (mark_line, "    ");

   return;
}
void prntvmap_tr0 (void)
{
   sprintf (trait_line, " T  ");
   sprintf (trait_tic_marks, " +");
   sprintf (dist_line, "  ");
   sprintf (mark_tic_marks, " -");
   sprintf (mark_line, "    ");

   return;
}
void prntmap_tr0b (void)
{
   sprintf (trait_line, "    ");
   sprintf (trait_tic_marks, " +");
   sprintf (dist_line, "  ");
   sprintf (mark_tic_marks, " -");
   sprintf (mark_line, "    ");

   return;
}
void prntmap_tr (int n_loc, int t)
{
   sprintf (trait_line + n_loc * loc_spac - 3, "   T%d  ", t);
   sprintf (trait_tic_marks + n_loc * loc_spac - 5, "------+  ");
   sprintf (mark_tic_marks + n_loc * loc_spac - 5, "-------  ");
   sprintf (mark_line + n_loc * loc_spac - 3, "       ");

   return;
}
void prntvmap_tr (int n_loc, int f)
{
   if (f)
      sprintf (trait_line + n_loc * loc_spac - 3, "    T* ");
   else
      sprintf (trait_line + n_loc * loc_spac - 3, "    T  ");
   sprintf (trait_tic_marks + n_loc * loc_spac - 5, "------+  ");
   sprintf (mark_tic_marks + n_loc * loc_spac - 5, "-------  ");
   sprintf (mark_line + n_loc * loc_spac - 3, "       ");

   return;
}
void prntmap_mrk (int n_loc, int m)
{
   sprintf (trait_line + n_loc * loc_spac - 3, "       ");
   sprintf (trait_tic_marks + n_loc * loc_spac - 5, "-------  ");
   sprintf (mark_tic_marks + n_loc * loc_spac - 5, "------+  ");
   sprintf (mark_line + n_loc * loc_spac - 3, "   M%d  ", m);

   return;
}
void prntmap_dist (int n_loc, double d)
{
   sprintf (dist_line + n_loc * loc_spac - 5, "%5.1f    ", d);

   return;
}

/**
 * prints the next segment of the chromosome map.
 * prints 3 lines.
 * first makes all lines the same length
 */
void prntmap_line (int n_loc)
{
   *(trait_line + n_loc * loc_spac - 3) = '\0';
   *(trait_tic_marks + n_loc * loc_spac - 3) = '\0';
   *(trait_tic_marks + n_loc * loc_spac - 3) = '\0';
   *(dist_line + n_loc * loc_spac - 3) = '\0';
   *(mark_tic_marks + n_loc * loc_spac - 3) = '\0';
   *(mark_line + n_loc * loc_spac - 3) = '\0';
   fprintf (stdout, "%s\n", trait_line);
   fprintf (stdout, "%s\n", trait_tic_marks);
   fprintf (stdout, "%s\n", dist_line);
   fprintf (stdout, "%s\n", mark_tic_marks);
   fprintf (stdout, "%s\n\n", mark_line);

   fflush (stdout);
   return;
}

/**
 * prints the marker frequencies for this chromosome and gender.
 */
void prnt_mark_freqs (SIM_PTR p_sim)
{
   char  prnt_lin[81] = "";
   int   cols_used = 0;
   int   tot_cols = 15;
   int   ix, flg;
   double val;
   int   m = 0;

   LOCUS_PTR cur_mark;

   if (!first_mark_locus_node)
      return;

   cur_mark = first_mark_locus_node;    /* finds the alleles for the chrom    */
   while (cur_mark->p_next && cur_mark->chrom_no < p_sim->chrom_no)
      cur_mark = cur_mark->p_next;

   if (cur_mark->chrom_no != p_sim->chrom_no)
      return;

   if (p_sim->chrom_no != 0)
      fprintf (stdout, " Marker allele frequencies for"
               " chromosome %d:\n\n", p_sim->chrom_no);
   else
      fprintf (stdout, " Marker allele frequencies:\n\n");

   while (cur_mark && cur_mark->chrom_no == p_sim->chrom_no)
   {
      if ((!p_sim->p_mark)              /* prints freqs for ea selected markr */
          || (m < p_sim->no_mark_sel
              && (p_sim->p_mark && cur_mark->no == *(p_sim->p_mark + m))))
      {
         if (cols_used > 0)
            if (cur_mark->no_alleles > tot_cols - cols_used - 2)
            {
               fprintf (stdout, "%s\n", prnt_lin);
               cols_used = 0;
            }
            else
            {
               sprintf (prnt_lin + 5 * cols_used, "     ");
               ++cols_used;
            }
         sprintf (prnt_lin + 5 * cols_used, " M%d  ", cur_mark->no);
         ++cols_used;

         ix = 0;
         flg = norm_frq (norm_frq_spec, cur_mark->p_freq_node->sum);
         while (ix < cur_mark->no_alleles)
         {
            if (cols_used >= tot_cols)
            {
               fprintf (stdout, "%s\n", prnt_lin);
               cols_used = 0;
            }
            val = *(cur_mark->p_freq_node->p_freq + ix);
            if (flg)
               val /= cur_mark->p_freq_node->sum;
            sprintf (prnt_lin + 5 * cols_used, "%5.3f", val);
            *(prnt_lin + 5 * cols_used) = ' ';
            ++ix;
            ++cols_used;
         }
         ++m;
      }
      cur_mark = cur_mark->p_next;
   }
   if (cols_used > 0)
      fprintf (stdout, "%s\n", prnt_lin);
   fprintf (stdout, "\n");
   fflush (stdout);

   return;
}

void prnt_trait_freqs (PRNTMAP_PTR p_pmap)
{
   char  prnt_lin[81] = "";
   int   i;

   PRNTLOC_PTR to_do;

   if (!p_pmap->no_traits)
      return;

   fprintf (stdout, " Trait allele frequencies:\n\n");

   to_do = p_pmap->p_locus;

   for (i = 0; i < p_pmap->no_traits; ++i)
   {                                    /* finds next trait no in map        */
      while (to_do->locus_typ == MD && to_do->p_next)
         to_do = to_do->p_next;
      fmt_trait_freqs (prnt_lin, i, to_do->no); /* formats the freqs        */

      if (i + 1 == p_pmap->no_traits || !((i + 1) % 4))
         fprintf (stdout, "%s\n", prnt_lin); /* prints the line             */
      else
         sprintf (prnt_lin + (i % 4) * 20 + 15, "     ");

      if (to_do->p_next)
         to_do = to_do->p_next;
   }
   fprintf (stdout, "\n");
   fflush (stdout);
}

void prnt_trait_pars (PRNTMAP_PTR p_pmap)
{
   PRNTLOC_PTR to_do;
   char  prnt_lin[81];
   int   i;

   if (!p_pmap->no_traits)
      return;

   fprintf (stdout, " Trait genotype means                  "
            " Additive, residual variances\n\n");

   to_do = p_pmap->p_locus;

   for (i = 0; i < p_pmap->no_traits; ++i)
   {
      while (to_do->locus_typ == MD && to_do->p_next) /* finds next trait no in map        */
         to_do = to_do->p_next;
      /*                                 * formats the means and variances   */
      fmt_trait_pars (prnt_lin, to_do->no);

      fprintf (stdout, "%s\n", prnt_lin); /* prints the line                */

      if (to_do->p_next)
         to_do = to_do->p_next;
   }
   fprintf (stdout, "\n");
   fflush (stdout);
}

void prnt_unlnkd_freqs (void)
{
   UNLNKD_PTR to_do;
   char  prnt_lin[81];
   int   i;

   if (!first_unlnkd_node)
      return;

   if (!first_unlnkd_node->p_next)
      fprintf (stdout, "\n Unlinked trait:\n\n");
   else
      fprintf (stdout, "\n Unlinked traits:\n\n");

   fprintf (stdout, " Allele frequencies:\n\n");

   to_do = first_unlnkd_node;
   i = 0;

   while (to_do)                        /* formats the freqs for the next    */
   {                                    /* unlinkd trait                     */
      fmt_trait_freqs (prnt_lin, i, to_do->no);

      if (!to_do->p_next || !((i + 1) % 4))
         fprintf (stdout, "%s\n", prnt_lin); /* prints the line             */
      else
         sprintf (prnt_lin + (i % 4) * 20 + 15, "     ");

      to_do = to_do->p_next;
      ++i;
   }
   fprintf (stdout, "\n");
   fflush (stdout);
}

void prnt_unlnkd_pars (void)
{
   UNLNKD_PTR to_do;
   char  prnt_lin[81];

   if (!first_unlnkd_node)
      return;

   fprintf (stdout, " Genotype means                        "
            " Additive, residual variances\n\n");

   to_do = first_unlnkd_node;

   while (to_do)                        /* formats the means and variances   */
   {                                    /* for the next unlinked trait       */
      fmt_trait_pars (prnt_lin, to_do->no);

      fprintf (stdout, "%s\n", prnt_lin); /* prints the line                */

      to_do = to_do->p_next;
   }
   fprintf (stdout, "\n");
   fflush (stdout);

   return;
}

void fmt_trait_freqs (char *p_lin, int n, int t_no)
{
   int   flg;
   double val;

   LOCUS_PTR cur_trait;
   /*                                    * finds locus node for this trait   */
   cur_trait = first_trait_locus_node;
   while (cur_trait->p_next && cur_trait->no < t_no)
      cur_trait = cur_trait->p_next;
   /*                                    * adds params for the trait to line */
   sprintf (p_lin + (n % 4) * 20, " T%d  ", cur_trait->no);
   flg = norm_frq (norm_frq_spec, cur_trait->p_freq_node->sum);

   val = *(cur_trait->p_freq_node->p_freq);
   if (flg)
      val /= cur_trait->p_freq_node->sum;
   sprintf (p_lin + (n % 4) * 20 + 5, "%5.3f", val);
   *(p_lin + (n % 4) * 20 + 5) = ' ';
   val = *(cur_trait->p_freq_node->p_freq + 1);
   if (flg)
      val /= cur_trait->p_freq_node->sum;
   sprintf (p_lin + (n % 4) * 20 + 10, "%5.3f", val);
   *(p_lin + (n % 4) * 20 + 10) = ' ';

   return;
}

void fmt_trait_pars (char *p_lin, int t_no)
{
   MEANS_PTR cur_means;
   VARI_PTR cur_vari;
   int   deci;
   double mini, maxi;
   /*                                    * finds means node for this trait   */
   cur_means = first_means_node;
   while (cur_means->p_next && cur_means->no < t_no)
      cur_means = cur_means->p_next;
   /*                                    * determines print format for       */
   /*                                    * parameters                        */
   mini = *(cur_means->mean);
   if (*(cur_means->mean + 1) < mini)
      mini = *(cur_means->mean + 1);
   if (*(cur_means->mean + 2) < mini)
      mini = *(cur_means->mean + 2);
   maxi = *(cur_means->mean);
   if (*(cur_means->mean + 1) > maxi)
      maxi = *(cur_means->mean + 1);
   if (*(cur_means->mean + 2) > maxi)
      maxi = *(cur_means->mean + 2);
   deci = decWid (mini, maxi, 4);
   if (deci > 7)
      deci = 7;

   sprintf (p_lin, " T%d  ", cur_means->no);
   /*adds means for the         */
   /*trait to line              */
   sprintf (p_lin + 5, " %9.*f", deci, *(cur_means->mean));
   sprintf (p_lin + 15, " %9.*f", deci, *(cur_means->mean + 1));
   sprintf (p_lin + 25, " %9.*f", deci, *(cur_means->mean + 2));
   sprintf (p_lin + 35, "     ");

   /*                                    * finds additive vari for this      */
   cur_vari = first_addi_node;          /* trait                             */
   while (cur_vari->p_next && cur_vari->no < t_no)
      cur_vari = cur_vari->p_next;      /* and adds to line                  */
   sprintf (p_lin + 40, " %9.*f", deci, cur_vari->vari);
   /*                                    * finds residual vari for this      */
   cur_vari = first_resi_node;          /* trait                             */
   while (cur_vari->p_next && cur_vari->no < t_no)
      cur_vari = cur_vari->p_next;      /* and adds to line                  */
   sprintf (p_lin + 50, " %9.*f", deci, cur_vari->vari);

   return;
}

void alloc_recomb (void)
{
   PRecombMap = malloc (NLoci * sizeof (RMAP));
   alloc_chk (WHERE, PRecombMap);

   return;
}

void mak_recomb (PRNTLOC_PTR p_ploc, int loc, int sx)
{
   RMAP_PTR p_recomb;
   PRNTLOC_PTR p_bck, p_nxt;
   int   i;
   double d, r_a, r_b, r_star;
   /*                                    * initializes recomb map for        */
   p_recomb = PRecombMap + loc;         /* markers & linked traits           */
   p_recomb->no = p_ploc->no;
   p_recomb->iA = p_recomb->iB = 0;
   p_recomb->probA[sx] = p_recomb->probB[sx] = 0.;

   if (p_ploc->locus_typ == MD)         /*       **marker**                  */
   {
      p_bck = p_ploc;
      i = loc;
      /*                                 * looks for preceding marker        */
      while ((p_bck = p_bck->p_prev) && p_bck->locus_typ != MD)
         --i;

      if (!p_bck || p_bck->locus_typ != MD)
         p_recomb->recombTyp = FM;      /* 1st marker on chromosome          */

      else
      {
         p_recomb->recombTyp = OM;      /* not 1st marker on chromosome      */
         p_recomb->iA = i - 1;
         p_recomb->probA[sx] = dist2rf (p_ploc->dist);
      }
   }
   else if (!p_ploc->p_prev)            /*         **trait**                 */
   {
      p_recomb->recombTyp = FT;         /* trait is 1st locus on chromosome  */
      p_recomb->probB[sx] = dist2rf (p_ploc->dist);

      p_nxt = p_ploc;
      i = loc;
      /*                                 * finds following marker            */
      while ((p_nxt = p_nxt->p_next) && p_nxt->locus_typ != MD)
         ++i;

      p_recomb->iB = i + 1;
   }
   else
      /*                                 * trait is preceded by marker       */
      /*                                 * or trait                          */
   {
      p_recomb->iA = loc - 1;

      p_nxt = p_ploc;
      i = loc;
      d = 0.;
      /*                                 * looks for following marker        */
      while ((p_nxt = p_nxt->p_next) && p_nxt->locus_typ != MD)
      {
         ++i;
         d += p_nxt->loc_dist;
      }
      if (p_nxt && p_nxt->locus_typ == MD)
      {                                 /* trait has following marker        */
         p_recomb->recombTyp = IT;
         p_recomb->iB = i + 1;
         /*                              * probA: prob recomb if kA==kB      */
         /*                              * probB: prob recomb with B         */
         /*                              *        if kA!=kB                  */
         d += p_nxt->loc_dist;
         r_a = dist2rf (p_ploc->loc_dist);
         r_b = dist2rf (d);
         r_star = dist2rf (p_ploc->loc_dist + d);
         p_recomb->probA[sx] = r_a * r_b / (1 - r_star);
         if (r_star != 0.)
            p_recomb->probB[sx] = (1 - r_a) * r_b / r_star;
         else
            p_recomb->probB[sx] = 0.;
      }
      else
         /*                              * trait has no following marker     */
      {
         p_recomb->recombTyp = ET;
         p_recomb->probA[sx] = dist2rf (p_ploc->loc_dist);
      }
   }
   return;
}

void mak_no_recomb (UNLNKD_PTR p_nolnk, int loc, int sx)
{
   RMAP_PTR p_recomb;
   /*                                    * initializes recomb map for        */
   /*                                    * unlinked traits                   */
   p_recomb = PRecombMap + loc;
   p_recomb->no = p_nolnk->no;
   p_recomb->recombTyp = UT;
   p_recomb->iA = p_recomb->iB = 0;
   p_recomb->probA[sx] = p_recomb->probB[sx] = 0.;

   return;
}

void alloc_trait (void)
{
   PTraitPars = malloc (NTraits * sizeof (TPARS));
   alloc_chk (WHERE, PTraitPars);

   PTraitOut = malloc (NTraits * sizeof (TOUT));
   alloc_chk (WHERE, PTraitOut);

   return;
}

void mak_trait (int no, int loc, int tno)
{
   TPARS_PTR p_trait;
   TOUT_PTR p_tout;
   MEANS_PTR p_means;
   VARI_PTR p_vari;
   TLIST_PTR p_tlist;

   double mini, maxi;

   p_trait = PTraitPars + tno;
   p_trait->no = no;

   p_means = first_means_node;
   while (p_means->p_next && p_means->no < no)
      p_means = p_means->p_next;
   p_trait->means[0] = p_means->mean[0];
   p_trait->means[1] = p_means->mean[1];
   p_trait->means[2] = p_means->mean[2];

   mini = p_trait->means[0];
   if (p_trait->means[1] < mini)
      mini = p_trait->means[1];
   if (p_trait->means[2] < mini)
      mini = p_trait->means[2];
   maxi = p_trait->means[0];
   if (p_trait->means[1] > maxi)
      maxi = p_trait->means[1];
   if (p_trait->means[2] > maxi)
      maxi = p_trait->means[2];

   p_vari = first_addi_node;
   while (p_vari->p_next && p_vari->no < no)
      p_vari = p_vari->p_next;
   p_trait->addi_var = p_vari->vari;
   p_trait->addi_term_found = sqrt (p_vari->vari);
   p_trait->addi_term = sqrt (p_vari->vari / 2.);

   p_vari = first_resi_node;
   while (p_vari->p_next && p_vari->no < no)
      p_vari = p_vari->p_next;
   p_trait->resi_var = p_vari->vari;
   p_trait->resi_term = sqrt (p_vari->vari);

   p_tlist = first_tlist_node;
   p_tout = PTraitOut;
   while (p_tlist->p_next && p_tlist->no < no)
   {
      p_tlist = p_tlist->p_next;
      ++p_tout;
   }
   p_tout->no = no;
   p_tout->dec = decWid (mini, maxi, SIGFIG);
   p_tout->wid = fldWid (mini, maxi, p_tout->dec) + 1;
   p_tout->ixG = loc;
   p_tout->ixPh = tno;

   return;
}

void mak_freq (PRNTMAP_PTR p_pmap, PRNTLOC_PTR p_ploc, int no, int loc)
{
   /*                                * stores an array of allele frequencies */
   /*                                * for the locus                         */
   int   i, flg;
   double *p_c, sum;

   LOCUS_PTR p_locus;

   if (!PAlleleFreq)
   {
      PAlleleFreq = malloc (NLoci * sizeof (ALLELE));
      alloc_chk (WHERE, PAlleleFreq);
   }
   if (p_ploc && p_ploc->locus_typ == MD)
   {
      p_locus = first_mark_locus_node;
      while ((p_locus->p_next && (p_locus->chrom_no < p_pmap->chrom_no)
              || (p_locus->chrom_no == p_pmap->chrom_no
                  && p_locus->no < p_ploc->no)))
         p_locus = p_locus->p_next;
   }
   else
   {
      p_locus = first_trait_locus_node;
      while (p_locus->p_next && p_locus->no < no)
         p_locus = p_locus->p_next;
   }

   (PAlleleFreq + loc)->nAlleles = p_locus->no_alleles;
   (PAlleleFreq + loc)->pCumFreq
      = malloc (p_locus->no_alleles * sizeof (double));
   p_c = (PAlleleFreq + loc)->pCumFreq;
   alloc_chk (WHERE, p_c);
   /*                                * cummulative frequencies are stored    */
   flg = norm_frq (norm_frq_spec, p_locus->p_freq_node->sum);
   sum = p_locus->p_freq_node->sum;

   *(p_c) = *(p_locus->p_freq_node->p_freq);
   if (flg)
      *(p_c) /= sum;
   for (i = 1; i < p_locus->no_alleles; ++i)
   {
      if (flg)
         *(p_c + i) = *(p_c + i - 1) + *(p_locus->p_freq_node->p_freq + i)
            / sum;
      else
         *(p_c + i) = *(p_c + i - 1) + *(p_locus->p_freq_node->p_freq + i);
   }

   return;
}
