/* MORGAN: Copyright 1991-2005: Univ. of Washington. All rights reserved. */
/* You may copy, use, modify, but NOT redistribute, modified or otherwise.*/
/* See: http://www.stat.washington.edu/thompson/Genepi/license.shtml      */

/*************************************************************************
  Routine to realise switches from the prior.
  There can be cases where this might not be compatible with data.
  April 1994, EAT.

  Added, January 1999; 
     routine to set segs for trait locus using a partial specification
  and
    routine to set marker segs and data conditional on those,
  and
    routine to set genotypes of observed individuals
  (EAT)

  Updated to use the linked list of switches; EAT Jan 2001.

***************************************************************************

  set_trait_segs(), next_marker_segs() and set_genotypes() not presently used
  by MORGAN;  all were used to simulate marker data conditional on trait
  segregations (MAJ).

  set_trait_segs(), next_marker_segs()  updated from MORGAN_F1 to use Switch 
  lists etc , so can be used by MORGAN 2.3, and MORGAN 2.4  (EAT: 01/2002)
 
  Modify routines to use standard sex-spec maps, and other MORGAN variables
   (EAT 02/2002)

  realise() is modified for sex-spec maps                      (MAJ 04/02)

  Updated for new nghd.g_dat structures                        EAT 10/2004

(Need to incorparorate all Saonli's updates for MORGAN 2.6.1)

  Added routine: realize_null() or single unlinked locus       EAT 10/2004

***************************************************************************/

#include "lsample.h"
#include "msample.h"

void realize_null (int loc, int ns)
{
   static int first = 1;
   int   i, ii, **xx;

   if (first)                           /* on first entry, initialize meiosis */
   {                                    /* indicators to -1                   */
      for (i = 0; i < pedsize; ++i)
      {
         xx0[loc][i] = -1;
         xx1[loc][i] = -1;
      }
      first = 0;
   }

   for (i = 1; i <= ns; ++i)            /* for each meiosis switch, get       */
   {                                    /* gender (maternal or paternal gene) */

      if (meiosis[i].sex == FEM)
         xx = xx0;
      else
         xx = xx1;

      ii = (meiosis[i].ind).index;      /* get index for this individual      */

      if (rans () < .5)                 /* randomly choose inheritance        */
         xx[loc][ii] = 0;
      else
         xx[loc][ii] = 1;
   }
}

void realise (int nl, double ***recf, int ns)
{
   static int first = 1;
   int   i, j, ii, **xx;
   double *rf;

   if (first)                           /* on first entry, initialize meiosis */
   {                                    /* indicators to -1                   */
      for (i = 0; i < pedsize; ++i)
         for (j = 0; j <= nl; ++j)
         {
            xx0[j][i] = -1;
            xx1[j][i] = -1;
         }
      first = 0;
   }

   for (i = 1; i <= ns; ++i)            /* for each meiosis switch, get       */
   {                                    /* gender (maternal or paternal gene) */
      if (meiosis[i].sex == FEM)
      {
         xx = xx0;
         rf = recf[FEM][0];
      }
      else
      {
         xx = xx1;
         rf = recf[MAL][0];
      }

      ii = (meiosis[i].ind).index;      /* get index for this individual      */

      if (rans () < .5)                 /* randomly choose inheritance for    */
         xx[0][ii] = 0;                 /* first locus                        */
      else
         xx[0][ii] = 1;

      for (j = 1; j <= nl; ++j)         /* for subsequent loci, use sex-      */
      {                                 /* specific recomb fracts             */
         if (rans () > rf[j - 1])
            xx[j][ii] = xx[j - 1][ii];
         else
            xx[j][ii] = !(xx[j - 1][ii]);
      }
   }
   return;
}

void set_trait_segs (int ns)
/* this assumes that the trait segregations to be conditioned on
   were read in as columns 5 and 6 of the original data matrix
   (pedigree data type is INHERITANCE) */
{
   int   i, ii, k, ss, **xx;

   for (i = 0; i < pedsize; ++i)
   {
      xx0[0][i] = -1;
      xx1[0][i] = -1;
   }

   for (i = 1; i <= ns; ++i)
   {
      ii = (meiosis[i].ind).index;
      ss = meiosis[i].sex;
      if (ss == 0)                      /* maternal indicator */
      {
         xx = xx0;
         k = 0;
      }
      else
      {
         xx = xx1;                      /* paternal indicator */
         k = 1;
      }
      if (PedDatTyp == INHERITANCE)
         xx[0][ii] = data[ii][5 + k];   /* segs from pedigree file */
      if (xx[0][ii] == -1)
      {
         if (rans () < 0.5)
            xx[0][ii] = 0;
         else
            xx[0][ii] = 1;
      }
   }
#if 0
   printf ("\n list of init segs");
   for (i = 0; i < pedsize; ++i)
   {
      printf ("\n %d %s;", nghd[i].index, nghd[i].self);
      printf ("  %2d %2d", xx1[0][i], xx0[0][i]);
   }
   fflush (stdout);
#endif

   return;
}

void next_marker_segs (int ns, int mf, int mt, int ir)
/* from marker mf, at recombination rec, set marker mt    */
{
   int   i, ii, ss, **xx;
   double recm, recf, rec;
   for (i = 0; i < pedsize; ++i)
   {
      xx0[mt][i] = -1;
      xx1[mt][i] = -1;
   }
   recm = xecomo[1][0][ir];
   recf = xecomo[0][0][ir];

   printf ("\n simulating from locus %d to %d at male and female recs %f %f",
           mf, mt, recm, recf);
   for (i = 1; i <= ns; ++i)
   {
      ii = (meiosis[i].ind).index;
      ss = meiosis[i].sex;
      if (ss == 0)
      {
         xx = xx0;
         rec = recf;
      }
      else
      {
         xx = xx1;
         rec = recm;
      }

      if (xx[mf][ii] != -1)
      {
         if (rans () > rec)
            xx[mt][ii] = xx[mf][ii];
         else
            xx[mt][ii] = 1 - xx[mf][ii];
      }
   }
   return;
}

void set_genotypes (int nfou, int loc, double *freq)
{
   int  *allotype, k, a1;               /* indexing of types of observed genes */
   double qf, ff;

   allotype = (int *) calloc ((size_t) (2 * nfou + 1), sizeof (int));
   for (k = 1; k <= 2 * nfou; ++k)
      allotype[k] = 0;

   for (k = 0; k < pedsize; ++k)
      if (nghd[k].g_dat)
      {
         if (allotype[gg0[loc][k]] == 0)
         {
            qf = rans ();
            a1 = 0;
            ff = 0.0;
            while ((qf > ff) && (a1 < NAll[loc]))
            {
               ++a1;
               ff += freq[a1];
            }
            allotype[gg0[loc][k]] = a1;
         }
         if (allotype[gg1[loc][k]] == 0)
         {
            qf = rans ();
            a1 = 0;
            ff = 0.0;
            while ((qf > ff) && (a1 < NAll[loc]))
            {
               ++a1;
               ff += freq[a1];
            }
            allotype[gg1[loc][k]] = a1;
         }
      }
   for (k = 0; k < pedsize; ++k)
   {
      if (nghd[k].g_dat)
      {
         if (nghd[k].g_dat[loc + loc])
            gg0[loc][k] = allotype[gg0[loc][k]];
         else
            gg0[loc][k] = 0;
         if (nghd[k].g_dat[loc + loc + 1])
            gg1[loc][k] = allotype[gg1[loc][k]];
         else
            gg1[loc][k] = 0;
      }
      else
      {
         gg0[loc][k] = 0;
         gg1[loc][k] = 0;
      }
   }
   free (allotype);
   return;
}
