/* MORGAN: Copyright 1991-2005: Univ. of Washington. All rights reserved. */
/* You may copy, use, modify, but NOT redistribute, modified or otherwise.*/
/* See: http://www.stat.washington.edu/thompson/Genepi/license.shtml      */
#include "lsample.h"

/* routines here set the quantitative trait locus information,
   both for lm_markers and for lm_schnell                      */

double set_quant_trait (int nloc)
{
   /* This routine sets the parameters of the QTL    */
   /* It replaces schl_set_traitdat () for schnell   */
   /* Here we assume there is only one trait         */
   int   i, j, m_geno;
   double mu;
   /* allele frequencies */
   double f1, f2;
   /* double pve, pva; */
   TPARS_PTR ptrait = PTraitPars;

   fflush (stdout);
   for (j = 0; j <= nloc; ++j)
      if (loci[j].locus_type == TRAIT_LOCUS)
      {
         loci[j].ve = ptrait->resi_var;
         loci[j].va = ptrait->addi_var;
         printf ("\n  Residual variance = %f, Additive Variance = %f\n",
                 loci[j].ve, loci[j].va);

         i = loci[j].n_all;
         if (i != 2)
            fprintf (stderr,
                     "Error: expect 2 alleles for the trait, got %d!\n", i);

         f1 = loci[j].freq[0];
         f2 = loci[j].freq[1];
         /* given three genotypic means, use allele frequencies to compute
          * global means, assuming there are only two alleles for the trait
          * */
         mu = f1 * f1 * ptrait->means[0] +
            2.0 * f1 * f2 * ptrait->means[1] + f2 * f2 * ptrait->means[2];
         printf ("  Trait global mean = %f\n", mu);
         printf ("  Trait Id %d: no of alleles = %d\n", j, i);
         m_geno = i * (i + 1) / 2;
         for (i = 0; i < m_geno; i++)
         {
            loci[j].g_mu[i] = ptrait->means[i];
            printf ("    Genotype %d, deviation from mean: %f \n", i,
                    loci[j].g_mu[i] - mu);
         }
      }
   fflush (stdout);
   return (mu);
}
